// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "CardBoxTreeWidget.h"
#include "Card.h"
#include "MimeData.h"
#include <QPainter>
#include <QPixmap>
#include <QIcon>
#include <QApplication>
#include <QDrag>
#include <QTimer>

CardBoxTreeWidget::CardBoxTreeWidget(QWidget *parent)
   : QWidget(parent)
{

}
void CardBoxTreeWidget::setCardBox(const CardBox *cb)
{
   cardBox = cb;
   updateCards();
}

QIcon CardBoxTreeWidget::createIcon(const QList<QColor> &colors)
{
   QPixmap pixmap(16, 16);
   pixmap.fill(Qt::transparent);
   QPainter painter(&pixmap);
   painter.setRenderHint(QPainter::Antialiasing);
   painter.setPen(Qt::NoPen);

   for (int i=0; i<colors.size(); i++)
   {
      painter.setBrush(QBrush(colors[i]));
      painter.drawPie(0, 0, 16, 16, 360/colors.size()*i*16+90*16, 16*360/colors.size());
   }
   return QIcon(pixmap);
}

QIcon CardBoxTreeWidget::createIcon(const QColor &color)
{
   QList<QColor> colors;
   colors << color;
   return createIcon(colors);
}

QStringList CardBoxTreeWidget::mimeTypes() const
{
   QStringList types;
   types << CARD_POSITION;
   return types;
}

//======================================================================

TreeWidget::TreeWidget(QWidget *parent)
   : QTreeWidget(parent)
{
   timer = new QTimer(this);
   timer -> setSingleShot(true);
   connect(timer, SIGNAL(timeout()), this, SLOT(timerSlot()));
   connect (this, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)),
            this, SLOT(currentItemChangedSlot(QTreeWidgetItem*)));
   connect (this, SIGNAL(itemSelectionChanged()), this, SLOT(itemSelectionChangedSlot()));
}

void TreeWidget::startDrag(Qt::DropActions actions)
{
   QDrag *drag = new QDrag(this);
   QMimeData *mime = mimeData(selectedItems());
   drag -> setMimeData(mime);
   drag -> setPixmap(QPixmap(":/images/drag.png"));
   drag -> exec(actions);
}

void TreeWidget::currentItemChangedSlot(QTreeWidgetItem *newItem)
{
   qDebug("TreeWidget::currentItemChangedSlot()");
   timer -> start(100);
}

void TreeWidget::itemSelectionChangedSlot()
{
   qDebug("TreeWidget::itemSelectionChangedSlot()");
   timer -> start(100);
}

void TreeWidget::timerSlot()
{
   qDebug("TreeWidget::timerSlot()");
   emit currentItemChanged();
}

QTreeWidgetItem* TreeWidget::currentItem() const
{
   return QTreeWidget::currentItem();
}
