// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDBOXTREEWIDGET_H
#define CARDBOXTREEWIDGET_H

#include <QTreeWidget>
class QTreeWidgetItem;
class CardBox;
class CardPosition;
class QColor;
class QIcon;
class MimeData;

enum ItemType {ItemType_Stage, ItemType_Stack, ItemType_Card, ItemType_None};

class CardBoxTreeWidget : public QWidget
{
   Q_OBJECT
   
   public:
      CardBoxTreeWidget(QWidget* = 0);
      virtual ~CardBoxTreeWidget(){}

      void setCardBox(const CardBox*);
      virtual QList<CardPosition> selectedPositions() const = 0;
      virtual ItemType currentCardPosition(CardPosition*) const = 0;
      virtual bool isAnyCardSelected() const = 0;
      virtual bool isAnythingSelected() const = 0;
      virtual void selectCard(const CardPosition&) = 0;
      virtual void setSelectionMode(QAbstractItemView::SelectionMode) = 0;

      virtual void dropMimeData(QTreeWidgetItem*, const MimeData*, Qt::DropAction) = 0;
      virtual QMimeData* mimeData(const QList<QTreeWidgetItem*>) const = 0;
      virtual QStringList mimeTypes() const;
      virtual void setDragDropMode(QAbstractItemView::DragDropMode) = 0;
      virtual QAbstractItemView::DragDropMode dragDropMode () const = 0;

      
   public slots:
      virtual void updateCards() = 0;
      virtual void updateCurrentCardTitle() = 0;
      
   signals:
      void cardActivated(const CardPosition&);
      void currentItemChanged();
      void dropMimeDataSignal(const CardPosition&, const MimeData*, Qt::DropAction);

   protected:
      const CardBox *cardBox;
      QIcon createIcon(const QList<QColor>&);
      QIcon createIcon(const QColor&);
};

//======================================================================

// due to a strange behaviour of QTreeWidget, it should be
// reimplemented. this strange behaviour might be a bug in Qt
// or just a normal but unwanted behaviour

class TreeWidget : public QTreeWidget
{
   Q_OBJECT

   public:
      TreeWidget(QWidget* = 0);
      virtual ~TreeWidget() {}

      QTreeWidgetItem* currentItem() const;
      

   signals:
      void currentItemChanged();

   protected:
      void startDrag(Qt::DropActions);      
      
   private slots:
      void currentItemChangedSlot(QTreeWidgetItem*);
      void itemSelectionChangedSlot();
      void timerSlot();
   
   private:
      QTimer *timer;
};


#endif

