// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include <QPainterPath>
#include "CardBoxTreeWidget_ActiveCards.h"
#include "Settings.h"
#include "SearchConfigureDialog.h"
#include "Utilities.h"
#include "MimeData.h"

ActiveCardTreeWidgetItem::ActiveCardTreeWidgetItem(QTreeWidget *parent, const QStringList &strings, int s, int cn, int ca, int ta)
   : QTreeWidgetItem(parent, strings)
{
   stage = s;
   cardNumber = cn;
   correctAnswers = ca;
   totalAnswers = ta;
   cardState = ActiveCards::CardState_Normal;
}

ActiveCardTreeWidgetItem::ActiveCardTreeWidgetItem(QTreeWidgetItem *parent, const QStringList &strings, int s, int cn, int ca, int ta)
   : QTreeWidgetItem(parent, strings)
{
   stage = s;
   cardNumber = cn;
   correctAnswers = ca;
   totalAnswers = ta;
}

int ActiveCardTreeWidgetItem::getStage() const
{
   return stage;
}

void ActiveCardTreeWidgetItem::setStage(int s)
{
   stage = s;
}

int ActiveCardTreeWidgetItem::getCardNumber() const
{
   return cardNumber;
}

void ActiveCardTreeWidgetItem::setCardNumber(int cn)
{
   cardNumber = cn;
}

int ActiveCardTreeWidgetItem::getCorrectAnswers() const
{
   return correctAnswers;
}

void ActiveCardTreeWidgetItem::setCorrectAnswers(int ca)
{
   correctAnswers = ca;
}

int ActiveCardTreeWidgetItem::getTotalAnswers() const
{
   return totalAnswers;
}

void ActiveCardTreeWidgetItem::setTotalAnswers(int ta)
{
   totalAnswers = ta;
}

int ActiveCardTreeWidgetItem::getCardState() const
{
   return cardState;
}

void ActiveCardTreeWidgetItem::setCardState(int newState)
{
   cardState = newState;
}

///====================================================

TreeWidgetForActiveCard::TreeWidgetForActiveCard(CardBoxTreeWidget *parent)
   : TreeWidget(parent),
     cardBoxTreeWidget(parent)
{
   
}

void TreeWidgetForActiveCard::drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
   QTreeWidget::drawRow(painter, option, index);
   int alpha = 255;
   if (selectedIndexes().contains(index))
      alpha = 100;

   painter -> setRenderHint(QPainter::Antialiasing);
   if (index.parent().row() == -1)
      return;
   
   ActiveCardTreeWidgetItem *item = static_cast<ActiveCardTreeWidgetItem*>(itemFromIndex(index));
   int correctAnswers = item->getCorrectAnswers();
   int totalAnswers = item->getTotalAnswers();
   double ratio = totalAnswers == 0 ? 0.0 : (static_cast<double>(correctAnswers)/totalAnswers);

   QRect outerRect(columnViewportPosition(1)+5, option.rect.top(), columnWidth(1)-10, option.rect.height());
   painter->setPen(QPen(QColor::fromRgb(160, 160, 160, alpha)));
   painter->setBrush(QBrush(QColor::fromRgb(255,255,255, alpha)));
   painter -> drawRoundedRect(outerRect, 5, 5);

   QRect innerRect;
   if (QApplication::layoutDirection() == Qt::LeftToRight)
      innerRect.setRect(outerRect.left()+1, outerRect.top()+1,
                        static_cast<int>((outerRect.width()-2)*ratio), outerRect.height()-2);
   else
      innerRect.setRect(static_cast<int>(outerRect.left()+outerRect.width()-(outerRect.width()-2)*ratio), outerRect.top()+1,
                        static_cast<int>((outerRect.width()-2)*ratio), outerRect.height()-2);

   if (innerRect.isValid())
   {
      painter->setPen(QPen(QColor::fromRgb(0,200,0, alpha)));
      painter -> drawRoundedRect(innerRect, 5, 5);
      innerRect.adjust(1,1,-1,-1);

      if (innerRect.isValid())
      {
         painter->setPen(QPen(QColor::fromRgb(100,255,100, alpha)));
         painter -> drawRoundedRect(innerRect, 5, 5);
         innerRect.adjust(1,1,-1,-1);

         if (innerRect.isValid())
         {
            QPainterPath path;
            #if QT_VERSION >= 0x040400
               path.addRoundedRect(innerRect, 5, 5);
            #else
               path.addRect(innerRect);
            #endif
            painter->fillPath(path, QColor::fromRgb(50,255,0, alpha));
         }
      }
   }
   painter->setPen(QPen(Qt::black));
   QFont myFont;
   myFont.setPointSize(myFont.pointSize()*2/3);
   painter->setFont(myFont);
   painter->drawText(outerRect, Qt::AlignHCenter | Qt::AlignVCenter,
                    QString("%1% (%2/%3)").arg(static_cast<int>(ratio*100)).arg(correctAnswers).arg(totalAnswers));
}

int TreeWidgetForActiveCard::sizeHintForColumn(int column) const
{
   QStringList headerLabels;
   headerLabels << "StageStageStageStage" << "CorrectAnswers000" << "LastAnswer000000";

   QLabel label(headerLabels[column]);
   return label.sizeHint().width();
}

void TreeWidgetForActiveCard::dropEvent(QDropEvent *event)
{
   QTreeWidgetItem *item = itemAt(event->pos());
   if (!item)
      return;
   const MimeData *mime = dynamic_cast<const MimeData*>(event->mimeData());
   if (!mime)
      return;
   cardBoxTreeWidget -> dropMimeData(item, mime, event->dropAction());
}

QMimeData* TreeWidgetForActiveCard::mimeData(const QList<QTreeWidgetItem*> items) const
{
   qDebug("TreeWidgetForActiveCard::mimeData()");
   return cardBoxTreeWidget -> mimeData(items);
}

QStringList TreeWidgetForActiveCard::mimeTypes() const
{
   return cardBoxTreeWidget -> mimeTypes();
}

Qt::DropActions TreeWidgetForActiveCard::supportedDropActions() const
{
   return Qt::MoveAction;
}

//======================================================================

CardBoxTreeWidget_ActiveCards::CardBoxTreeWidget_ActiveCards(QWidget *parent)
   : CardBoxTreeWidget(parent)
{
   searchConfigureDialog = new SearchConfigureDialog(this);

   treeWidget = new TreeWidgetForActiveCard(this);
   treeWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
   treeWidget->setDragDropMode(QAbstractItemView::DragDrop);
   treeWidget->setColumnCount(3);
   treeWidget->resizeColumnToContents(0);
   treeWidget->resizeColumnToContents(1);
   treeWidget->resizeColumnToContents(2);
   QStringList headerLabels;
   headerLabels << tr("Title") << tr("Correct Answers") << tr("   Last Answer");
   treeWidget->setHeaderLabels(headerLabels);

   newCardsAction = new QAction(tr("Mark new cards"), this);
   newCardsAction -> setCheckable(true);
   newCardsAction -> setChecked(true);
   connect(newCardsAction, SIGNAL(triggered()), this, SLOT(updateCards()));

   correctAnswersAction = new QAction(tr("Mark correct answers"), this);
   correctAnswersAction -> setCheckable(true);
   correctAnswersAction -> setChecked(true);
   connect(correctAnswersAction, SIGNAL(triggered()), this, SLOT(updateCards()));

   wrongAnswersAction = new QAction(tr("Mark wrong answers"), this);
   wrongAnswersAction -> setCheckable(true);
   wrongAnswersAction -> setChecked(true);
   connect(wrongAnswersAction, SIGNAL(triggered()), this, SLOT(updateCards()));

   extractTitlesAction = new QAction(QIcon(":/images/extract.png"), tr("Extract titles"), this);
   extractTitlesAction -> setCheckable(true);
   extractTitlesAction -> setChecked(true);
   connect(extractTitlesAction, SIGNAL(triggered()), this, SLOT(updateCards()));

   toBeAnsweredCardsAction = new QAction(tr("Mark cards to be answered"), this);
   toBeAnsweredCardsAction -> setCheckable(true);
   toBeAnsweredCardsAction -> setChecked(Settings::settings() -> reviewMarkCards);
   toBeAnsweredCardsAction -> setVisible(false);
   connect(toBeAnsweredCardsAction, SIGNAL(triggered()), this, SLOT(toBeAnsweredCardsSlot()));

   firstToolBar = new QToolBar(this);
   firstToolBar -> addAction(extractTitlesAction);
   firstToolBar -> addAction(newCardsAction);
   firstToolBar -> addAction(correctAnswersAction);
   firstToolBar -> addAction(wrongAnswersAction);
   firstToolBar -> addAction(toBeAnsweredCardsAction);
   
   searchLabel = new QLabel(tr("&Search"), this);
   searchLine = new QLineEdit(this);
   searchConfigureButton = new QToolButton(this);
   searchConfigureButton->setText(tr("..."));
   searchLabel->setBuddy(searchLine);
   QHBoxLayout *horizontalLayout = new QHBoxLayout();
   horizontalLayout->addWidget(searchLabel);
   horizontalLayout->addWidget(searchLine);
   horizontalLayout->addWidget(searchConfigureButton);
   QVBoxLayout *verticalLayout = new QVBoxLayout();
   verticalLayout->setContentsMargins(0,0,0,0);
   verticalLayout->addLayout(horizontalLayout);
   verticalLayout->addWidget(firstToolBar);
   verticalLayout->addWidget(treeWidget);
   setLayout(verticalLayout);
   
   connect (treeWidget, SIGNAL(itemActivated(QTreeWidgetItem*, int)), this, SLOT(itemActivated(QTreeWidgetItem*)));
   connect (treeWidget, SIGNAL(currentItemChanged()), this, SIGNAL(currentItemChanged()));
   connect (searchLine, SIGNAL(textChanged(const QString&)), this, SLOT(updateSearch()));
   connect (searchConfigureButton, SIGNAL(clicked()), this, SLOT(searchConfigureButtonClicked()));
}

QSize CardBoxTreeWidget_ActiveCards::sizeHint() const
{
   int width = treeWidget->columnWidth(0) + treeWidget->columnWidth(1) + 15;
   return QSize(width, size().height());
}

QList<CardPosition> CardBoxTreeWidget_ActiveCards::selectedPositions() const
{
   QList<QTreeWidgetItem*> selectedItems = treeWidget -> selectedItems();
   QList<CardPosition> selectedPositions;

   for (int i=0; i<selectedItems.size(); i++)
   {
      CardPosition position;
      cardPositionOfItem(selectedItems[i], &position);
      selectedPositions << position;
   }
   return selectedPositions;
}

ItemType CardBoxTreeWidget_ActiveCards::currentCardPosition(CardPosition *position) const
{
   return cardPositionOfItem(treeWidget->currentItem(), position);
}

ItemType CardBoxTreeWidget_ActiveCards::cardPositionOfItem(QTreeWidgetItem *item, CardPosition *position) const
{
   if (!item || !position)
      return ItemType_None;
   ActiveCardTreeWidgetItem *myItem = static_cast<ActiveCardTreeWidgetItem*>(item);
   position -> boxSection = BoxSection_ActiveCards;
   position -> stage = myItem->getStage();
   position -> cardNumber = myItem->getCardNumber();
   
   if (position -> cardNumber == -1)
      return ItemType_Stage;
   else
      return ItemType_Card;
}

bool CardBoxTreeWidget_ActiveCards::isAnyCardSelected() const
{
   QList<QTreeWidgetItem*> selectedItems = treeWidget -> selectedItems();

   for (int i=0; i<selectedItems.size(); i++)
   {
      CardPosition position;
      if (cardPositionOfItem(selectedItems[i], &position) == ItemType_Card)
         return true;
   }
   
   return false;
}

bool CardBoxTreeWidget_ActiveCards::isAnythingSelected() const
{
   return treeWidget->selectedItems().size();
}

void CardBoxTreeWidget_ActiveCards::selectCard(const CardPosition &p)
{
   CardPosition pos = p;
   for (; pos.stage >= 0 && pos.stage<treeWidget->topLevelItemCount(); pos.stage++, pos.cardNumber=0)
   {
      QTreeWidgetItem *topLevelItem = treeWidget->topLevelItem(pos.stage);
      if (pos.cardNumber == -1)
      {
         treeWidget->setCurrentItem(topLevelItem);
         break;
      }
      else if (pos.cardNumber >= 0 && pos.cardNumber < topLevelItem->childCount())
      {
         if (!topLevelItem->child(pos.cardNumber)->isHidden())
            treeWidget->setCurrentItem(topLevelItem->child(pos.cardNumber));
         if (!topLevelItem->isExpanded())
            topLevelItem->setExpanded(true);
         break;
      }
   }
}

void CardBoxTreeWidget_ActiveCards::setSelectionMode(QAbstractItemView::SelectionMode mode)
{
   treeWidget -> setSelectionMode(mode);
}

QMimeData* CardBoxTreeWidget_ActiveCards::mimeData(const QList<QTreeWidgetItem*> items) const
{
   MimeData *mime = new MimeData;
   for (int i=0; i<items.size(); i++)
   {
      CardPosition pos;
      if (cardPositionOfItem(items[i], &pos) != ItemType_None)
         mime -> addCardPosition(pos);
   }
   
   return mime;
}

void CardBoxTreeWidget_ActiveCards::dropMimeData(QTreeWidgetItem *item,
                                                 const MimeData *mime, Qt::DropAction action)
{
   CardPosition insertPos;
   cardPositionOfItem(item, &insertPos);

   emit dropMimeDataSignal(insertPos, mime, action);
}

void CardBoxTreeWidget_ActiveCards::setDragDropMode(QAbstractItemView::DragDropMode mode)
{
   treeWidget -> setDragDropMode(mode);
}

QAbstractItemView::DragDropMode CardBoxTreeWidget_ActiveCards::dragDropMode() const
{
   return treeWidget -> dragDropMode();
}

void CardBoxTreeWidget_ActiveCards::setToBeAnsweredCards(const QList<CardPosition> &tbac)
{
   toBeAnsweredCards = tbac;
}

void CardBoxTreeWidget_ActiveCards::clearToBeAnsweredCards()
{
   toBeAnsweredCards = QList<CardPosition>();
}

bool CardBoxTreeWidget_ActiveCards::selectNextToBeAnsweredCard(const CardPosition &p)
{
   CardPosition pos = p;
//   pos.cardNumber++;
   for (; pos.stage >= 0 && pos.stage<treeWidget->topLevelItemCount(); pos.stage++, pos.cardNumber=0)
   {
      QTreeWidgetItem *topLevelItem = treeWidget->topLevelItem(pos.stage);
      for (; pos.cardNumber >= 0 && pos.cardNumber < topLevelItem->childCount(); pos.cardNumber++)
      {
         ActiveCardTreeWidgetItem *item = static_cast<ActiveCardTreeWidgetItem*>(topLevelItem->child(pos.cardNumber));
         if (!item->isHidden() && (item->getCardState() & ActiveCards::CardState_ToBeAnswered))
         {
            treeWidget->setCurrentItem(topLevelItem->child(pos.cardNumber));
            return true;
         }
      }
   }
   return false;
}

bool CardBoxTreeWidget_ActiveCards::selectPreviousToBeAnsweredCard(const CardPosition &p)
{
   CardPosition pos = p;
   pos.cardNumber--;
   while (pos.stage >= 0 && pos.stage < treeWidget->topLevelItemCount())
   {
      QTreeWidgetItem *topLevelItem = treeWidget->topLevelItem(pos.stage);
      for (; pos.cardNumber >= 0 && pos.cardNumber < topLevelItem->childCount(); pos.cardNumber--)
      {
         ActiveCardTreeWidgetItem *item = static_cast<ActiveCardTreeWidgetItem*>(topLevelItem->child(pos.cardNumber));
         if (!item->isHidden() && (item->getCardState() & ActiveCards::CardState_ToBeAnswered))
         {
            treeWidget->setCurrentItem(topLevelItem->child(pos.cardNumber));
            return true;
         }
      }
      pos.stage--;
      if (pos.stage < 0 || pos.stage >= treeWidget->topLevelItemCount())
         break;
      pos.cardNumber = treeWidget->topLevelItem(pos.stage)->childCount()-1;
   }
   return false;
}

void CardBoxTreeWidget_ActiveCards::extractTitles(bool mark)
{
   extractTitlesAction -> setChecked(mark);
   updateCards();
}

void CardBoxTreeWidget_ActiveCards::showToBeAnsweredCardsAction(bool show)
{
   toBeAnsweredCardsAction -> setVisible(show);
}

void CardBoxTreeWidget_ActiveCards::updateCards()
{
   QDate currentDate = QDate::currentDate();
   QList<bool> expandedItems;
   for (int i=0; i<treeWidget->topLevelItemCount(); i++)
      expandedItems.append(treeWidget->topLevelItem(i)->isExpanded());

   treeWidget -> clear();
   for (int i = 0; i<cardBox->countStages(); i++)
   {
      QTreeWidgetItem *parent = new ActiveCardTreeWidgetItem(treeWidget, QStringList(tr("Stage %1   (%2)").arg(i+1)
                                                                               .arg(cardBox->countActiveCards(i))), i, -1);
      for (int column=0; column<treeWidget->columnCount(); column++)
         parent -> setBackground(column, QBrush(Qt::gray));
      
      for (int j=0; j<cardBox->countActiveCards(i); j++)
      {
         CardPosition cardPosition(i, j);
         const CardStatistics &cardStatistics = cardBox->at(cardPosition)->getCardStatistics();
         QString title;
         if (extractTitlesAction -> isChecked())
            title = cardBox->at(cardPosition)->getTitle();
         else
            title = tr("Card %1").arg(j+1);
         ActiveCardTreeWidgetItem *child = new ActiveCardTreeWidgetItem(parent, QStringList(title), i, j,
                                                            cardStatistics.correctAnswersCount(),
                                                            cardStatistics.totalAnswersCount());
         child -> setText(2, cardStatistics.getLastAnswer().getDate().toString("    d.M.yyyy"));

         QList<Answer> todaysAnswers = cardStatistics.getTodaysAnswers();
         QList<QColor> iconColors;
         child -> setCardState(ActiveCards::CardState_Normal);
         if (toBeAnsweredCards.contains(CardPosition(i, j)))
         {
            if (toBeAnsweredCardsAction->isChecked())
               iconColors << Settings::settings() -> generalToBeAnsweredCards;
            child -> setCardState(child->getCardState() | ActiveCards::CardState_ToBeAnswered);
         }
         if (correctAnswersAction -> isChecked()
             && todaysAnswers.contains(Answer(true, currentDate)))
         {
            iconColors << Settings::settings() -> generalCorrectlyAnsweredCards;
            child -> setCardState(child->getCardState() | ActiveCards::CardState_CorrectlyAnswered);
         }
         if (wrongAnswersAction->isChecked()
             && todaysAnswers.contains(Answer(false, currentDate)))
         {
            iconColors << Settings::settings() -> generalWronglyAnsweredCards;
            child -> setCardState(child->getCardState() | ActiveCards::CardState_WronglyAnswered);
         }
         if (newCardsAction->isChecked()
                  && cardStatistics.getDateCreated() == currentDate)
         {
            iconColors << Settings::settings() -> generalNewCards;
            child -> setCardState(child->getCardState() | ActiveCards::CardState_New);
         }
         
         if (iconColors.size())
            child -> setIcon(0, getIcon(iconColors));
      }
   }

   if (!searchLine->text().isEmpty())
      updateSearch();
   for (int i=0; i<treeWidget->topLevelItemCount(); i++)
      treeWidget->topLevelItem(i)->setExpanded(i<expandedItems.size()? expandedItems[i] : false);

   updateActions();
}

void CardBoxTreeWidget_ActiveCards::updateActions()
{
   newCardsAction -> setIcon(getIcon(Settings::settings() -> generalNewCards));
   correctAnswersAction -> setIcon(getIcon(Settings::settings() -> generalCorrectlyAnsweredCards));
   wrongAnswersAction -> setIcon(getIcon(Settings::settings() -> generalWronglyAnsweredCards));
   toBeAnsweredCardsAction -> setIcon(getIcon(Settings::settings() -> generalToBeAnsweredCards));
   toBeAnsweredCardsAction -> setChecked(Settings::settings() -> reviewMarkCards);
}

QIcon CardBoxTreeWidget_ActiveCards::getIcon(const QList<QColor> &colors)
{
   int index = iconsCache_Colors.indexOf(colors);
   if (index != -1)
      return iconsCache_Icons[index];
   else
   {
      QIcon tempIcon;
      tempIcon = createIcon(colors);
      iconsCache_Icons << tempIcon;
      iconsCache_Colors << colors;
      return tempIcon;
   }
}

QIcon CardBoxTreeWidget_ActiveCards::getIcon(const QColor &color)
{
   QList<QColor> colors;
   colors << color;
   return getIcon(colors);
}

void CardBoxTreeWidget_ActiveCards::updateCurrentCardTitle()
{
   QTreeWidgetItem *item = treeWidget->currentItem();
   CardPosition pos;
   if (cardPositionOfItem(item, &pos) == ItemType_Card)
      item -> setText(0, cardBox->at(pos)->getTitle());
}

void CardBoxTreeWidget_ActiveCards::itemActivated(QTreeWidgetItem *item)
{
   CardPosition pos;
   cardPositionOfItem(item, &pos);
   emit cardActivated(pos);
}

void CardBoxTreeWidget_ActiveCards::updateSearch()
{
   QString searchString = searchLine->text();
   if (searchString.isEmpty())
   {
      updateCards();
      return;
   }
   setAllItemsHidden(true);
   FindIn findIn = searchConfigureDialog->getFindIn();
   bool isRegExp = searchConfigureDialog->isRegExp();

   if (findIn == FindIn_Title)
   {
      for (int i=0; i<cardBox->countStages(); i++)
      {
         for (int j=0; j<cardBox->countActiveCards(i); j++)
         {
            QTreeWidgetItem *item = treeWidget->topLevelItem(i)->child(j);
            bool contains;
            if (isRegExp)
               contains = item->text(0).contains(QRegExp(searchString));
            else
               contains = item->text(0).contains(QRegExp(searchString, Qt::CaseInsensitive, QRegExp::Wildcard));
            if (contains)
               item->setHidden(false);
         }
         qApp->processEvents();
         if (searchLine->text() != searchString)
            break;
      }
      return;
   }
   
   CardPosition searchPosition(0,-1);
   while(1)
   {
      searchPosition = cardBox->search(searchString, searchPosition, isRegExp);
      if (!cardBox->isPositionValid(searchPosition))
         break;
      treeWidget->topLevelItem(searchPosition.stage)->child(searchPosition.cardNumber)->setHidden(false);
      //stay responsive:
      qApp->processEvents();
      if (searchLine->text() != searchString)
         break;
   }
}

void CardBoxTreeWidget_ActiveCards::setAllItemsHidden(bool hidden)
{
   for (int i=0; i<cardBox->countStages(); i++)
      for (int j=0; j<cardBox->countActiveCards(i); j++)
         treeWidget->topLevelItem(i)->child(j)->setHidden(hidden);
}

void CardBoxTreeWidget_ActiveCards::searchConfigureButtonClicked()
{
   searchConfigureDialog->exec();
   updateSearch();
}

void CardBoxTreeWidget_ActiveCards::toBeAnsweredCardsSlot()
{
   Settings::settings() -> reviewMarkCards = toBeAnsweredCardsAction -> isChecked();
   updateCards();
}

