// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDBOXTREEWIDGET_ACTIVECARDS_H
#define CARDBOXTREEWIDGET_ACTIVECARDS_H

#include <QTreeWidget>
#include "Card.h"
#include "CardBoxTreeWidget.h"
class QLabel;
class QLineEdit;
class QToolButton;
class QToolBar;
class SearchConfigureDialog;

namespace ActiveCards
{
   enum CardState {CardState_Normal=0,
                   CardState_CorrectlyAnswered=1,
                   CardState_WronglyAnswered=2,
                   CardState_ToBeAnswered=4,
                   CardState_New=8};
}


class ActiveCardTreeWidgetItem : public QTreeWidgetItem
{
   public:
      ActiveCardTreeWidgetItem(QTreeWidget*, const QStringList&, int, int, int=-1, int=-1);
      ActiveCardTreeWidgetItem(QTreeWidgetItem*, const QStringList&, int, int, int=-1, int=-1);
      int getStage() const;
      void setStage(int);
      int getCardNumber() const;
      void setCardNumber(int);
      int getCorrectAnswers() const;
      void setCorrectAnswers(int);
      int getTotalAnswers() const;
      void setTotalAnswers(int);
      int getCardState() const;
      void setCardState(int);
      
   
   private:
      int stage;
      int cardNumber;
      int correctAnswers;
      int totalAnswers;
      int cardState; //CardState enum
};


class TreeWidgetForActiveCard : public TreeWidget
{
   public:
      TreeWidgetForActiveCard(CardBoxTreeWidget*);

   protected:
      virtual void drawRow(QPainter*, const QStyleOptionViewItem&, const QModelIndex&) const;
      virtual int sizeHintForColumn(int) const;

      virtual QMimeData* mimeData(const QList<QTreeWidgetItem*>) const;
      virtual QStringList mimeTypes() const;
      Qt::DropActions supportedDropActions() const;
      virtual void dropEvent(QDropEvent*);
      
   private:
      CardBoxTreeWidget *cardBoxTreeWidget;
};


class CardBoxTreeWidget_ActiveCards : public CardBoxTreeWidget
{
   Q_OBJECT
   
   public:
      CardBoxTreeWidget_ActiveCards(QWidget* = 0);
      QSize sizeHint() const;

      virtual QList<CardPosition> selectedPositions() const;
      virtual ItemType currentCardPosition(CardPosition*) const;
      virtual bool isAnyCardSelected() const;
      virtual bool isAnythingSelected() const;
      //if cardNumber >= cardBox->countActiveCards(stage)
      //then selects the card in the next stage
      virtual void selectCard(const CardPosition&);
      virtual void setSelectionMode(QAbstractItemView::SelectionMode);

      virtual void dropMimeData(QTreeWidgetItem*, const MimeData*, Qt::DropAction);
      virtual QMimeData* mimeData(const QList<QTreeWidgetItem*>) const;
      virtual void setDragDropMode(QAbstractItemView::DragDropMode);
      virtual QAbstractItemView::DragDropMode dragDropMode () const;

      
      void setToBeAnsweredCards(const QList<CardPosition>&);
      void clearToBeAnsweredCards();
      bool selectNextToBeAnsweredCard(const CardPosition&);
      bool selectPreviousToBeAnsweredCard(const CardPosition&);
      void extractTitles(bool);
      void showToBeAnsweredCardsAction(bool);
      
   public slots:
      virtual void updateCards();
      virtual void updateCurrentCardTitle();
      
   private slots:
      void itemActivated(QTreeWidgetItem*);
      void updateSearch();
      void searchConfigureButtonClicked();
      void toBeAnsweredCardsSlot();
      
   private:
      QLabel *searchLabel;
      QLineEdit *searchLine;
      QToolButton *searchConfigureButton;
      TreeWidgetForActiveCard *treeWidget;
      SearchConfigureDialog *searchConfigureDialog;
      QList<CardPosition> toBeAnsweredCards;
      QToolBar *firstToolBar;
      QAction *newCardsAction;
      QAction *correctAnswersAction;
      QAction *wrongAnswersAction;
      QAction *extractTitlesAction;
      QAction *toBeAnsweredCardsAction;
      QList< QList<QColor> > iconsCache_Colors;
      QList<QIcon> iconsCache_Icons;
      
      ItemType cardPositionOfItem(QTreeWidgetItem*, CardPosition*) const;
      void setAllItemsHidden(bool);
      void updateActions();
      QIcon getIcon(const QList<QColor>&);
      QIcon getIcon(const QColor&);
};


#endif

