// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDBOXTREEWIDGET_STACK_H
#define CARDBOXTREEWIDGET_STACK_H

#include <QWidget>
#include "Card.h"
#include "CardBoxTreeWidget.h"
class QLabel;
class QLineEdit;
class QToolButton;
class SearchConfigureDialog;
class QTreeWidgetItem;
class QTreeWidget;

class TreeWidgetForStack : public TreeWidget
{
   public:
      TreeWidgetForStack(CardBoxTreeWidget*);

   protected:
      virtual QMimeData* mimeData(const QList<QTreeWidgetItem*>) const;
      virtual QStringList mimeTypes() const;
      Qt::DropActions supportedDropActions() const;
      virtual void dropEvent(QDropEvent*);
      
   private:
      CardBoxTreeWidget *cardBoxTreeWidget;
};

class CardBoxTreeWidget_Stack : public CardBoxTreeWidget
{
   Q_OBJECT

   public:
      CardBoxTreeWidget_Stack(QWidget* = 0);
      
      virtual QSize sizeHint() const;
      virtual QList<CardPosition> selectedPositions() const;
      virtual ItemType currentCardPosition(CardPosition*) const;
      virtual bool isAnyCardSelected() const;
      virtual bool isAnythingSelected() const;
      virtual void selectCard(const CardPosition&);
      virtual void setSelectionMode(QAbstractItemView::SelectionMode);

      virtual void dropMimeData(QTreeWidgetItem*, const MimeData*, Qt::DropAction);
      virtual QMimeData* mimeData(const QList<QTreeWidgetItem*>) const;
      virtual void setDragDropMode(QAbstractItemView::DragDropMode);
      virtual QAbstractItemView::DragDropMode dragDropMode () const;

   public slots:

      virtual void updateCards();
      virtual void updateCurrentCardTitle();

   private slots:
      void itemActivated(QTreeWidgetItem*);
      void updateSearch();
      void searchConfigureButtonClicked();

   private:
      QLabel *searchLabel;
      QLineEdit *searchLine;
      QToolButton *searchConfigureButton;
      TreeWidgetForStack *treeWidget;
      SearchConfigureDialog *searchConfigureDialog;

      ItemType cardPositionOfItem(QTreeWidgetItem*, CardPosition*) const;
      void setAllItemsHidden(bool);
      void updateActions();
};

#endif


