// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab.sh.70@gmail.com>                 |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab.sh.70@gmail.com>                        |
// +-------------------------------------------------------------------------+

#include "CardEditorMap.h"
#include "CardMap.h"
#include "map.h"
#include "maphelper.h"
#include "MainWindow.h"

CardEditorMap::CardEditorMap(QWidget *parent)
   : CardEditor(parent)
{
   setupUi(this);
   card = 0;
   currentMap = 0;
   splitter -> setStretchFactor(0, 10);
   splitter -> setStretchFactor(1, 1);

   for (int i=0; i<MapHelper::getListOfMaps().size(); i++)
      mapsComboBox -> addItem(MapHelper::getListOfMaps()[i]->getName());

   connect(mapsComboBox, SIGNAL(activated(int)), this, SLOT(currentMapChangedSlot(int)));
   connect(regionsComboBox, SIGNAL(activated(int)), this, SLOT(currentRegionChangedSlot(int)));
}

void CardEditorMap::editCard(Card *newCard)
{
   card = dynamic_cast<CardMap*>(newCard);
   if (!card)
   {
      qDebug("CardMap::editCard(): received card is not a CardMap");
      return;
   }
   updateEditor();
   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("Middle mouse button: move map")
                                                      + "     "
                                                      + tr("Ctrl + mouse wheel: zoom in/out"));
}

void CardEditorMap::clearEditor()
{
   _mapWidget -> clear();
   commentsTextEditor->clear();
}

Card* CardEditorMap::currentCard() const
{
   return card;
}
      
void CardEditorMap::focusInEvent(QFocusEvent *event)
{
   mapsComboBox -> setFocus();
}

void CardEditorMap::on_commentsTextEditor_editingFinished()
{
   card -> setComments(commentsTextEditor->getText());
   emit somethingEdited();
}

void CardEditorMap::currentMapChangedSlot(int index)
{
   if (index >= 0 && index < MapHelper::getListOfMaps().size())
   {
      selectMap(index);
      saveMapAndRegion();
   }
   else
      clearEditor();
}

void CardEditorMap::currentRegionChangedSlot(int index)
{
   if (index >= 0 && index < currentDivisions.size())
   {
      divisionSelected(currentDivisions[index]);
      saveMapAndRegion();
      highlightSelectedRegion(currentMap->getColor(currentDivisions[index]->getName()));
   }
}

void CardEditorMap::on__mapWidget_clicked(QRgb rgb, const QPoint &point)
{
   division *div = currentMap -> getDivision(rgb);
   if (!div)
   {
      qDebug("CardEditorMap::on__mapWidget_clicked(): User has clicked on an unknown region: r=%d, g=%d, b=%d",
             qRed(rgb), qGreen(rgb), qBlue(rgb));
      return;
   }
   if (selectDivision(div -> getName()))
   {
      highlightSelectedRegion(QColor(rgb));
      saveMapAndRegion();
   }
}

void CardEditorMap::on_capitalButton_clicked()
{
   if (capitalButton -> isChecked())
   {
//   qDebug("CardEditorMap::on_capitalButton_clicked(): capital=%s", );
      
      selectDivision(currentMap -> getCapital());
      saveMapAndRegion();

      highlightSelectedRegion(currentMap->getColor(currentMap -> getCapital()));
   }
   else
   {
      capitalButton -> setChecked(true);
      highlightSelectedRegion(currentMap->getColor(currentMap -> getCapital()));
   }
}

void CardEditorMap::updateEditor()
{
   clearEditor();

   capitalButton -> setChecked(false);
   //update map
   QString name = card -> getMap();
   if (name.isEmpty())
      name = "World";
   selectMap(name);
   
   //update region
   name = card -> getRegion();
   if (!selectDivision(name))
      selectDivision();

   if (mapsComboBox -> currentText() != card -> getMap()
       || regionsComboBox -> currentText() != card -> getRegion())
      saveMapAndRegion();

   //update hints
   commentsTextEditor -> setText(card->getComments());
}

void CardEditorMap::selectMap(const QString &name)
{
   int index = -1;
   for (int i=0; i<MapHelper::getListOfMaps().size(); i++)
      if (MapHelper::getListOfMaps()[i] -> getName() == name)
      {
         index = i;
         break;
      }

   if (index == -1)
      index = 0;

   selectMap(index);
}

void CardEditorMap::selectMap(int index)
{
   if (MapHelper::getListOfMaps().size())
   {
      currentMap = MapHelper::getListOfMaps()[index];
      mapsComboBox -> setCurrentIndex(index);
      currentDivisions = currentMap -> getAvailableDivisions();
      updateRegionsComboBox();
      selectDivision(0);
      _mapWidget -> init(currentMap -> getMapFile());

      capitalButton -> setEnabled(!currentMap->getCapital().isEmpty());
   }
}

bool CardEditorMap::selectDivision(const QString &name)
{
   for (int i=0; i<currentDivisions.size(); i++)
      if (currentDivisions[i] -> getName() == name)
         return selectDivision(i);

   return false;
}

bool CardEditorMap::selectDivision(int index)
{
   if (index >= 0 && index < currentDivisions.size())
   {
      regionsComboBox -> setCurrentIndex(index);
      divisionSelected(currentDivisions[index]);
      return true;
   }
   else
      return false;
}

void CardEditorMap::divisionSelected(division *div)
{
   capitalButton -> setChecked(div -> getName() == currentMap -> getCapital());
}

void CardEditorMap::updateRegionsComboBox()
{
   regionsComboBox -> clear();
   for (int i=0; i<currentDivisions.size(); i++)
      regionsComboBox -> addItem(currentDivisions[i]->getName());
}

void CardEditorMap::saveMapAndRegion()
{
   card -> setMap(mapsComboBox -> currentText());
   card -> setRegion(regionsComboBox -> currentText());
   emit somethingEdited();
}

void CardEditorMap::highlightSelectedRegion(const QColor &color)
{
   if (color.isValid())
      _mapWidget -> highlightRegion(color.rgb(), qRgb(100,100,255), 600);
}   
