// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include "CardEditorRegular.h"
#include "Card.h"
#include "CardRegular.h"

CardEditorRegular::CardEditorRegular(QWidget *parent)
   : CardEditor(parent)
{
   setupUi(this);
   card = 0;
   splitter -> setStretchFactor(0, 3);
   splitter -> setStretchFactor(1, 3);
   splitter -> setStretchFactor(2, 1);
}

void CardEditorRegular::editCard(Card *newCard)
{
   card = dynamic_cast<CardRegular*>(newCard);
   if (!card)
   {
      qDebug("CardEditorRegular::editCard(): received card is not a CardRegular");
      return;
   }
   updateEditor();
}

void CardEditorRegular::clearEditor()
{
   frontSideTextEditor->clear();
   backSideTextEditor->clear();
   commentsTextEditor->clear();
}

Card* CardEditorRegular::currentCard() const
{
   return card;
}

void CardEditorRegular::focusInEvent(QFocusEvent *event)
{
   CardEditor::focusInEvent(event);
   frontSideTextEditor->startEditing();
}

void CardEditorRegular::on_frontSideTextEditor_editingFinished()
{
   *card -> getFrontSideDocument() = frontSideTextEditor->getDocument();
   emit somethingEdited();
}

void CardEditorRegular::on_backSideTextEditor_editingFinished()
{
   *card -> getBackSideDocument() = backSideTextEditor->getDocument();
   emit somethingEdited();
}

void CardEditorRegular::updateEditor()
{
   clearEditor();
   frontSideTextEditor -> setDocument(*card -> getFrontSideDocument());
   backSideTextEditor -> setDocument(*card -> getBackSideDocument());
   commentsTextEditor -> setText(card->getComments());
}

void CardEditorRegular::on_frontSideTextEditor_editingFinishKeyPressed()
{
   backSideTextEditor -> startEditing();
}

void CardEditorRegular::on_backSideTextEditor_editingFinishKeyPressed()
{
   commentsTextEditor -> setFocus();
}

void CardEditorRegular::on_commentsTextEditor_editingFinished()
{
   card -> setComments(commentsTextEditor->getText());
   emit somethingEdited();
}


