// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "CardEditorVocabulary.h"
#include "Card.h"
#include "CardVocabulary.h"
#include "LanguageDialog.h"
#include "PrettyButton.h"
#include <QInputDialog>
#include <QToolBar>

CardEditorVocabulary::CardEditorVocabulary(QWidget *parent)
   : CardEditor(parent)
{
   setupUi(this);
   card = 0;

   addLanguageButton = new QToolButton(this);
   addLanguageButton -> setText(tr("Add language"));
   addLanguageButton -> setIcon(QIcon(":/images/addCard.png")); //addCard.png? confusing name... huh?
   addLanguageButton -> setShortcut(Qt::CTRL + Qt::Key_T);
   addLanguageButton -> setToolTip(addLanguageButton->text() + tr(" (") + addLanguageButton->shortcut().toString() + tr(")"));
   connect(addLanguageButton, SIGNAL(clicked()), this, SLOT(addLanguage()));

   removeLanguageButton = new QToolButton(this);
   removeLanguageButton -> setText(tr("Remove language"));
   removeLanguageButton -> setIcon(QIcon(":/images/removeCard.png")); //removeCard.png? confusing name... huh?
   removeLanguageButton -> setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_T);
   removeLanguageButton -> setToolTip(removeLanguageButton->text() + tr(" (") + removeLanguageButton->shortcut().toString() + tr(")"));
   connect(removeLanguageButton, SIGNAL(clicked()), this, SLOT(removeLanguage()));

   translationsToolBar = new QToolBar(this);
   translationsToolBar -> addWidget(addLanguageButton);
   translationsToolBar -> addWidget(removeLanguageButton);
   translationsToolBar -> addSeparator();
   removeLanguageButton -> setAutoRaise(false);
   addLanguageButton -> setAutoRaise(false);
   translationsGroupBox -> layout() -> addWidget(translationsToolBar);

   wordToolBar = new QToolBar(this);
   wordGroupBox -> layout() -> addWidget(wordToolBar);

   wordLanguageButton = new PrettyButton(Qt::Horizontal, this);
   wordLanguageButton -> setCheckable(true);
   connect(wordLanguageButton, SIGNAL(clicked()), this, SLOT(wordLanguageButtonClicked()));
   wordToolBar -> addWidget(wordLanguageButton);
   wordLanguageButton -> setChecked(true);
   wordLanguageButton -> setCursor(Qt::IBeamCursor);
   
   splitter -> setStretchFactor(0, 3);
   splitter -> setStretchFactor(1, 3);
   splitter -> setStretchFactor(2, 1);
}

void CardEditorVocabulary::editCard(Card *newCard)
{
   card = dynamic_cast<CardVocabulary*>(newCard);
   if (!card)
   {
      qDebug("CardEditorVocabulary::editCard(): received card is not a CardVocabulary");
      return;
   }
   updateEditor();
   selectLanguage(1);
}

void CardEditorVocabulary::clearEditor()
{
   wordTextEditor -> clear();
   translationTextEditor -> clear();
   nounCheckBox -> setChecked(false);
   verbCheckBox -> setChecked(false);
   adverbCheckBox -> setChecked(false);
   adjectiveCheckBox -> setChecked(false);
   otherCheckBox -> setChecked(false);
   wordTypeLineEdit -> clear();
   examplesTextEditor -> clear();
   commentsTextEditor -> clear();
   synonymsList -> clear();
   antonymsList -> clear();
}

Card* CardEditorVocabulary::currentCard() const
{
   return card;
}

void CardEditorVocabulary::focusInEvent(QFocusEvent *event)
{
   CardEditor::focusInEvent(event);
   wordTextEditor->startEditing();
}

void CardEditorVocabulary::on_wordTextEditor_editingFinished()
{
   Document *document = card -> getTranslationDocuments()[0];
   *document = wordTextEditor->getDocument();
   emit somethingEdited();
}

void CardEditorVocabulary::on_translationTextEditor_editingFinished()
{
   Document *currentDocument = getCurrentTranslationDocument();
   *currentDocument = translationTextEditor->getDocument();
   emit somethingEdited();
}

void CardEditorVocabulary::on_wordTextEditor_editingFinishKeyPressed()
{
   translationTextEditor -> startEditing();
}

void CardEditorVocabulary::on_translationTextEditor_editingFinishKeyPressed()
{
   examplesTextEditor -> setFocus();
}

void CardEditorVocabulary::on_nounCheckBox_clicked(bool checked)
{
   if (checked)
      card -> setWordType(card->getWordType() | WordType_Noun);
   else
      card -> setWordType(card->getWordType() & ~WordType_Noun);
   emit somethingEdited();
}

void CardEditorVocabulary::on_verbCheckBox_clicked(bool checked)
{
   if (checked)
      card -> setWordType(card->getWordType() | WordType_Verb);
   else
      card -> setWordType(card->getWordType() & ~WordType_Verb);
   emit somethingEdited();
}

void CardEditorVocabulary::on_adverbCheckBox_clicked(bool checked)
{
   if (checked)
      card -> setWordType(card->getWordType() | WordType_Adverb);
   else
      card -> setWordType(card->getWordType() & ~WordType_Adverb);
   emit somethingEdited();
}

void CardEditorVocabulary::on_adjectiveCheckBox_clicked(bool checked)
{
   if (checked)
      card -> setWordType(card->getWordType() | WordType_Adjective);
   else
      card -> setWordType(card->getWordType() & ~WordType_Adjective);
   emit somethingEdited();
}

void CardEditorVocabulary::on_otherCheckBox_clicked(bool checked)
{
   if (checked)
      card -> setWordType(card->getWordType() | WordType_Other);
   else
      card -> setWordType(card->getWordType() & ~WordType_Other);
   wordTypeLineEdit -> setEnabled(checked);
   emit somethingEdited();
}


void CardEditorVocabulary::on_wordTypeLineEdit_editingFinished()
{
   card -> setUserDefinedWordType(wordTypeLineEdit -> text());
   emit somethingEdited();
}

void CardEditorVocabulary::on_addSynonymButton_clicked()
{
   bool ok;
   QString newSynonym = QInputDialog::getText(this, tr("Add"), tr("Enter a synonym:"),
                                        QLineEdit::Normal, "", &ok);
   if (ok && !newSynonym.isEmpty())
   {
      QStringList synonyms = card->getSynonyms();
      synonyms.append(newSynonym);
      card -> setSynonyms(synonyms);
      updateSynonymsList();
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_removeSynonymButton_clicked()
{
   if (synonymsList -> selectedItems().size())
   {
      QStringList synonyms = card->getSynonyms();
      synonyms.removeAt(synonymsList->currentRow());
      card -> setSynonyms(synonyms);
      updateSynonymsList();
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_moveUpSynonymButton_clicked()
{
   if (synonymsList -> selectedItems().size() && synonymsList -> currentRow() != 0)
   {
      int newPosition = synonymsList -> currentRow()-1;
      QStringList synonyms = card->getSynonyms();
      synonyms.swap(synonymsList -> currentRow(), synonymsList -> currentRow()-1);
      card -> setSynonyms(synonyms);
      updateSynonymsList();
      synonymsList->setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_moveDownSynonymButton_clicked()
{
   if (synonymsList -> selectedItems().size() && synonymsList -> currentRow() != synonymsList->count()-1)
   {
      int newPosition = synonymsList -> currentRow()+1;
      QStringList synonyms = card->getSynonyms();
      synonyms.swap(synonymsList -> currentRow(), synonymsList -> currentRow()+1);
      card -> setSynonyms(synonyms);
      updateSynonymsList();
      synonymsList->setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_addAntonymButton_clicked()
{
   bool ok;
   QString newAntonym = QInputDialog::getText(this, tr("Add"), tr("Enter an antonym:"),
                                        QLineEdit::Normal, "", &ok);
   if (ok && !newAntonym.isEmpty())
   {
      QStringList antonym = card->getAntonyms();
      antonym.append(newAntonym);
      card -> setAntonyms(antonym);
      updateAntonymsList();
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_removeAntonymButton_clicked()
{
   if (antonymsList -> selectedItems().size())
   {
      QStringList antonyms = card->getAntonyms();
      antonyms.removeAt(antonymsList->currentRow());
      card -> setAntonyms(antonyms);
      updateAntonymsList();
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_moveUpAntonymButton_clicked()
{
   if (antonymsList -> selectedItems().size() && antonymsList -> currentRow() != 0)
   {
      int newPosition = antonymsList -> currentRow()-1;
      QStringList antonyms = card->getAntonyms();
      antonyms.swap(antonymsList -> currentRow(), antonymsList -> currentRow()-1);
      card -> setAntonyms(antonyms);
      updateAntonymsList();
      antonymsList->setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_moveDownAntonymButton_clicked()
{
   if (antonymsList -> selectedItems().size() && antonymsList -> currentRow() != antonymsList->count()-1)
   {
      int newPosition = antonymsList -> currentRow()+1;
      QStringList antonyms = card->getAntonyms();
      antonyms.swap(antonymsList -> currentRow(), antonymsList -> currentRow()+1);
      card -> setAntonyms(antonyms);
      updateAntonymsList();
      antonymsList->setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void CardEditorVocabulary::on_examplesTextEditor_editingFinished()
{
   card -> setExamples(examplesTextEditor->getText());
   emit somethingEdited();
}

void CardEditorVocabulary::on_commentsTextEditor_editingFinished()
{
   card -> setComments(commentsTextEditor->getText());
   emit somethingEdited();
}

void CardEditorVocabulary::updateSynonymsList()
{
   int currentRow = synonymsList -> currentRow();
   QStringList synonyms = card->getSynonyms();
   synonymsList->clear();
   for (int i=0; i<synonyms.size(); i++)
      synonymsList->addItem(synonyms[i]);
   synonymsList -> setCurrentRow(currentRow);
}

void CardEditorVocabulary::updateAntonymsList()
{
   int currentRow = antonymsList -> currentRow();
   QStringList antonyms = card->getAntonyms();
   antonymsList->clear();
   for (int i=0; i<antonyms.size(); i++)
      antonymsList->addItem(antonyms[i]);
   antonymsList -> setCurrentRow(currentRow);
}

void CardEditorVocabulary::updateEditor()
{
   updateLanguageButtons();
   clearEditor();
   wordTextEditor -> setDocument(*card -> getTranslationDocuments()[0]);
   updateTranslationEditor();

   nounCheckBox -> setChecked(card -> getWordType() & WordType_Noun);
   verbCheckBox -> setChecked(card -> getWordType() & WordType_Verb);
   adverbCheckBox -> setChecked(card -> getWordType() & WordType_Adverb);
   adjectiveCheckBox -> setChecked(card -> getWordType() & WordType_Adjective);
   otherCheckBox -> setChecked(card -> getWordType() & WordType_Other);
   wordTypeLineEdit -> setEnabled(otherCheckBox -> isChecked());

   wordTypeLineEdit -> setText(card -> getUserDefinedWordType());
   updateSynonymsList();
   updateAntonymsList();
   examplesTextEditor->setText(card->getExamples());
   commentsTextEditor->setText(card->getComments());
}

void CardEditorVocabulary::updateLanguageButtons()
{
   int initialIndex = getCurrentTranslationDocumentIndex();
   while(languageButtons.size())
      delete languageButtons.takeAt(0);

   const QList<TranslationDocument*> translationDocuments = card -> getTranslationDocuments();
   wordLanguageButton -> setText(translationDocuments[0]->getLanguageName());

   for (int i=1; i<translationDocuments.size(); i++)
   {
      PrettyButton *button = new PrettyButton(Qt::Horizontal, this);
      button -> setText(translationDocuments[i]->getLanguageName());
      button -> setCheckable(true);
      connect(button, SIGNAL(hovered()), this, SLOT(languageButtonHovered()));
      connect(button, SIGNAL(clicked()), this, SLOT(languageButtonClicked()));
      button -> setCursor(Qt::IBeamCursor);
      languageButtons << button;
      translationsToolBar -> addWidget(button);
   }

   removeLanguageButton->setEnabled(languageButtons.size() > 1);
   selectLanguage(initialIndex);
}

void CardEditorVocabulary::addLanguage()
{
   LanguageDialog dialog(this);
   if (dialog.exec())
   {
      card -> getTranslationDocuments() << new TranslationDocument(dialog.getLanguage());
      updateLanguageButtons();
      selectLanguage(card -> getTranslationDocuments().size()-1);
      emit somethingEdited();
   }
}

void CardEditorVocabulary::removeLanguage()
{
   if (card -> getTranslationDocuments().size() <= 2)
      return;
   int index = getCurrentTranslationDocumentIndex();
   if (index == card -> getTranslationDocuments().size()-1)
      selectLanguage(index-1);
   else
      selectLanguage(index+1);
   delete languageButtons.takeAt(index-1);
   delete card -> getTranslationDocuments().takeAt(index);
   updateLanguageButtons();
   emit somethingEdited();

}

void CardEditorVocabulary::languageButtonHovered()
{
   translationTextEditor -> finishEditing();
   PrettyButton *button = static_cast<PrettyButton*>(sender());
   int index = languageButtons.indexOf(button);
   if (getCurrentLanguageButtonIndex() != index
       && index != -1)
   {
      selectLanguage(index+1);
      translationTextEditor -> setFocus();
   }
}

void CardEditorVocabulary::languageButtonClicked()
{
   PrettyButton *button = static_cast<PrettyButton*>(sender());
   int index = languageButtons.indexOf(button);
   button -> setChecked(true);

   LanguageDialog dialog(this);
   dialog.setLanguage(button->text());
   if (dialog.exec())
   {
      card -> getTranslationDocuments()[index+1] -> setLanguageName(dialog.getLanguage());
      button -> setText(dialog.getLanguage());
      emit somethingEdited();
   }
}

void CardEditorVocabulary::wordLanguageButtonClicked()
{
   wordLanguageButton -> setChecked(true);
   LanguageDialog dialog(this);
   dialog.setLanguage(wordLanguageButton -> text());
   if (dialog.exec())
   {
      card -> getTranslationDocuments()[0] -> setLanguageName(dialog.getLanguage());
      wordLanguageButton -> setText(dialog.getLanguage());
      emit somethingEdited();
   }
}

void CardEditorVocabulary::updateTranslationEditor()
{
   translationTextEditor -> setDocument(*getCurrentTranslationDocument());
}

int CardEditorVocabulary::getCurrentTranslationDocumentIndex()
{
   return getCurrentLanguageButtonIndex()+1;
}

TranslationDocument* CardEditorVocabulary::getCurrentTranslationDocument()
{
   return card -> getTranslationDocuments()[getCurrentTranslationDocumentIndex()];
}   

PrettyButton* CardEditorVocabulary::getCurrentLanguageButton()
{
   return languageButtons[getCurrentLanguageButtonIndex()];
}

int CardEditorVocabulary::getCurrentLanguageButtonIndex()
{
   for (int i=0; i<languageButtons.size(); i++)
      if (languageButtons[i]->isChecked())
         return i;
   qDebug("CardEditorVocabulary::getCurrentLanguageButtonIndex() couldn't find any checked button");
   return 0;
}

void CardEditorVocabulary::selectLanguage(int index)
{
   for (int i=0; i<languageButtons.size(); i++)
      languageButtons[i]->setChecked(false);
   index--;
   if (index >= languageButtons.size())
      languageButtons[languageButtons.size()-1] -> setChecked(true);
   else
      languageButtons[index]->setChecked(true);
   updateTranslationEditor();
}

