// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDEDITORVOCABULARY_H
#define CARDEDITORVOCABULARY_H

#include "CardEditor.h"
#include "ui_CardEditorVocabulary.h"
class CardVocabulary;
class TranslationDocument;
class QToolBar;
class PrettyButton;

class CardEditorVocabulary : public CardEditor, public Ui::CardEditorVocabulary
{
   Q_OBJECT

   public:
      CardEditorVocabulary(QWidget* = 0);
      virtual void editCard(Card*);
      virtual void clearEditor();
      virtual Card* currentCard() const;
      
   protected:
      void focusInEvent(QFocusEvent*);

   private slots:
      void on_wordTextEditor_editingFinished();
      void on_translationTextEditor_editingFinished();
      void on_wordTextEditor_editingFinishKeyPressed();
      void on_translationTextEditor_editingFinishKeyPressed();

      void on_nounCheckBox_clicked(bool);
      void on_verbCheckBox_clicked(bool);
      void on_adverbCheckBox_clicked(bool);
      void on_adjectiveCheckBox_clicked(bool);
      void on_otherCheckBox_clicked(bool);
      void on_wordTypeLineEdit_editingFinished();
      void on_addSynonymButton_clicked();
      void on_removeSynonymButton_clicked();
      void on_moveUpSynonymButton_clicked();
      void on_moveDownSynonymButton_clicked();
      void on_addAntonymButton_clicked();
      void on_removeAntonymButton_clicked();
      void on_moveUpAntonymButton_clicked();
      void on_moveDownAntonymButton_clicked();
      void on_examplesTextEditor_editingFinished();
      void on_commentsTextEditor_editingFinished();

      void addLanguage();
      void removeLanguage();
      void languageButtonHovered();
      void languageButtonClicked();
      void wordLanguageButtonClicked();

   private:
      CardVocabulary *card;
      QToolBar *translationsToolBar;
      QToolBar *wordToolBar;
      QToolButton *addLanguageButton;
      QToolButton *removeLanguageButton;
      QList<PrettyButton*> languageButtons;
      PrettyButton *wordLanguageButton;

      void updateSynonymsList();
      void updateAntonymsList();
      void updateEditor();
      void updateTranslationEditor();
      void updateLanguageButtons();
      TranslationDocument* getCurrentTranslationDocument();
      int getCurrentTranslationDocumentIndex();
      PrettyButton* getCurrentLanguageButton();
      int getCurrentLanguageButtonIndex();
      void selectLanguage(int);
};



#endif

