// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include "CardManagerWidget.h"
#include "CardBoxTreeWidget_ActiveCards.h"
#include "CardBoxTreeWidget_Stack.h"
#include "CardRegular.h"
#include "CardEditorVocabulary.h"
#include "CardEditorRegular.h"
#include "CardEditorMap.h"
#include "SidePanel.h"
#include "PrettyButton.h"
#include "MimeData.h"
#include "Settings.h"
#include "CardVocabulary.h"
#include "MainWindow.h"

CardManagerWidget::CardManagerWidget(SidePanel *sp, QWidget *parent)
   : QWidget(parent)
{
   sidePanel = sp;
   lastAddAction = 0;
   widgetForSidePanel = new QWidget;
   QVBoxLayout *verticalLayout = new QVBoxLayout(widgetForSidePanel);
   QHBoxLayout *bottonsLayout = new QHBoxLayout;
   cardEditorStackedWidget = new QStackedWidget;
   cardBoxTreeWidgets = new QStackedWidget;

   cardBoxTreeWidget_ActiveCards = new CardBoxTreeWidget_ActiveCards;
   connect(cardBoxTreeWidget_ActiveCards, SIGNAL(currentItemChanged()), this, SLOT(updateButtons()));
   connect(cardBoxTreeWidget_ActiveCards, SIGNAL(currentItemChanged()), this, SLOT(updateCardEditor()));
   connect(cardBoxTreeWidget_ActiveCards, SIGNAL(cardActivated(const CardPosition&)), this, SLOT(cardActivatedSlot()));
   connect(cardBoxTreeWidget_ActiveCards, SIGNAL(dropMimeDataSignal(const CardPosition&, const MimeData*, Qt::DropAction)),
           this, SLOT(dropMimeDataSlot(const CardPosition&, const MimeData*, Qt::DropAction)));

   cardBoxTreeWidget_Stack = new CardBoxTreeWidget_Stack(cardBoxTreeWidgets);
   connect(cardBoxTreeWidget_Stack, SIGNAL(currentItemChanged()), this, SLOT(updateButtons()));
   connect(cardBoxTreeWidget_Stack, SIGNAL(currentItemChanged()), this, SLOT(updateCardEditor()));
   connect(cardBoxTreeWidget_Stack, SIGNAL(cardActivated(const CardPosition&)), this, SLOT(cardActivatedSlot()));
   connect(cardBoxTreeWidget_Stack, SIGNAL(dropMimeDataSignal(const CardPosition&, const MimeData*, Qt::DropAction)),
           this, SLOT(dropMimeDataSlot(const CardPosition&, const MimeData*, Qt::DropAction)));

   cardBoxTreeWidgets -> addWidget(cardBoxTreeWidget_ActiveCards);
   cardBoxTreeWidgets -> addWidget(cardBoxTreeWidget_Stack);
   cardBoxTreeWidgets -> setCurrentWidget(cardBoxTreeWidget_ActiveCards);

   createActionsAndWidgets();

   bottonsLayout -> addWidget(activeCardsButton);
   bottonsLayout -> addWidget(stackButton);
   
   verticalLayout -> addLayout(bottonsLayout);
   verticalLayout -> addWidget(cardBoxTreeWidgets);
   verticalLayout -> addWidget(toolBar);
   verticalLayout -> setContentsMargins(0,0,0,0);

   sidePanel -> addItem(widgetForSidePanel, tr("Card List"), QIcon(":/images/list.png"));
   sidePanel -> hideItemOfWidget(widgetForSidePanel);
   
   QGridLayout *mainLayout = new QGridLayout(this);
   mainLayout -> setContentsMargins(0,0,0,0);
   mainLayout->addWidget(cardEditorStackedWidget, 0, 0);
   
   createEditors();
   disableEditor();
}

void CardManagerWidget::createActionsAndWidgets()
{
   activeCardsButton = new PrettyButton(Qt::Horizontal, this);
   activeCardsButton -> setCheckable(true);
   activeCardsButton -> setText(tr("&Active Cards"));
   activeCardsButton -> setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   connect(activeCardsButton, SIGNAL(clicked()), this, SLOT(selectActiveCards()));
   connect(activeCardsButton, SIGNAL(hovered_dragging()), this, SLOT(selectActiveCards()));
   connect(activeCardsButton, SIGNAL(clicked()), this, SLOT(activeCardsButtonClicked()));
   connect(activeCardsButton, SIGNAL(dropMimeDataSignal(const MimeData*, Qt::DropAction)),
           this, SLOT(dropMimeDataOnActiveCardsButtonSlot(const MimeData*, Qt::DropAction)));

   stackButton = new PrettyButton(Qt::Horizontal, this);
   stackButton -> setCheckable(true);
   stackButton -> setText(tr("S&tack"));
   stackButton -> setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   connect(stackButton, SIGNAL(clicked()), this, SLOT(selectStack()));
   connect(stackButton, SIGNAL(hovered_dragging()), this, SLOT(selectStack()));
   connect(stackButton, SIGNAL(clicked()), this, SLOT(stackButtonClicked()));
   connect(stackButton, SIGNAL(dropMimeDataSignal(const MimeData*, Qt::DropAction)),
           this, SLOT(dropMimeDataOnStackButtonSlot(const MimeData*, Qt::DropAction)));
   
   addRegularCardAction = new QAction(tr("Add regular card"), this);
   addRegularCardAction->setShortcut(Qt::CTRL + Qt::Key_1);
   addRegularCardAction->setToolTip(addRegularCardAction->toolTip() + tr(" (")
                                    + addRegularCardAction->shortcut().toString() + tr(")"));
   connect(addRegularCardAction, SIGNAL(triggered()), this, SLOT(addRegularCardSlot()));

   addVocabularyCardAction = new QAction(tr("Add vocabulary card"), this);
   addVocabularyCardAction->setShortcut(Qt::CTRL + Qt::Key_2);
   addVocabularyCardAction->setToolTip(addVocabularyCardAction->toolTip() + tr(" (")
                                       + addVocabularyCardAction->shortcut().toString() + tr(")"));
   connect(addVocabularyCardAction, SIGNAL(triggered()), this, SLOT(addVocabularyCardSlot()));

   addMapCardAction = new QAction(tr("Add map card"), this);
   addMapCardAction->setShortcut(Qt::CTRL + Qt::Key_3);
   addMapCardAction->setToolTip(addMapCardAction->toolTip() + tr(" (")
                                + addMapCardAction->shortcut().toString() + tr(")"));
   connect(addMapCardAction, SIGNAL(triggered()), this, SLOT(addMapCardSlot()));

   reAddAction = new QAction(tr("Re-Add card"), this);
   reAddAction->setIcon(QIcon(":/images/addCard.png"));
   reAddAction->setShortcut(Qt::CTRL + Qt::Key_D);
   reAddAction->setToolTip(reAddAction->toolTip() + tr(" (") + reAddAction->shortcut().toString() + tr(")"));
   connect(reAddAction, SIGNAL(triggered()), this, SLOT(reAddSlot()));


   QMenu *addMenu = new QMenu;
   addMenu -> addAction(reAddAction);
   addMenu -> addSeparator();
   addMenu -> addAction(addRegularCardAction);
   addMenu -> addAction(addVocabularyCardAction);
   addMenu -> addAction(addMapCardAction);

   addCardButton = new QToolButton(this);
   addCardButton -> setIcon(QIcon(":/images/addCard.png"));
   addCardButton -> setText(tr("Add card"));
   addCardButton -> setToolTip(addCardButton->text());
   addCardButton -> setAutoRaise(true);
   addCardButton -> setPopupMode(QToolButton::MenuButtonPopup);
   addCardButton -> setMenu(addMenu);
   addCardButton -> setDefaultAction(reAddAction);
   
   removeCardAction = new QAction(QIcon(":/images/removeCard.png"), tr("Remove card(s)"), this);
   removeCardAction->setShortcut(Qt::CTRL + Qt::Key_R);
   removeCardAction->setToolTip(removeCardAction->toolTip() + tr(" (") + removeCardAction->shortcut().toString() + tr(")"));
   connect(removeCardAction, SIGNAL(triggered()), this, SLOT(removeCardSlot()));
   
   addStageAction = new QAction(QIcon(":/images/addStage.png"), tr("Add stage"), this);
   addStageAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_D);
   addStageAction->setToolTip(addStageAction->toolTip() + tr(" (") + addStageAction->shortcut().toString() + tr(")"));
   connect(addStageAction, SIGNAL(triggered()), this, SLOT(addStageSlot()));
   
   removeStageAction = new QAction(QIcon(":/images/removeStage.png"), tr("Remove stage"), this);
   removeStageAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_R);
   removeStageAction->setToolTip(removeStageAction->toolTip() + tr(" (") + removeStageAction->shortcut().toString() + tr(")"));
   connect(removeStageAction, SIGNAL(triggered()), this, SLOT(removeStageSlot()));

   shuffleAction = new QAction(QIcon(":/images/shuffle.png"), tr("Shuffle"), this);
   shuffleAction->setShortcut(Qt::CTRL + Qt::Key_H);
   shuffleAction->setToolTip(shuffleAction->toolTip() + tr(" (") + shuffleAction->shortcut().toString() + tr(")"));
   connect(shuffleAction, SIGNAL(triggered()), this, SLOT(shuffleSlot()));
   
   moveToStackAction = new QAction(QIcon(":/images/moveToStack.png"), tr("Move to stack"), this);
   connect(moveToStackAction, SIGNAL(triggered()), this, SLOT(moveToStackSlot()));
   
   moveToActiveCardsAction = new QAction(QIcon(":/images/moveToActiveCards.png"), tr("Move to active cards"), this);
   connect(moveToActiveCardsAction, SIGNAL(triggered()), this, SLOT(moveToActiveCardsSlot()));

   resetStatisticsAction = new QAction(QIcon(":/images/reset.png"), tr("Reset statistics"), this);
   connect(resetStatisticsAction, SIGNAL(triggered()), this, SLOT(resetStatisticsSlot()));
   
   toolBar = new QToolBar(tr("Main toolbar"));
   toolBar -> setIconSize(QSize(22, 22));
   toolBar -> addWidget(addCardButton);
   toolBar -> addAction(removeCardAction);
   toolBar -> addAction(addStageAction);
   toolBar -> addAction(removeStageAction);
   toolBar -> addSeparator();
   toolBar -> addAction(moveToActiveCardsAction);
   toolBar -> addAction(moveToStackAction);
   toolBar -> addAction(shuffleAction);
   toolBar -> addAction(resetStatisticsAction);
}

void CardManagerWidget::createEditors()
{
   cardEditorRegular = new CardEditorRegular;
   connect(cardEditorRegular, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(cardEditorRegular, SIGNAL(editingFinished()), this, SLOT(focusNextWidgetAfterEditing()));

   cardEditorVocabulary = new CardEditorVocabulary;
   connect(cardEditorVocabulary, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(cardEditorVocabulary, SIGNAL(editingFinished()), this, SLOT(focusNextWidgetAfterEditing()));

   cardEditorMap = new CardEditorMap;
   connect(cardEditorMap, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(cardEditorMap, SIGNAL(editingFinished()), this, SLOT(focusNextWidgetAfterEditing()));

   cardEditorStackedWidget -> addWidget(cardEditorRegular);
   cardEditorStackedWidget -> addWidget(cardEditorVocabulary);
   cardEditorStackedWidget -> addWidget(cardEditorMap);
   cardEditorStackedWidget -> setCurrentWidget(cardEditorRegular);
}

void CardManagerWidget::somethingEditedSlot()
{
   currentTreeWidget() -> updateCurrentCardTitle();
   emit somethingEdited();
}

void CardManagerWidget::cardActivatedSlot()
{
   setFocusOnEditorSlot();
}

void CardManagerWidget::manageCards(CardBox *cb)
{
   cardBox = cb;
   sidePanel -> showItemOfWidget(widgetForSidePanel);
   sidePanel -> setCurrentWidget(widgetForSidePanel);

   if (Settings::settings() -> generalAutoMoveFromStackToActiveCards)
      autoMoveCardsFromStackToActiveCards();
   
   cardBoxTreeWidget_ActiveCards -> setCardBox(cardBox);
   cardBoxTreeWidget_Stack -> setCardBox(cardBox);
   selectBoxSection(BoxSection_ActiveCards);
   updateCardEditor();

   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("You can add a card by pressing Ctrl+1 "
                                                         "(regular card), Ctrl+2 (vocabulary card) or "
                                                         "Ctrl+3 (map card)"), 0);
}

bool CardManagerWidget::askToFinish()
{
   sidePanel -> hideItemOfWidget(widgetForSidePanel);
   return true;
}

void CardManagerWidget::settingsChanged()
{
   QList<CardPosition> positions = selectedPositions();
   updateList();
   selectCard(positions.value(0));
}

void CardManagerWidget::addVocabularyCard(const QString &translation1, const QString &translation2)
{
   CardVocabulary *card = static_cast<CardVocabulary*>(addVocabularyCardSlot());
   card -> getTranslationDocuments()[0] -> setText(translation1);
   card -> getTranslationDocuments()[1] -> setText(translation2);
   somethingEditedSlot();
}

void CardManagerWidget::updateList()
{
   CardBoxTreeWidget *widget;
   if (activeCardsButton -> isChecked())
      widget = cardBoxTreeWidget_ActiveCards;
   else
      widget = cardBoxTreeWidget_Stack;
   cardBoxTreeWidgets -> setCurrentWidget(widget);
   widget -> setFocus();
   widget -> updateCards();
}

void CardManagerWidget::updateButtons()
{
//   bool anyCardSelected = currentTreeWidget() -> isAnyCardSelected();
   bool anythingSelected = currentTreeWidget() -> isAnythingSelected();
   removeStageAction -> setEnabled(anythingSelected && currentBoxSection() == BoxSection_ActiveCards);
   removeCardAction -> setEnabled(anythingSelected);
   shuffleAction -> setEnabled(anythingSelected);
   moveToActiveCardsAction -> setEnabled(currentBoxSection() == BoxSection_Stack && anythingSelected);
   moveToStackAction -> setEnabled(currentBoxSection() == BoxSection_ActiveCards && anythingSelected);
   resetStatisticsAction -> setEnabled(anythingSelected);
}

void CardManagerWidget::updateListAndButtons()
{
   updateList();
   updateButtons();
}

Card* CardManagerWidget::addRegularCardSlot()
{
   Card *card = addCard(CardType_Regular);
   cardAdded();
   lastAddAction = addRegularCardAction;
   return card;
}

Card* CardManagerWidget::addVocabularyCardSlot()
{
   Card *card = addCard(CardType_Vocabulary);
   cardAdded();
   lastAddAction = addVocabularyCardAction;
   return card;
}

Card* CardManagerWidget::addMapCardSlot()
{
   Card *card = addCard(CardType_Map);
   cardAdded();
   lastAddAction = addMapCardAction;
   return card;
}

Card* CardManagerWidget::addCard(CardType type)
{
   if (Settings::settings() -> generalAddNewCardsTo == BoxSection_ActiveCards)
      return cardBox -> addCardToActiveCards(type, 0, BeginningOrEnd_Beginning);
   else
      return cardBox -> addCardToStack(type, BeginningOrEnd_Beginning);
}


void CardManagerWidget::reAddSlot()
{
   if (lastAddAction)
      lastAddAction -> trigger();
   else
      addRegularCardAction -> trigger();
}

void CardManagerWidget::cardAdded()
{
   updateListAndButtons();
   if (Settings::settings() -> generalAddNewCardsTo == BoxSection_ActiveCards)
      selectCard(CardPosition(0,0));
   else
      selectCard(CardPosition(0));
   emit somethingEdited();
   QTimer::singleShot(100, this, SLOT(setFocusOnEditorSlot()));
//   cardEditorStackedWidget->currentWidget()->setFocus();
}

void CardManagerWidget::removeCardSlot()
{
   QList<CardPosition> positions = selectedPositions();
   if (!positions.size())
      return;
   adjustCardPositions(positions);
   
   if (positions.size() > 1)
   {
      int answer = QMessageBox::warning(this, tr("Delete"),
                                        tr("%1 cards are selected.\nAre you sure you want to delete them?").arg(positions.size()),
                                        QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
      if (answer == QMessageBox::No)
         return;
   }

   disableEditor();
   for (int i=positions.size()-1; i >= 0; i--)
      cardBox -> removeCard(positions[i]);
   
   updateListAndButtons();
   selectCard(positions.value(0));
   emit somethingEdited();
}

void CardManagerWidget::addStageSlot()
{
   cardBox->addStage();
   selectBoxSection(BoxSection_ActiveCards);
   updateListAndButtons();
   emit somethingEdited();
}

void CardManagerWidget::removeStageSlot()
{
   if (currentBoxSection() != BoxSection_ActiveCards)
      return;

   QList<CardPosition> positions = selectedPositions();
   QList<bool> stages;
   for (int i=0; i<cardBox->countStages(); i++)
      stages << false;

   for (int i=0; i<positions.size(); i++)
      stages[positions[i].stage] = true;

   if (!stages.count(true))
   {
      QMessageBox::warning(this, tr("Delete"),
                           tr("No stage is selected!"
                              "\nPlease select the stage(s) you want to remove, then try again."));
      return;
   }

   int answer = QMessageBox::warning(this, tr("Delete"),
                                     tr("The selected stage(s) may contain some cards."
                                        "\nAre you sure you want to delete the stage(s) and the cards?"),
                                     QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
   if (answer == QMessageBox::No)
      return;

   
   disableEditor();
   for (int i=stages.size()-1; i>=0; i--)
      if (stages[i])
         cardBox->removeStage(i);

   updateListAndButtons();
   selectCard(positions.value(0));
   emit somethingEdited();
}

void CardManagerWidget::activeCardsButtonClicked()
{
   //think of a more reasonable way!
   activeCardsButton -> setChecked(stackButton -> isChecked() ? false : true);
   selectActiveCards();
}

void CardManagerWidget::stackButtonClicked()
{
   //think of a more reasonable way!
   stackButton -> setChecked(activeCardsButton -> isChecked() ? false : true);
   selectStack();
}

void CardManagerWidget::selectActiveCards()
{
   if (currentBoxSection() != BoxSection_ActiveCards)
   {
      disableEditor();
      activeCardsButton -> setChecked(true);
      stackButton -> setChecked(false);
      updateListAndButtons();
   }
}

void CardManagerWidget::selectStack()
{
   if (currentBoxSection() != BoxSection_Stack)
   {
      disableEditor();
      stackButton -> setChecked(true);
      activeCardsButton -> setChecked(false);
      updateListAndButtons();
   }
}

void CardManagerWidget::updateCardEditor()
{
   CardPosition pos;
   ItemType type = currentCardPosition(&pos);

   if (type != ItemType_Card)
      disableEditor();
   else
   {
      Card *card = cardBox->at(pos);
      CardEditor *editor = dynamic_cast<CardEditor*>(cardEditorStackedWidget -> currentWidget());
      if (!card)
      {
         disableEditor();
         return;
      }
      else if (card -> getType() == CardType_Regular)
         editor = cardEditorRegular;
      else if(card -> getType() == CardType_Vocabulary)
         editor = cardEditorVocabulary;
      else if(card -> getType() == CardType_Map)
         editor = cardEditorMap;
      else // should not happen
      {
         disableEditor();
         return;
      }

      cardEditorStackedWidget -> setEnabled(true);
      cardEditorStackedWidget -> setCurrentWidget(editor);
      editor -> editCard(card);
   }
}

void CardManagerWidget::disableEditor()
{
   if (cardEditorStackedWidget -> isEnabled())
   {
      CardEditor *editor = static_cast<CardEditor*>(cardEditorStackedWidget -> currentWidget());
      cardEditorStackedWidget -> setEnabled(false);
      editor -> clearEditor();
   }
}

void CardManagerWidget::focusNextWidgetAfterEditing()
{
   addCardButton -> setFocus();
}

void CardManagerWidget::shuffleSlot()
{
   CardPosition selectPos;
   
   disableEditor();
   if (currentBoxSection() == BoxSection_ActiveCards)
   {
      QList<bool> stages;
      for (int i=0; i<cardBox->countStages(); i++)
         stages << false;

      QList<CardPosition> positions = selectedPositions();
      for (int i=0; i<positions.size(); i++)
         stages[positions[i].stage] = true;

      for (int i=0; i<stages.size(); i++)
            cardBox->shuffleActiveCards(i);
      selectPos = positions.value(0);
   }
   else
   {
      cardBox->shuffleStack();
      selectPos = CardPosition(0);
   }
   
   updateListAndButtons();
   selectCard(selectPos);
   emit somethingEdited();
}

void CardManagerWidget::moveToStackSlot()
{
   if (currentBoxSection() != BoxSection_ActiveCards)
      return;

   QList<CardPosition> positions = selectedPositions();
   disableEditor();
   adjustCardPositions(positions);
   for (int i=positions.size()-1; i >= 0; i--)
      cardBox -> moveToStack(positions[i], BeginningOrEnd_Beginning);
   
   updateListAndButtons();
   selectCard(positions.value(0));
   emit somethingEdited();
}

void CardManagerWidget::moveToActiveCardsSlot()
{
   if (currentBoxSection() != BoxSection_Stack)
      return;

   QList<CardPosition> positions = selectedPositions();
   disableEditor();
   adjustCardPositions(positions);
   for (int i=positions.size()-1; i >= 0; i--)
      cardBox -> moveToActiveCards(positions[i], 0, BeginningOrEnd_Beginning);
   
   updateListAndButtons();
   selectCard(positions.value(0));
   emit somethingEdited();
}

void CardManagerWidget::resetStatisticsSlot()
{
   QList<CardPosition> positions = selectedPositions();
   adjustCardPositions(positions);

   if (positions.size() > 1)
   {
      int answer = QMessageBox::warning(this, tr("Reset Statistics"),
                                        tr("%1 cards are selected.\nAre you sure you want to reset statistics?").arg(positions.size()),
                                        QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
      if (answer == QMessageBox::No)
         return;
   }
   disableEditor();
   for (int i=positions.size()-1; i >= 0; i--)
      cardBox -> at(positions[i]) -> clearAnswers();
   
   updateListAndButtons();
   selectCard(positions.value(0));
   emit somethingEdited();
}

void CardManagerWidget::dropMimeDataSlot(const CardPosition &pos, const MimeData *mime, Qt::DropAction action)
{
   //Do not delete it
   qDebug("CardManagerWidget::dropMimeDataSlot() started");

   disableEditor();

   Card *target = cardBox -> at(pos);
   QList<CardPosition> positions = mime -> getCardPositions();
   adjustCardPositions(positions);

   QList<Card*> cards;
   for (int i=positions.size()-1; i >= 0; i--)
      cards << cardBox -> takeAt(positions[i]);

   CardPosition targetPosition;
   if (pos.cardNumber == -1)
      targetPosition = CardPosition(pos.boxSection, pos.stage, 0);
   else
   {
      targetPosition = cardBox -> positionOfCard(target);
      targetPosition.cardNumber++;
   }

   for (int i=0; i<cards.size(); i++)
      cardBox -> insertCardAt(cards[i], targetPosition);

   updateListAndButtons();
   selectCard(targetPosition);
   emit somethingEdited();
   qDebug("CardManagerWidget::dropMimeDataSlot() finished");
}

void CardManagerWidget::setFocusOnEditorSlot()
{
   cardEditorStackedWidget->currentWidget()->setFocus();
}

void CardManagerWidget::dropMimeDataOnActiveCardsButtonSlot(const MimeData *mime, Qt::DropAction action)
{
   dropMimeDataSlot(CardPosition(0, -1), mime, action);
}

void CardManagerWidget::dropMimeDataOnStackButtonSlot(const MimeData *mime, Qt::DropAction action)
{
   dropMimeDataSlot(CardPosition(-1), mime, action);
}

void CardManagerWidget::selectCard(const CardPosition &pos)
{
   if (currentBoxSection() != pos.boxSection)
      selectBoxSection(pos.boxSection);
   currentTreeWidget() -> selectCard(pos);
}

void CardManagerWidget::selectBoxSection(BoxSection boxSection)
{
   if (boxSection == BoxSection_ActiveCards)
      selectActiveCards();
   else
      selectStack();
}

ItemType CardManagerWidget::currentCardPosition(CardPosition *pos) const
{
   return currentTreeWidget() -> currentCardPosition(pos);
}

QList<CardPosition> CardManagerWidget::selectedPositions() const
{
   return currentTreeWidget() -> selectedPositions();
}

BoxSection CardManagerWidget::currentBoxSection() const
{
   if (activeCardsButton -> isChecked())
      return BoxSection_ActiveCards;
   else
      return BoxSection_Stack;
}

CardBoxTreeWidget* CardManagerWidget::currentTreeWidget()
{
   return static_cast<CardBoxTreeWidget*>(cardBoxTreeWidgets -> currentWidget());
}

const CardBoxTreeWidget* CardManagerWidget::currentTreeWidget() const
{
   return static_cast<const CardBoxTreeWidget*>(cardBoxTreeWidgets -> currentWidget());
}

void CardManagerWidget::adjustCardPositions(QList<CardPosition> &positions)
{
   bool wholeStack = false;
   QList<int> wholeStage;
   for (int i=0; i<positions.size(); i++)
   {
      CardPosition curPos = positions[i];
      if (curPos.boxSection == BoxSection_Stack && curPos.cardNumber==-1)
         wholeStack = true;
      else if (curPos.boxSection == BoxSection_ActiveCards && curPos.cardNumber==-1)
         wholeStage << curPos.stage;
   }

   for (int i = 0; i < positions.size(); i++)
   {
      CardPosition curPos = positions[i];

      if ((wholeStack && curPos.boxSection == BoxSection_Stack)
          || (wholeStage.contains(curPos.stage) && curPos.boxSection == BoxSection_ActiveCards))
         positions.removeAt(i--);
   }
   if (wholeStack)
      for (int i=0; i<cardBox->countStack(); i++)
         positions << CardPosition(i);
   for (int i=0; i<wholeStage.size(); i++)
      for (int j=0; j<cardBox->countActiveCards(wholeStage[i]); j++)
         positions << CardPosition(wholeStage[i], j);

   qStableSort(positions);
}

void CardManagerWidget::autoMoveCardsFromStackToActiveCards()
{
   while(cardBox -> countActiveCards(0) < Settings::settings() -> generalCardsInFirstStage
         && cardBox -> countStack())
      cardBox -> moveToActiveCards(CardPosition(0), 0, BeginningOrEnd_End);
}


