// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDMANAGER_H
#define CARDMANAGER_H

#include <QWidget>
#include "CardBoxTreeWidget.h"
#include "Card.h"
class Card;
class CardBox;
class CardEditor;
class CardBoxTreeWidget_ActiveCards;
class CardBoxTreeWidget_Stack;
class QStackedWidget;
class QToolBar;
class QToolButton;
class SidePanel;
class PrettyButton;

class CardManagerWidget : public QWidget
{
   Q_OBJECT
   
   public:
      CardManagerWidget(SidePanel *sidePanel, QWidget* = 0);
      void manageCards(CardBox*);
      bool askToFinish();
      void settingsChanged();
      void addVocabularyCard(const QString&, const QString&);
      
   signals:
      void somethingEdited();
      
   private slots:
      void updateList();
      void updateButtons();
      void updateListAndButtons();
      Card* addRegularCardSlot();
      Card* addVocabularyCardSlot();
      Card* addMapCardSlot();
      void reAddSlot();
      void removeCardSlot();
//      void moveUpSlot();
//      void moveDownSlot();
      void shuffleSlot();
      void updateCardEditor();
      void focusNextWidgetAfterEditing();
      void somethingEditedSlot();
      void cardActivatedSlot();
      void addStageSlot();
      void removeStageSlot();
      void activeCardsButtonClicked();
      void stackButtonClicked();
      void selectActiveCards();
      void selectStack();
      void moveToStackSlot();
      void moveToActiveCardsSlot();
      void resetStatisticsSlot();
      void dropMimeDataSlot(const CardPosition&, const MimeData*, Qt::DropAction);
      void setFocusOnEditorSlot();
      void dropMimeDataOnActiveCardsButtonSlot(const MimeData*, Qt::DropAction);
      void dropMimeDataOnStackButtonSlot(const MimeData*, Qt::DropAction);
   
   private:
      CardBox *cardBox;
      CardEditor *cardEditorRegular;
      CardEditor *cardEditorVocabulary;
      CardEditor *cardEditorMap;
      SidePanel *sidePanel;
      QWidget *widgetForSidePanel;

      QStackedWidget *cardEditorStackedWidget;
      QStackedWidget *cardBoxTreeWidgets;
      CardBoxTreeWidget_ActiveCards *cardBoxTreeWidget_ActiveCards;
      CardBoxTreeWidget_Stack *cardBoxTreeWidget_Stack;
      
      QToolBar *toolBar;

      PrettyButton *activeCardsButton;
      PrettyButton *stackButton;
      
      QToolButton *addCardButton;
      QAction *addRegularCardAction;
      QAction *addVocabularyCardAction;
      QAction *addMapCardAction;
      QAction *removeCardAction;
      QAction *addStageAction;
      QAction *removeStageAction;
//      QAction *moveDownAction;
//      QAction *moveUpAction;
      QAction *shuffleAction;
      QAction *reAddAction;
      QAction *lastAddAction;
      QAction *moveToStackAction;
      QAction *moveToActiveCardsAction;
      QAction *resetStatisticsAction;

      void createActionsAndWidgets();
      void createEditors();
      void disableEditor();
      Card* addCard(CardType);
      void cardAdded();
      void selectCard(const CardPosition&);
      void selectBoxSection(BoxSection);
      ItemType currentCardPosition(CardPosition*) const;
      QList<CardPosition> selectedPositions() const;
      BoxSection currentBoxSection() const;
      CardBoxTreeWidget* currentTreeWidget();
      const CardBoxTreeWidget* currentTreeWidget() const;
      //if a position was equal to stage/stack, all of the cards inside
      //that stage/stack will be inserted into the list and the initial
      //position will be deleted
      //it will sort the positions at the end
      void adjustCardPositions(QList<CardPosition>&);
      void autoMoveCardsFromStackToActiveCards();
};

#endif
