// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include <cstdlib>
#include <ctime>
#include "Settings.h"
#include "CardReviewerWidget.h"
#include "CardBoxTreeWidget_ActiveCards.h"
#include "CardReviewerRegular.h"
#include "CardReviewerVocabulary.h"
#include "CardReviewerMap.h"
#include "SidePanel.h"
#include "ReportDialog.h"
#include "Utilities.h"
#include "MainWindow.h"

CardReviewerWidget::CardReviewerWidget(SidePanel *sp, QWidget *parent)
   : QWidget(parent)
{
   sidePanel = sp;
   isVerifying = false;
   createActionsAndWidgets();
   createLayouts();
   createReviewers();
   updateReviewerAndActions();
}

void CardReviewerWidget::createActionsAndWidgets()
{
   reportDialog = new ReportDialog(this);
   cardReviewerStackedWidget = new QStackedWidget(this);
   cardBoxTreeWidget_ActiveCards = new CardBoxTreeWidget_ActiveCards;
   cardBoxTreeWidget_ActiveCards -> setSelectionMode(QAbstractItemView::SingleSelection);
   cardBoxTreeWidget_ActiveCards -> setDragDropMode(QAbstractItemView::NoDragDrop);
   cardBoxTreeWidget_ActiveCards -> showToBeAnsweredCardsAction(true);
   sidePanel -> addItem(cardBoxTreeWidget_ActiveCards, tr("Cards List"), QIcon(":/images/list.png"));
   sidePanel -> hideItemOfWidget(cardBoxTreeWidget_ActiveCards);
   connect(cardBoxTreeWidget_ActiveCards, SIGNAL(currentItemChanged()), this, SLOT(updateReviewerAndActions()));
   progressBar = new QProgressBar(this);
   answersStatisticsLabel = new QLabel(this);
   updateAnswersStatisticsLabel();

   verifyAction = new QAction(QIcon(":/images/verify.png"), tr("Verify"), this);
   verifyAction -> setShortcut(Qt::Key_Space);
   verifyAction->setToolTip(verifyAction->toolTip() + tr(" (") + verifyAction->shortcut().toString() + tr(")"));
   connect(verifyAction, SIGNAL(triggered()), this, SLOT(verifySlot()));

   yesAction = new QAction(QIcon(":/images/yes.png"), tr("Yes"), this);
   yesAction -> setShortcut(Qt::Key_1);
   yesAction->setToolTip(yesAction->toolTip() + tr(" (") + yesAction->shortcut().toString() + tr(")"));
   connect(yesAction, SIGNAL(triggered()), this, SLOT(yesSlot()));

   noAction = new QAction(QIcon(":/images/no.png"), tr("No"), this);
   noAction -> setShortcut(Qt::Key_2);
   noAction->setToolTip(noAction->toolTip() + tr(" (") + noAction->shortcut().toString() + tr(")"));
   connect(noAction, SIGNAL(triggered()), this, SLOT(noSlot()));

   skipAction = new QAction(QApplication::layoutDirection() == Qt::LeftToRight
                            ? QIcon(":/images/skip.png") : QIcon(":/images/skipRTL.png"), tr("Skip"), this);
   skipAction -> setShortcut(Qt::Key_3);
   skipAction->setToolTip(skipAction->toolTip() + tr(" (") + skipAction->shortcut().toString() + tr(")"));
   connect(skipAction, SIGNAL(triggered()), this, SLOT(skipSlot()));

   playStopAction = new QAction(tr("Play/Stop"), this);
   if (QApplication::layoutDirection() == Qt::RightToLeft)
      playStopAction->setIcon(QIcon(":/images/playRTL.png"));
   else
      playStopAction -> setIcon(QIcon(":/images/play.png"));
   playStopAction -> setShortcut(Qt::CTRL + Qt::Key_P);
   playStopAction -> setToolTip(playStopAction->toolTip() + tr(" (") + playStopAction->shortcut().toString() + tr(")"));
   connect(playStopAction, SIGNAL(triggered()), this, SLOT(playStopSlot()));

   autoPlayAction = new QAction(QIcon(":/images/autoPlay.png"), tr("Auto play"), this);
   autoPlayAction -> setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_P);
   autoPlayAction -> setCheckable(true);
   autoPlayAction -> setChecked(Settings::settings()->reviewAutoPlay);
   connect(autoPlayAction, SIGNAL(toggled(bool)), this, SLOT(autoPlaySlot(bool)));

   showHintsAction = new QAction(QIcon(":/images/hint.png"), tr("Show Hints"), this);
   showHintsAction -> setCheckable(true);
   showHintsAction -> setShortcut(Qt::CTRL + Qt::Key_H);
   connect(showHintsAction, SIGNAL(triggered()), this, SLOT(showHintsSlot()));
   
   showHintsOnVerifyAction = new QAction(tr("Show Hints only on Verify"), this);
   showHintsOnVerifyAction -> setCheckable(true);
   showHintsOnVerifyAction -> setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_H);
   connect(showHintsOnVerifyAction, SIGNAL(triggered()), this, SLOT(showHintsOnVerifySlot()));

   hintsMenu = new QMenu(this);
   hintsMenu -> addAction(showHintsAction);
   hintsMenu -> addAction(showHintsOnVerifyAction);

   hintsButton = new QToolButton(this);
   hintsButton -> setPopupMode(QToolButton::MenuButtonPopup);
   hintsButton -> setMenu(hintsMenu);
   hintsButton -> setDefaultAction(showHintsAction);

   firstToolBar = new QToolBar(this);
   firstToolBar -> addAction(playStopAction);
   firstToolBar -> addAction(autoPlayAction);
   firstToolBar -> addWidget(hintsButton);

   secondToolBar = new QToolBar(this);
   secondToolBar -> setIconSize(QSize(32, 32));
   secondToolBar -> addAction(verifyAction);
   secondToolBar -> addSeparator();
   secondToolBar -> addAction(yesAction);
   secondToolBar -> addAction(noAction);
   secondToolBar -> addAction(skipAction);
   secondToolBar -> addSeparator();
   secondToolBar -> addWidget(answersStatisticsLabel);
   secondToolBar -> addWidget(progressBar);
}

void CardReviewerWidget::createLayouts()
{
  QVBoxLayout *mainLayout = new QVBoxLayout(this);
  mainLayout -> addWidget(firstToolBar);
  mainLayout -> addWidget(cardReviewerStackedWidget);
  mainLayout -> addWidget(new QLabel(tr("Did you answer correctly?"), this));
  mainLayout -> addWidget(secondToolBar);
  mainLayout -> setContentsMargins(0,0,0,0);
}

void CardReviewerWidget::createReviewers()
{
   cardReviewerRegular = new CardReviewerRegular;
   connect(cardReviewerRegular, SIGNAL(playingFinishedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerRegular, SIGNAL(playingStartedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerRegular, SIGNAL(userAnsweredSignal(UserAnswer)), this, SLOT(cardReviewerReportsUserAnswered(UserAnswer)));
   cardReviewerVocabulary = new CardReviewerVocabulary;
   connect(cardReviewerVocabulary, SIGNAL(playingFinishedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerVocabulary, SIGNAL(playingStartedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerVocabulary, SIGNAL(userAnsweredSignal(UserAnswer)), this, SLOT(cardReviewerReportsUserAnswered(UserAnswer)));
   cardReviewerMap = new CardReviewerMap;
   connect(cardReviewerMap, SIGNAL(playingFinishedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerMap, SIGNAL(playingStartedSignal()), this, SLOT(setPlayStopActionIcon()));
   connect(cardReviewerMap, SIGNAL(userAnsweredSignal(UserAnswer)), this, SLOT(cardReviewerReportsUserAnswered(UserAnswer)));
   cardReviewerStackedWidget -> addWidget(cardReviewerRegular);
   cardReviewerStackedWidget -> addWidget(cardReviewerVocabulary);
   cardReviewerStackedWidget -> addWidget(cardReviewerMap);
   cardReviewerStackedWidget -> setCurrentWidget(cardReviewerRegular);
}

void CardReviewerWidget::reviewCards(CardBox *cb)
{
   cardBox = cb;
   sidePanel -> showItemOfWidget(cardBoxTreeWidget_ActiveCards);
   cardBoxTreeWidget_ActiveCards -> setCardBox(cardBox);
   cardBoxTreeWidget_ActiveCards -> extractTitles(Settings::settings() -> reviewExtractTitles);
   if (Settings::settings() -> reviewShowCardsList)
      sidePanel -> setCurrentWidget(cardBoxTreeWidget_ActiveCards);
   else
      sidePanel -> hideCurrentWidget();
   findToBeAnsweredCards();
   resetProgressBar();
   updateList();
   totalAnswers = 0;
   correctAnswers = 0;

   reviewingInProgress = toBeAnsweredCards.size() > 0;
   if (reviewingInProgress)
   {
      if (Settings::settings() -> reviewNavigation == Navigation_SelectNextMarkedCard
              || Settings::settings() -> reviewNavigation == Navigation_SelectNextCard)
         cardBoxTreeWidget_ActiveCards -> selectCard(toBeAnsweredCards[0]);
      else if (Settings::settings() -> reviewNavigation == Navigation_SelectPreviousMarkedCard
              || Settings::settings() -> reviewNavigation == Navigation_SelectPreviousCard)
         cardBoxTreeWidget_ActiveCards -> selectCard(toBeAnsweredCards[toBeAnsweredCards.size() - 1]);
      else if (Settings::settings() -> reviewNavigation == Navigation_RandomlySelectMakedCard)
         selectARandomMarkedCard();
      else
         selectARandomCard();
   }

   updateActions();

   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("Try to recall the solution. "
                                                         "Then, press space to verify. "
                                                         "If you succeded, press 1. "
                                                         "Otherwise, press 2"), 0);
}

bool CardReviewerWidget::askToFinish()
{
   bool okToFinish = true;
   if (reviewingInProgress)
   {
      QMessageBox messageBox(this);
      messageBox.setText(tr("Reviewing is in progress...\nAre you sure you want to leave?"));
      messageBox.setWindowTitle(tr("Leave"));
      messageBox.setIcon(QMessageBox::Warning);
      QPushButton *showReport = messageBox.addButton(tr("&Show report then leave"), QMessageBox::AcceptRole);
      messageBox.addButton(tr("&Yes"), QMessageBox::AcceptRole);
      QPushButton *no = messageBox.addButton(tr("&No"), QMessageBox::RejectRole);
      messageBox.setDefaultButton(no);
      messageBox.exec();
      okToFinish = messageBox.clickedButton() != no;
      if (messageBox.clickedButton() == showReport)
      {
         reportDialog -> setData(totalAnswers, correctAnswers);
         reportDialog -> exec();
      }
   }
   if (okToFinish)
   {
      sidePanel -> hideItemOfWidget(cardBoxTreeWidget_ActiveCards);
      reviewingInProgress = false;
      return true;
   }
   else
      return false;
}

void CardReviewerWidget::settingsChanged()
{
   CardPosition pos;
   cardBoxTreeWidget_ActiveCards -> currentCardPosition(&pos);
   updateList();
   cardBoxTreeWidget_ActiveCards -> selectCard(pos);
}

void CardReviewerWidget::updateCardReviewer()
{
   updateList();
}

void CardReviewerWidget::findToBeAnsweredCards()
{
   QDate currentDate = QDate::currentDate();
   int maxMarkedCards = Settings::settings()->reviewCardsNumber;
   QList<int> daysForStages;
   daysForStages << 1 << 3 << 5 << 10 << 20;
   for (int i=5; i<cardBox->countStages()-1; i++)
      daysForStages.append(static_cast<int>(daysForStages.last()*1.7));

   QList<QList <CardPosition> > qualifiedCards1;
   QList<QList <CardPosition> > qualifiedCards2;
   for (int i=0; i<cardBox->countStages()-1; i++)
   {
      qualifiedCards1 << QList<CardPosition>();
      qualifiedCards2 << QList<CardPosition>();
      for (int j=0; j<cardBox->countActiveCards(i); j++)
      {
         CardStatistics &cardStatistics = cardBox->at(CardPosition(i, j))->getCardStatistics();
         QDate lastDate = cardStatistics.totalAnswersCount() ? cardStatistics.getLastAnswer().getDate()
            : QDate();
         if (lastDate.daysTo(currentDate) >= daysForStages[i] || lastDate.isNull())
            qualifiedCards1[i] << CardPosition(i, j);
         else
            qualifiedCards2[i] << CardPosition(i, j);
      }
   }

   QList<int> factors;
   factors << (cardBox->countStages()-1) * 4;
   for (int i=1; i<cardBox->countStages()-1; i++)
      factors << cardBox->countStages()-i-1;
   int totalFactors=0;
   for (int i=0; i<factors.size(); i++)
      totalFactors += factors[i];

   toBeAnsweredCards.clear();
   for (int n=0; ; n++)
   {
      int added=0;
      for (int stage=0; stage<cardBox->countStages()-1; stage++)
      {
         for (int card=0; card+n*factors[stage] < qualifiedCards1[stage].size()
                          && card < factors[stage]
                          && toBeAnsweredCards.size() < maxMarkedCards; card++)
         {
            toBeAnsweredCards << qualifiedCards1[stage][card+n*factors[stage]];
            added++;
         }
      }
      if (added==0)
         break;
   }

   for (int n=0; ; n++)
   {
      int added=0;
      for (int stage=0; stage<cardBox->countStages()-1; stage++)
      {
         for (int card=0; card+n*factors[stage] < qualifiedCards2[stage].size()
                          && card < factors[stage]
                          && toBeAnsweredCards.size() < maxMarkedCards; card++)
         {
            toBeAnsweredCards << qualifiedCards2[stage][card+n*factors[stage]];
            added++;
         }
      }
      if (added==0)
         break;
   }
   qStableSort(toBeAnsweredCards); //we need it for later
}

void CardReviewerWidget::updateList()
{
   cardBoxTreeWidget_ActiveCards -> clearToBeAnsweredCards();
   cardBoxTreeWidget_ActiveCards -> setToBeAnsweredCards(toBeAnsweredCards);
   cardBoxTreeWidget_ActiveCards -> updateCards();
}

void CardReviewerWidget::updateReviewerAndActions()
{
   CardPosition pos;
   bool rightItemSelected;

   rightItemSelected = cardBoxTreeWidget_ActiveCards -> currentCardPosition(&pos) == ItemType_Card;
   verifyAction -> setEnabled(rightItemSelected);
   yesAction -> setEnabled(rightItemSelected);
   noAction -> setEnabled(rightItemSelected);
   skipAction -> setEnabled(rightItemSelected);
   playStopAction -> setEnabled(rightItemSelected);
   
   getCurrentReviewer() -> setEnabled(rightItemSelected);

   if (rightItemSelected)
   {
      Card *card = cardBox->at(pos);
      CardReviewer *reviewer = 0;
      if (card -> getType() == CardType_Regular)
         reviewer = cardReviewerRegular;
      else if (card -> getType() == CardType_Vocabulary)
         reviewer = cardReviewerVocabulary;
      else if (card -> getType() == CardType_Map)
         reviewer = cardReviewerMap;
      else // should not happen
      {
         getCurrentReviewer()->clear();
         return;
      }

      isVerifying = false;
      cardReviewerStackedWidget -> setCurrentWidget(reviewer);
      reviewer -> showCard(card);
      updateHints();
   }
   else
      getCurrentReviewer() -> clear();
}

void CardReviewerWidget::verifySlot()
{
   isVerifying = !isVerifying;
   getCurrentReviewer() -> verify();
   updateHints();
}

void CardReviewerWidget::yesSlot()
{
   CardPosition pos;
   if (cardBoxTreeWidget_ActiveCards -> currentCardPosition(&pos) != ItemType_Card)
      return;
   userAnswered(pos, UserAnswer_Yes);
   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("Correct answer!"));
}

void CardReviewerWidget::noSlot()
{
   CardPosition pos;
   if (cardBoxTreeWidget_ActiveCards -> currentCardPosition(&pos) != ItemType_Card)
      return;
   userAnswered(pos, UserAnswer_No);
   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("Wrong answer!"));
}

void CardReviewerWidget::skipSlot()
{
   CardPosition pos;
   if (cardBoxTreeWidget_ActiveCards -> currentCardPosition(&pos) != ItemType_Card)
      return;
   userAnswered(pos, UserAnswer_Skip);
   MainWindow::mainWindow() -> showMessageOnStatusBar(tr("Skiped!"));
}

void CardReviewerWidget::resetProgressBar()
{
   progressBar -> setValue(0);
   progressBar -> setMaximum(0);
   updateAnswersStatisticsLabel();
}

void CardReviewerWidget::userAnswered(CardPosition position, UserAnswer answer)
{
   if (answer != UserAnswer_Skip)
   {
      totalAnswers++;
      if (answer == UserAnswer_Yes)
         correctAnswers++;
      progressBar -> setMaximum(progressBar->maximum() + 1);
      //setValue() doesn't have any affect unless progressBar maximum value
      //is greater than 0. so setValue(0) should be called to avoid from getting
      //unknown numbers as value.
      if (progressBar -> maximum() == 1)
         progressBar -> setValue(0);
      if (answer == UserAnswer_Yes)
         progressBar -> setValue(progressBar->value() + 1);
      updateAnswersStatisticsLabel();
      progressBar -> update();

      cardBox -> at(position) -> getCardStatistics().addAnswer(Answer(answer==UserAnswer_Yes));

      if (answer == UserAnswer_Yes && position.stage < cardBox->countStages() - 1)
         cardBox -> moveToActiveCards(position, position.stage + 1, BeginningOrEnd_End);

      else if (answer == UserAnswer_No && position.stage > 0)
         cardBox -> moveToActiveCards(position, 0, BeginningOrEnd_End);

   }

   userAnswered_updateToBeAnsweredCards(position, answer);
   updateList();
   if (answer != UserAnswer_Skip)
      emit somethingEdited();

   if (checkWhetherReviewIsFinished() && reviewingInProgress)
   {
      showReport();
      reviewingInProgress = false;
      emit reviewingFinished();
   }
   else
      userAnswered_SelectNextCard(position, answer);
}

bool CardReviewerWidget::checkWhetherReviewIsFinished()
{
   return !toBeAnsweredCards.size();
}

void CardReviewerWidget::showReport()
{
   reportDialog -> setData(totalAnswers, correctAnswers);
   reportDialog -> exec();
}

void CardReviewerWidget::userAnswered_updateToBeAnsweredCards(CardPosition position, UserAnswer answer)
{
   //Note:
   //This algorithm works perfectly but a better
   //algorithm should be written later!
   if (toBeAnsweredCards.indexOf(position) != -1
       && position.stage == 0 && answer == UserAnswer_No)
      toBeAnsweredCards.removeAt(toBeAnsweredCards.indexOf(position));
   else if (toBeAnsweredCards.indexOf(position) != -1
            && position.stage == cardBox->countStages()-1 && answer == UserAnswer_Yes)
      toBeAnsweredCards.removeAt(toBeAnsweredCards.indexOf(position));
   else if (toBeAnsweredCards.indexOf(position) != -1
            && answer == UserAnswer_Skip)
      toBeAnsweredCards.removeAt(toBeAnsweredCards.indexOf(position));
   else if (toBeAnsweredCards.size() > 0)
   {
      int i=toBeAnsweredCards.indexOf(position);
      if (i==-1)
      {
         for (i=0; i<toBeAnsweredCards.size(); i++)
            if (toBeAnsweredCards[i].stage >= position.stage)
               break;

         if (i>=toBeAnsweredCards.size())
         {
            toBeAnsweredCards.removeLast();
            return;
         }
         else if (toBeAnsweredCards[i].stage > position.stage)
         {
            toBeAnsweredCards.removeLast();
            return;
         }
         else if (position.stage == 0 && answer == UserAnswer_No)
         {
            for (; i<toBeAnsweredCards.size(); i++)
               if (toBeAnsweredCards[i].stage != position.stage)
                  break;
            toBeAnsweredCards.removeAt(i-1);
            return;
         }
         else
         {
            for (; i<toBeAnsweredCards.size(); i++)
            {
               if (toBeAnsweredCards[i].stage != position.stage)
               {
                  toBeAnsweredCards.removeAt(i-1);
                  return;
               }
               else if (toBeAnsweredCards[i].cardNumber > position.cardNumber)
                  break;
            }
            if (i>=toBeAnsweredCards.size())
            {
               toBeAnsweredCards.removeLast();
               return;
            }
            else
            {
               for(;i<toBeAnsweredCards.size(); i++)
               {
                  if (position.stage != toBeAnsweredCards[i].stage)
                     break;
                  toBeAnsweredCards[i].cardNumber--;
               }
               toBeAnsweredCards.removeAt(i-1);
            }
         }
      }
      else
      {
         toBeAnsweredCards.removeAt(i);

         for(;i<toBeAnsweredCards.size(); i++)
         {
            if (position.stage != toBeAnsweredCards[i].stage)
               break;
            toBeAnsweredCards[i].cardNumber--;
         }
      }
   }
}

void CardReviewerWidget::updateAnswersStatisticsLabel()
{
   answersStatisticsLabel -> setText(tr("   Correct answers: %1/%2   ").arg(progressBar->value()).arg(progressBar->maximum()));
}

void CardReviewerWidget::userAnswered_SelectNextCard(CardPosition pos, UserAnswer answer)
{
   if (Settings::settings() -> reviewNavigation == Navigation_SelectNextMarkedCard)
   {
      if (answer == UserAnswer_Skip)
         pos.cardNumber++;
      if (!cardBoxTreeWidget_ActiveCards->selectNextToBeAnsweredCard(pos))
         if (!cardBoxTreeWidget_ActiveCards->selectNextToBeAnsweredCard(CardPosition(0, 0)))
         {
            if ((pos.stage == 0 && answer == UserAnswer_No)
                    || (pos.stage == cardBox->countStages() - 1 && answer == UserAnswer_Yes))
               cardBoxTreeWidget_ActiveCards->selectCard(CardPosition(pos.stage, pos.cardNumber+1));
            else
               cardBoxTreeWidget_ActiveCards->selectCard(pos);
         }
   }

   else if (Settings::settings() -> reviewNavigation == Navigation_SelectPreviousMarkedCard)
   {
      if (!cardBoxTreeWidget_ActiveCards->selectPreviousToBeAnsweredCard(pos))
      {
         if (!cardBoxTreeWidget_ActiveCards->selectPreviousToBeAnsweredCard
                 (CardPosition(cardBox->countStages()-1, cardBox->countActiveCards(cardBox->countStages()-1)-1)))
         {
            cardBoxTreeWidget_ActiveCards->selectCard(
                 cardBox->prevPosition(pos, MovePolicy_ThroughActiveCards));
         }
      }
   }
   
   else if (Settings::settings() -> reviewNavigation == Navigation_SelectNextCard)
   {
      if (answer == UserAnswer_Skip
              || (answer == UserAnswer_No && pos.stage == 0)
              || (answer == UserAnswer_Yes && pos.stage == cardBox->countStages()-1))
      {
         cardBoxTreeWidget_ActiveCards->selectCard(
              cardBox->nextPosition(pos, MovePolicy_ThroughActiveCards));
      }
      else
         cardBoxTreeWidget_ActiveCards->selectCard(pos);
   }

   else if (Settings::settings() -> reviewNavigation == Navigation_SelectPreviousCard)
   {
      cardBoxTreeWidget_ActiveCards->selectCard(
              cardBox->prevPosition(pos, MovePolicy_ThroughActiveCards));
   }

   else if (Settings::settings() -> reviewNavigation == Navigation_RandomlySelectCard
           || (Settings::settings() -> reviewNavigation == Navigation_RandomlySelectMakedCard
               && toBeAnsweredCards.size()==0))
   {
      selectARandomCard();
   }

   else if (Settings::settings() -> reviewNavigation == Navigation_RandomlySelectMakedCard)
   {
      selectARandomMarkedCard();
   }
}
   
void CardReviewerWidget::setPlayStopActionIcon()
{
   if (getCurrentReviewer() -> isPlaying())
      playStopAction -> setIcon(QIcon(":/images/stop.png"));
   else
   {
      if (QApplication::layoutDirection() == Qt::RightToLeft)
         playStopAction->setIcon(QIcon(":/images/playRTL.png"));
      else
         playStopAction -> setIcon(QIcon(":/images/play.png"));
   }
}

void CardReviewerWidget::playStopSlot()
{
   CardReviewer *reviewer = getCurrentReviewer();
   if (reviewer -> isPlaying())
      reviewer -> stop();
   else
      play();
}

void CardReviewerWidget::play()
{
   getCurrentReviewer() -> play();
   setPlayStopActionIcon();
}

void CardReviewerWidget::autoPlaySlot(bool checked)
{
   Settings::settings()->reviewAutoPlay = checked;
}

CardReviewer* CardReviewerWidget::getCurrentReviewer()
{
   return static_cast<CardReviewer*>(cardReviewerStackedWidget->currentWidget());
}

void CardReviewerWidget::showHintsSlot()
{
   Settings::settings() -> reviewShowHints = showHintsAction -> isChecked();
   updateHints();
   updateActions();
}

void CardReviewerWidget::showHintsOnVerifySlot()
{
   Settings::settings() -> reviewShowHintsOnlyOnVerify = showHintsOnVerifyAction -> isChecked();
   updateHints();
}

void CardReviewerWidget::cardReviewerReportsUserAnswered(UserAnswer answer)
{
   if (answer == UserAnswer_Yes)
      yesSlot();
   else if (answer == UserAnswer_No)
      noSlot();
   else //skip
      skipSlot();
}

void CardReviewerWidget::updateActions()
{
   autoPlayAction -> setChecked(Settings::settings() -> reviewAutoPlay);
   showHintsAction -> setChecked(Settings::settings() -> reviewShowHints);
   showHintsOnVerifyAction -> setChecked(Settings::settings() -> reviewShowHintsOnlyOnVerify);
   showHintsOnVerifyAction -> setEnabled(showHintsAction->isChecked());
}

void CardReviewerWidget::updateHints()
{
   CardReviewer *reviewer = getCurrentReviewer();
   if (!showHintsAction -> isChecked())
      reviewer -> hideHints();
   else if (showHintsOnVerifyAction -> isChecked())
   {
      if (isVerifying)
         reviewer -> showHints();
      else
         reviewer -> hideHints();
   }
   else
      reviewer -> showHints();
}

void CardReviewerWidget::selectARandomMarkedCard()
{
   if (toBeAnsweredCards.size() == 0)
      qDebug("CardReviewerWidget::selectARandomMarkedCard(): Division by zero!");
   else
      cardBoxTreeWidget_ActiveCards->selectCard(toBeAnsweredCards[Random::random()->getARandomNumber(0, toBeAnsweredCards.size()-1)]);
}

void CardReviewerWidget::selectARandomCard()
{
   int allCards = cardBox->countActiveCards();
   if (!allCards)
      return;

   int cardNumber = Random::random() -> getARandomNumber(0, allCards-1);
   int stage;
   for (stage=0; cardNumber >= cardBox->countActiveCards(stage); stage++)
      cardNumber -= cardBox->countActiveCards(stage);

   cardBoxTreeWidget_ActiveCards->selectCard(CardPosition(stage, cardNumber));
}



