// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDREVIEWERWIDGET_H
#define CARDREVIEWERWIDGET_H

#include <QWidget>
//#include "CardReviewer.h"
#include "Card.h"
class QTreeWidgetItem;
class CardBoxTreeWidget_ActiveCards;
class QAction;
class QToolBar;
class QStackedWidget;
class QProgressBar;
class QLabel;
class QMenu;
class QToolButton;
class SidePanel;
class QSplitter;
class ReportDialog;
class CardReviewer;

enum Navigation {Navigation_SelectNextMarkedCard, Navigation_SelectPreviousMarkedCard,
                 Navigation_SelectNextCard, Navigation_SelectPreviousCard,
                 Navigation_RandomlySelectMakedCard, Navigation_RandomlySelectCard};
enum UserAnswer {UserAnswer_Yes, UserAnswer_No, UserAnswer_Skip};

class CardReviewerWidget : public QWidget
{
   Q_OBJECT
   
   public:
      CardReviewerWidget(SidePanel *sidePanel, QWidget* = 0);
      void reviewCards(CardBox*);
      bool askToFinish();
      void settingsChanged();
      void updateCardReviewer();
   
   signals:
      void somethingEdited();
      void reviewingFinished();
   
   private slots:
      void updateReviewerAndActions();
      void verifySlot();
      void yesSlot();
      void noSlot();
      void skipSlot();
      void updateList();
      void playStopSlot();
      void autoPlaySlot(bool);
      void setPlayStopActionIcon();
      void play();
      void updateHints();
      void updateActions();
      void showHintsSlot();
      void showHintsOnVerifySlot();
      void cardReviewerReportsUserAnswered(UserAnswer);
   
   private:
      QList<CardPosition> toBeAnsweredCards;
      bool reviewingInProgress;
      CardBox *cardBox;
      CardReviewer *cardReviewerRegular;
      CardReviewer *cardReviewerVocabulary;
      CardReviewer *cardReviewerMap;
      QStackedWidget *cardReviewerStackedWidget;
      CardBoxTreeWidget_ActiveCards *cardBoxTreeWidget_ActiveCards;
      bool isVerifying;
      SidePanel *sidePanel;
      int totalAnswers;
      int correctAnswers;
      ReportDialog *reportDialog;

      QAction *playStopAction;
      QAction *autoPlayAction;
      QAction *verifyAction;
      QAction *yesAction;
      QAction *noAction;
      QAction *skipAction;
      QAction *showHintsAction;
      QAction *showHintsOnVerifyAction;

      QMenu *hintsMenu;
      QToolButton *hintsButton;

      QProgressBar *progressBar;
      QLabel *answersStatisticsLabel;

      QToolBar *firstToolBar;
      QToolBar *secondToolBar;

      void createActionsAndWidgets();
      void createLayouts();
      void createReviewers();
      void userAnswered(CardPosition, UserAnswer);
      void userAnswered_updateToBeAnsweredCards(CardPosition, UserAnswer);
      void userAnswered_SelectNextCard(CardPosition, UserAnswer);
      bool checkWhetherReviewIsFinished();
      void showReport();
      void updateAnswersStatisticsLabel();
      void findToBeAnsweredCards();
      CardReviewer* getCurrentReviewer();
      void selectARandomMarkedCard();
      void selectARandomCard();
      void resetProgressBar();
};

#endif

