// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "CardReviewerMap.h"
#include "maphelper.h"
#include "CardMap.h"
#include <QTimer>

CardReviewerMap::CardReviewerMap(QWidget *parent)
   : CardReviewer(parent)
{
   setupUi(this);
   mainSplitter -> setStretchFactor(0,10);
   mainSplitter -> setStretchFactor(1,1);

   answered = false;
   isVerifying = false;
   connect(_mapWidget, SIGNAL(clicked(QRgb, const QPoint&)), this, SLOT(clickedOnMapWidget(QRgb, const QPoint&)));
}

void CardReviewerMap::showCard(const Card *c)
{
   card = dynamic_cast<const CardMap*>(c);
   if (!card)
   {
      qDebug("CardReviewerMap::showCard(): received card is not a CardMap");
      return;
   }
   answered = false;
   isVerifying = false;
   showMap();
   updateHints();
   askQuestion();
}

void CardReviewerMap::verify()
{
   isVerifying = !isVerifying;
   if (isVerifying)
   {
      highlightCorrectAnswer(0);
      questionLabel -> setText(tr("<p align=\"center\"><font size=\"6\" color=\"blue\">Verify</font></p>"));
   }
   else
   {
      _mapWidget -> clearHighlight();
      askQuestion();
   }
}

void CardReviewerMap::clear()
{
   questionLabel -> clear();
   _mapWidget -> clear();
   commentsTextViewer -> clear();
}

void CardReviewerMap::play()
{

}

void CardReviewerMap::stop()
{

}

bool CardReviewerMap::isPlaying()
{
   //NOTE:
   return false;
}

void CardReviewerMap::showHints()
{
   mainSplitter -> widget(1) -> show();
}

void CardReviewerMap::hideHints()
{
   mainSplitter -> widget(1) -> hide();
}

void CardReviewerMap::on_commentsCheckBox_toggled(bool checked)
{
   commentsTextViewer -> setVisible(checked);
}

void CardReviewerMap::clickedOnMapWidget(QRgb rgb, const QPoint &point)
{
   if (!answered && !isVerifying)
   {
      answered = true;
      highlightCorrectAnswer(0);

      if (correctRegion() == rgb)
      {
         questionLabel -> setText(tr("<p align=\"center\"><font size=\"6\" color=\"green\">Correct!</font></p>"));
         QTimer::singleShot(1700, this, SLOT(userAnsweredCorrectly()));
      }
      else
      {
         questionLabel -> setText(tr("<p align=\"center\"><font size=\"6\" color=\"red\">Wrong!</font></p>"));
         QTimer::singleShot(1700, this, SLOT(userAnsweredWrongly()));
      }

   }
}

void CardReviewerMap::userAnsweredCorrectly()
{
   emit userAnsweredSignal(UserAnswer_Yes);
}

void CardReviewerMap::userAnsweredWrongly()
{
   emit userAnsweredSignal(UserAnswer_No);
}

void CardReviewerMap::updateHints()
{
   commentsTextViewer -> setPlainText(card -> getComments());

   bool visible;
   int visibleWidgets=0;
   visible = !card -> getComments().isEmpty();
   commentsCheckBox -> setVisible(visible);
   commentsTextViewer -> setVisible(visible && commentsCheckBox->isChecked());
   visibleWidgets += visible ? 1 : 0;

   messageLabel -> setVisible(!visibleWidgets);
   if (!visibleWidgets)
      messageLabel -> setText(getNoHintsMessage());
}

void CardReviewerMap::showMap()
{
   for (int i=0; i<MapHelper::getListOfMaps().size(); i++)
      if (MapHelper::getListOfMaps()[i]->getName() == card -> getMap())
      {
         map = MapHelper::getListOfMaps()[i];
         _mapWidget -> init(map->getMapFile());
         break;
      }
}

QRgb CardReviewerMap::correctRegion()
{
   return map -> getColor(card -> getRegion()).rgb();
}

void CardReviewerMap::highlightCorrectAnswer(int time)
{
   _mapWidget -> highlightRegion(correctRegion(), qRgb(0, 255, 0), time);
}

void CardReviewerMap::askQuestion()
{
   questionLabel -> setText(("<p align=\"center\"><font size=\"5\" color=\"blue\">"
                            + tr("Please click on <b>%1</b>")
                             + "</font></p>").arg(card -> getRegion()));
}

