// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include "CardReviewerRegular.h"
#include "Card.h"
#include "CardRegular.h"
#include "Utilities.h"
#include "Settings.h"

CardReviewerRegular::CardReviewerRegular(QWidget *parent)
   : CardReviewer(parent)
{
   setupUi(this);
   textViewer -> setReadOnly(true);
   mainSplitter -> setStretchFactor(0,10);
   mainSplitter -> setStretchFactor(1,1);
   isFrontSideVisible = true;
   readingID = 0;
   connect(Reader::reader(), SIGNAL(readingFinished(int)), this, SLOT(readingFinishedSlot(int)));
}

void CardReviewerRegular::showCard(const Card *c)
{
   card = dynamic_cast<const CardRegular*>(c);
   if (!card)
   {
      qDebug("CardReviewerRegular::showCard(): received card is not a CardRegular");
      return;
   }
   if (Settings::settings() -> reviewRegularFrontToBack)
      showFrontSide();
   else
      showBackSide();
   updateHints();
}

void CardReviewerRegular::showFrontSide()
{
   QString frontSide = "<p style=\"color:#a0a0a0; font-size: x-small;"
                       "text-align: center; padding: 3px; background: #f0f0f0;"
                       "border-style: solid; border-width:2px; border-color:#f5f5f5;\">"
                       + tr("Front Side") + "</p>";
   Document document = *(card -> getFrontSideDocument());
   document.setText(frontSide + document.getText());
   textViewer -> setDocument(document);
   isFrontSideVisible = true;
   if (Settings::settings() -> reviewAutoPlay)
      play();
}

void CardReviewerRegular::showBackSide()
{
   QString backSide = "<p style=\"color:#a0a0a0; font-size: x-small;"
                      "text-align: center; padding: 3px; background: #f0f0f0;"
                      "border-style: solid; border-width:2px; border-color:#f5f5f5;\">"
                      + tr("Back Side") + "</p>";
   Document document = *(card -> getBackSideDocument());
   document.setText(backSide + document.getText());
   textViewer -> setDocument(document);
   isFrontSideVisible = false;
   if (Settings::settings() -> reviewAutoPlay)
      play();
}

void CardReviewerRegular::verify()
{
   if (isFrontSideVisible)
      showBackSide();
   else
      showFrontSide();
}

void CardReviewerRegular::updateHints()
{
   commentsTextViewer -> setPlainText(card -> getComments());

   bool visible;
   int visibleWidgets=0;
   visible = !card -> getComments().isEmpty();
   commentsCheckBox -> setVisible(visible);
   commentsTextViewer -> setVisible(visible && commentsCheckBox->isChecked());
   visibleWidgets += visible ? 1 : 0;

   messageLabel -> setVisible(!visibleWidgets);
   if (!visibleWidgets)
      messageLabel -> setText(getNoHintsMessage());
}

void CardReviewerRegular::clear()
{
   textViewer -> clear();
   commentsTextViewer -> clear();
}

void CardReviewerRegular::play()
{
   if (isFrontSideVisible)
       readingID = Reader::reader() -> readHtml(card -> getFrontSideDocument() -> getText());
    else
       readingID = Reader::reader() -> readHtml(card -> getBackSideDocument() -> getText());
   CardReviewer::playingStarted();
}

void CardReviewerRegular::stop()
{
   Reader::reader() -> stop();
   //can cause double signal emitting
   CardReviewer::playingFinished();
}

bool CardReviewerRegular::isPlaying()
{
   return Reader::reader() -> isReading();
}

void CardReviewerRegular::on_commentsCheckBox_toggled(bool checked)
{
   commentsTextViewer -> setVisible(checked);
}

void CardReviewerRegular::showHints()
{
   mainSplitter -> widget(1) -> show();
}

void CardReviewerRegular::hideHints()
{
   mainSplitter -> widget(1) -> hide();
}

void CardReviewerRegular::readingFinishedSlot(int id)
{
   if (readingID == id)
      CardReviewer::playingFinished();
}
