// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDREVIEWERVOCABULARY_H
#define CARDREVIEWERVOCABULARY_H

#include "CardReviewer.h"
#include "ui_CardReviewerVocabulary.h"
class CardVocabulary;
class QToolBar;
class TranslationDocument;
class PrettyButton;

class CardReviewerVocabulary : public CardReviewer, public Ui::CardReviewerVocabulary
{
   Q_OBJECT

   public:
      CardReviewerVocabulary(QWidget* = 0);

      virtual void showCard(const Card*);
      virtual void verify();
      virtual void clear();

   public slots:
      virtual void play();
      virtual void stop();
      virtual bool isPlaying();
      virtual void showHints();
      virtual void hideHints();
      
   private slots:
      void on_wordTypeCheckBox_toggled(bool);
      void on_examplesCheckBox_toggled(bool);
      void on_commentsCheckBox_toggled(bool);
      void on_synonymsCheckBox_toggled(bool);
      void on_antonymsCheckBox_toggled(bool);
      void readingFinishedSlot(int);
      void languageButtonHovered();
      void languageButtonClicked();

   private:
      const CardVocabulary *card;
      bool isFrontSideVisible;
      int readingID;
      QList<PrettyButton*> languageButtons;
      QToolBar *languagesToolBar;
      int frontSideLanguageIndex;
      int backSideLanguageIndex;

      void createLanguageButtons();
      void showFrontSide();
      void updateHints();
      void updateTextViewer();
      void updateLanguageButtons();
      void selectLanguage(int);
      TranslationDocument* getCurrentTranslationDocument();
      int getCurrentLanguageButtonIndex();
};

#endif


