// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "CardRegular.h"
#include "Utilities.h"
#include <QObject>

CardRegular::CardRegular()
{
   frontSideDocument = new Document;
   backSideDocument = new Document;
}

CardRegular::~CardRegular()
{
   delete frontSideDocument;
   delete backSideDocument;
}

CardType CardRegular::getType() const
{
   return CardType_Regular;
}

QString CardRegular::makeTitle() const
{
   if (frontSideDocument->isModified())
   {
      title = makeTitleFromString(HtmlParser::htmlParser()->htmlToPlainText(frontSideDocument->getText()));
      frontSideDocument->setModified(false);
   }
   return title;
}

bool CardRegular::contains(const QString &string, bool isRegExp) const
{
   HtmlParser *htmlParser = HtmlParser::htmlParser();
   if (htmlParser->contains(frontSideDocument->getText(), string, isRegExp)
           || htmlParser->contains(backSideDocument->getText(), string, isRegExp))
      return true;

   QRegExp regExp = isRegExp ? QRegExp(string) : QRegExp(string, Qt::CaseInsensitive, QRegExp::Wildcard);
   return (comments.contains(regExp));
}

Document* CardRegular::getFrontSideDocument()
{
   return frontSideDocument;
}

const Document* CardRegular::getFrontSideDocument() const
{
   return frontSideDocument;
}

Document* CardRegular::getBackSideDocument()
{
   return backSideDocument;
}

const Document* CardRegular::getBackSideDocument() const
{
   return backSideDocument;
}

QString CardRegular::getComments() const
{
   return comments;
}

void CardRegular::setComments(const QString &newComments)
{
   comments = newComments;
}

Card* CardRegular::cloneYourSelf() const
{
   CardRegular *newCard = new CardRegular(*this);
   newCard->frontSideDocument = new Document(*frontSideDocument);
   newCard->backSideDocument = new Document(*backSideDocument);
   return newCard;
}


