// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | YOU should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef CARDVOCABULARY_H
#define CARDVOCABULARY_H

#include "Card.h"

enum WordType {WordType_Noun=1, WordType_Verb=2, WordType_Adjective=4, WordType_Adverb=8, WordType_Other=16};

class TranslationDocument : public Document
{
   public:
      TranslationDocument(const QString& = QObject::tr("Unknown"));
      void setLanguageName(const QString&);
      QString getLanguageName() const;

   private:
      QString languageName;
};

class CardVocabulary : public Card
{
   public:
      CardVocabulary();
      virtual ~CardVocabulary();

      virtual CardType getType() const;
      virtual bool contains(const QString&, bool) const;

      QList<TranslationDocument*>& getTranslationDocuments();
      const QList<TranslationDocument*>& getTranslationDocuments() const;

      int getWordType() const;
      QString wordTypeToString(const QString&) const;
      void setWordType(int);
      int wordTypeFromString(const QString&, const QString&);
      QString getUserDefinedWordType() const;
      void setUserDefinedWordType(const QString&);
      QStringList getSynonyms() const;
      void setSynonyms(const QStringList&);
      QStringList getAntonyms() const;
      void setAntonyms(const QStringList&);
      QString getExamples() const;
      void setExamples(const QString&);
      QString getComments() const;
      void setComments(const QString&);
      
   protected:
      virtual QString makeTitle() const;
      virtual Card* cloneYourSelf() const;

   private:
      QList<TranslationDocument*> translationDocuments;

      mutable QString title;
      int wordType;
      QString userDefinedWordType;
      QStringList synonyms;
      QStringList antonyms;
      QString examples;
      QString comments;
};



#endif

