// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "Downloader.h"
#include <QEventLoop>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QProgressDialog>

Downloader::Downloader(QWidget *parent)
   : QObject(parent)
{
   reply = 0;
   eventLoop = new QEventLoop(this);
   network = new QNetworkAccessManager(this);
   progressDialog = new QProgressDialog(parent);
   connect(progressDialog, SIGNAL(canceled()), this, SLOT(downloadingCanceled()));
   progressDialog -> setLabelText(tr("Loading image..."));
   progressDialog -> setCancelButtonText(tr("Cancel"));
}

QByteArray Downloader::download(const QUrl &url)
{
   if (reply)
   {
      delete reply;
      reply = 0;
   }
   qDebug("Downloader::download(%s)", url.toString().toAscii().constData());
   reply = network -> get(QNetworkRequest(url));
//   reply = network -> get(QNetworkRequest(QUrl("http://www.google.com"))); //for testing
   connect(reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(downloadProgressSlot(qint64, qint64)));
   connect(reply, SIGNAL(finished()), this, SLOT(downloadingFinishedSlot()));

   progressDialog -> setWindowModality(Qt::WindowModal);
   progressDialog -> setMinimum(0);
   progressDialog -> setMaximum(100);
   progressDialog -> setValue(1);

   if (eventLoop -> exec())
      return reply -> readAll();
   else
      return QByteArray();
}

void Downloader::downloadProgressSlot(qint64 bytesReceived, qint64 bytesTotal)
{
   progressDialog -> setMaximum(bytesTotal);
   progressDialog -> setValue(bytesReceived);
}

void Downloader::downloadingFinishedSlot()
{
   //NOTE:
   qDebug("Downloader::downloadingFinishedSlot(): error=%d", reply->error());
   eventLoop -> exit(reply->error()==0);
}

void Downloader::downloadingCanceled()
{
   eventLoop -> exit(0);
}

