// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "ExportDialog.h"
#include "Card.h"
#include <QMessageBox>

ExportDialog::ExportDialog(const CardBoxes *cardBoxes,
                           FileWriterSettings::FileFormat _fileFormat,
                           QWidget *parent)
   : QDialog(parent), fileFormat(_fileFormat)
{
   setupUi(this);

   categoryList -> clear();
   for (int i=0; i<cardBoxes -> size(); i++)
   {
      QListWidgetItem *item = new QListWidgetItem(cardBoxes->at(i)->getBoxName(), categoryList);
      item -> setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
      item -> setCheckState(Qt::Checked);
   };

   if (fileFormat == FileWriterSettings::FileFormat_CSV)
   {
      stageStackGroupBox -> setEnabled(false);
      exportStatisticsCheckBox -> setEnabled(false);
   }
}

FileWriterSettings ExportDialog::getFileWriterSettings()
{
   FileWriterSettings settings;

   //categories
   for (int i=0; i<categoryList->count(); i++)
      if (categoryList->item(i)->checkState() == Qt::Checked)
         settings.categories << i;

   //stageStackMode
   if (moveToStackCheckBox -> isChecked())
      settings.stageStackMode = FileWriterSettings::StageStack_MoveToStack;
   else if (moveToStageCheckBox -> isChecked())
      settings.stageStackMode = FileWriterSettings::StageStack_MoveToStage;
   else if (doNotMoveCheckBox -> isChecked())
      settings.stageStackMode = FileWriterSettings::StageStack_DoNotMove;

   //stageNumber
   settings.stageNumber = stageSpinBox -> value()-1;

   //exportStatistics
   settings.exportStatistics = exportStatisticsCheckBox -> isChecked();

   //sortCards
   settings.sortCards = sortCardsCheckBox -> isChecked();

   //sortBy
   settings.sortBy = sortByComboBox -> currentIndex()==0 ? SortBy_CreationDate : SortBy_Title;

   //sortOrder
   settings.sortOrder = sortOrderComboBox -> currentIndex()==0 ? SortOrder_Ascending : SortOrder_Descending;

   //fileFormat
   settings.fileFormat = fileFormat;

   return settings;
}

void ExportDialog::on_selectAllButton_clicked()
{
   for (int i=0; i<categoryList->count(); i++)
      categoryList->item(i)->setCheckState(Qt::Checked);
}

void ExportDialog::on_deselectAllButton_clicked()
{
   for (int i=0; i<categoryList->count(); i++)
      categoryList->item(i)->setCheckState(Qt::Unchecked);
}

void ExportDialog::on_okButton_clicked()
{
   if (!getFileWriterSettings().categories.size())
      QMessageBox::warning(this, QObject::tr("Export..."),
                           QObject::tr("You should select at least one category."));
   else
      accept();
}



