// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef FILEREADER_H
#define FILEREADER_H

#include "Card.h"
class QString;
class QIODevice;
class QFile;
class QDomElement;
class QDomDocument;
class QDomNode;
class CardRegular;
class QTextStream;
class TranslationDocument;
class CardVocabulary;

class FileReaderSettings
{
   public:
      enum Category { Category_Create, Category_AddTo, Category_Automatic };
      enum StageStack { StageStack_AddToStack, StageStack_AddToStage, StageStack_Automatic };

      FileReaderSettings();

      Category categoryMode;
      StageStack stageStackMode;
      QString newCategoryName;
      int existingCategoryNumber;
      int stageNumber;
      bool importStatistics;
      bool clearExistingCards;
};

class FileReader
{
   public:
      FileReader() {}
      virtual ~FileReader() {}
      virtual bool readFromFile(const QString&, CardBoxes*, const FileReaderSettings&);

   private:
      bool read(QFile*, CardBoxes*);
      bool readFQ(QFile*, CardBoxes*);
      bool readCSV(QFile*, CardBoxes*);
      bool readKVTML(QFile*, CardBoxes*);

      bool isItFQ(QFile*) const;
      bool isItKVTML(QFile*) const;

      Card* cloneCard(const Card*, bool) const;
};

//======================================================================

class ParserFQ_1
{
   public:
      ParserFQ_1() {}
      virtual ~ParserFQ_1() {}
      virtual bool read(QDomDocument*, CardBoxes*);

   protected:
      virtual void parseBox(const QDomElement&, CardBox*);
      virtual void parseStage(const QDomElement&, CardBox*, int);
      virtual void parseCard(const QDomElement&, CardRegular*);
};

//======================================================================

class ParserFQ_2 : public ParserFQ_1
{
   public:
      ParserFQ_2();
      virtual bool read(QDomDocument*, CardBoxes*);
      void setReadingFromOldFiles(bool);

   protected:
      virtual void parseStage(const QDomElement&, CardBox*, int);
      virtual Card* parseCardRegular(const QDomElement&);
      virtual Card* parseCardVocabulary(const QDomElement&);

      virtual void parseDocument(const QDomElement&, Document*);
      virtual void parseCardStatistics(const QDomElement&, CardStatistics&);
      virtual QString getText(const QDomNode&);

      virtual QString removeStyle(QString);
      virtual QMap<QString, QString> substitudeNewNamesForImages(QString&);

   private:
      QString wordLanguage;
      QString meaningLanguage;
      bool readingFromOldFiles;
};

//======================================================================

class ParserFQ_3 : public ParserFQ_2
{
   public:
      ParserFQ_3() {}
      virtual bool read(QDomDocument*, CardBoxes*);

   protected:
      virtual void parseBox(const QDomElement&, CardBox*);
      //-1 means stack
      virtual void parseStageOrStack(const QDomElement&, CardBox*, int = -1);
      virtual Card* parseCardVocabulary(const QDomElement&);
      virtual Card* parseCardMap(const QDomElement&);
};

//======================================================================

class ParserCSV
{
   public:
      ParserCSV() {}
      bool read(QTextStream*, CardBoxes*);

   private:
      QList<QStringList> splitDocument(const QString&, const QString&, const QString&);
      QString extractValue(const QString&, const QString&, QString, int*, bool*);
};

//======================================================================

class ParserKVTML_2_0
{
   public:
      ParserKVTML_2_0() {}
      bool read(QDomDocument*, CardBoxes*);

   protected:
      QStringList languages;
      QList<Card*> cards;

      virtual void getLanguages(const QDomElement&);
      virtual Card* parseCardVocabulary(const QDomElement&);
      virtual QString getText(const QDomNode&);
      virtual void somethingIsWrong();
      virtual void parseTranslation(const QDomElement&, CardVocabulary*, TranslationDocument*);
      CardBox* parseCardBox(const QDomElement&);
};
   

#endif

