// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef FILEWRITER_H
#define FILEWRITER_H

#include <QList>
#include "Card.h"
class CardBoxes;
class CardRegular;
class CardVocabulary;
class CardMap;
class Document;
class CardStatistics;
class QString;
class QIODevice;
class QDomDocument;
class QDomElement;

class FileWriterSettings
{
   public:
      enum StageStack { StageStack_MoveToStack, StageStack_MoveToStage, StageStack_DoNotMove };
      enum FileFormat {FileFormat_FQ, FileFormat_CSV};
      
      FileWriterSettings();

      //NOTE: empty categories means all categories
      QList<int> categories;
      StageStack stageStackMode;
      int stageNumber;
      bool exportStatistics;
      bool sortCards;
      SortBy sortBy;
      SortOrder sortOrder;
      FileFormat fileFormat;
};

//======================================================================

class FileWriter
{
   public:
      FileWriter() {}
      ~FileWriter() {}
      bool writeToFile(const QString&, const CardBoxes*, const FileWriterSettings&);

   private:
      void addCardsToTheNewCardBoxes(const CardBoxes*, CardBoxes*, const FileWriterSettings&);
      Card* cloneCard(const Card*, bool) const;
};

//======================================================================

class FileWriterFQ
{
   public:
      void write(QIODevice*, const CardBoxes*);

   private:
      void writeCard(QDomDocument&, QDomElement&, const Card*);
      void writeCardRegular(QDomDocument&, QDomElement&, const CardRegular*);
      void writeCardVocabulary(QDomDocument&, QDomElement&, const CardVocabulary*);
      void writeCardMap(QDomDocument&, QDomElement&, const CardMap*);

      void writeDocument(QDomDocument&, QDomElement&, const Document*);
      void writeCardStatistics(QDomDocument&, QDomElement&, const CardStatistics&);
};

//======================================================================

class FileWriterCSV
{
   public:
      void write(QIODevice*, const CardBoxes*);

   private:
      QString cardRegularToString(const CardRegular*, const QString&, const QString&) const;
      QString cardVocabularyToString(const CardVocabulary*, const QString&, const QString&) const;
};


#endif


