 // +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "ImageFetcher.h"
#include "Card.h"
#include "Utilities.h"
#include <QLabel>
#include <QLineEdit>
#include <QToolBar>
#include <QProgressBar>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGraphicsView>
#include <QBuffer>
#include <QHttp>
#include <QUrl>
#include <QFile> //delete this line
#include <QTextStream> //delete this line
#include <QImageReader>
#include <QDrag>
#include <QMimeData>
#include <QGraphicsSceneMouseEvent>
#include <QToolButton>
#include <QApplication>

QString GraphicsPixmapItem::getUrl() const
{
   return url;
}

void GraphicsPixmapItem::setUrl(const QString &newUrl)
{
   url = newUrl;
}

void GraphicsPixmapItem::mousePressEvent (QGraphicsSceneMouseEvent *event)
{
   if (event->button() == Qt::LeftButton)
   {
      QDrag *drag = new QDrag(scene()->views()[0]);
      QMimeData *mimeData = new QMimeData;
      QList<QUrl> urls;
      urls << QUrl(getUrl());
      mimeData -> setUrls(urls);
      drag->setMimeData(mimeData);
      drag->setPixmap(pixmap());
      drag->exec();
   }
}

ImageFetcher::ImageFetcher(QWidget *parent)
   : QWidget(parent)
{
   pageRequestId = 0;
   pageRequestBuffer = 0;
   http = new QHttp(this);
   connect (http, SIGNAL(requestFinished(int, bool)), this, SLOT(requestFinished(int, bool)));
   connect (http, SIGNAL(done(bool)), this, SLOT(doneSlot()));
   connect (http, SIGNAL(requestStarted(int)), this, SLOT(requestStarted(int)));
   createWidgets();
   createLayout();
}

void ImageFetcher::requestStarted(int id)
{
    qDebug() << "Request started id: " << id;
}

void ImageFetcher::fetchImageFor(const QString &keyWords)
{
   searchLineEdit -> setText(keyWords);
   search();
}

void ImageFetcher::search()
{
   stop();
   QList<QGraphicsItem*> items = view -> scene() -> items();
   for (int i=0; i<items.size(); i++)
     view -> scene() -> removeItem(items[i]);
   images.clear();
   QMap<int, ImageRequest>::iterator iterator;
   for (iterator = imageRequests.begin(); iterator != imageRequests.end(); iterator++)
      delete iterator.value().buffer;
   imageRequests.clear();
   delete pageRequestBuffer;
   pageRequestBuffer = new QBuffer;

   /*

   QUrl url("http://images.google.com/images?q=" + searchLineEdit->text());
   http -> setHost("images.google.com");

     */

   QUrl url("http://ajax.googleapis.com/ajax/services/search/images?v=1.0&rsz=8&q=" + searchLineEdit->text());
   http -> setHost("ajax.googleapis.com");
   qDebug() << "Starting fetch " << url.toString();
   pageRequestId = http -> get(url.toString(), pageRequestBuffer);
   qDebug() << "Whatsup";
   progressBar -> setVisible(true);
   progressBar -> setMaximum(100);
   progressBar -> setValue(1);
}

void ImageFetcher::stop()
{
   qDebug("fetching aborted");
   http -> abort();
   progressBar -> setVisible(false);
}

void ImageFetcher::requestFinished(int requestId, bool error)
{
   if (error)
   {
      qDebug("Http request finished with ERROR");
      return;
   }
   if (requestId == pageRequestId)
   {
      /*QFile file("C:/Users/rstrogonovs/google.html");
      file.open(QIODevice::WriteOnly | QIODevice::Text);
      QTextStream out(&file);
      out << pageRequestBuffer->data();
      qDebug("wrote to google.html");*/

      fetchImagesFromHtml(pageRequestBuffer->data());
   }
   else
   {
      if (!imageRequests.contains(requestId))
         return;
      ImageRequest request = imageRequests[requestId];
      
      QImage image;
      if (!image.loadFromData(request.buffer->data()))
         return;
      image = image.scaled(150,150,Qt::KeepAspectRatio);
      GraphicsPixmapItem *item = new GraphicsPixmapItem;
      item -> setPixmap(QPixmap::fromImage(image));
      item -> setUrl(request.fullSizeUrl);
      images << item;
      view -> scene() -> addItem(item);
      progressBar -> setValue(progressBar->value()+1);
      updateView();
   }
}

void ImageFetcher::doneSlot()
{
   progressBar -> setVisible(false);
}

void ImageFetcher::fetchImagesFromHtml(const QString &html)
{   
    qDebug() <<"Extracting images";
   int index = 0;
   while(1)
   {
      //QRegExp regExp("imgurl\\\\x..([^\\\\]+)");
//       QRegExp regExp("imgurl=http://[\w\W]{4,400}&imgrefurl");
       QRegExp regExp("\"url\":\"(((http://www)|(http://)|(www))[-a-zA-Z0-9@:%_\\+.~#?&//=]+)\\.(jpg|jpeg|gif|png|bmp|tiff|tga|svg)\"");
      index = regExp.indexIn(html, index);
      //qDebug() << "HTML!";

      /*QFile file("C:/Users/rstrogonovs/google2.html");
            file.open(QIODevice::WriteOnly | QIODevice::Text);
            QTextStream out(&file);
            out << newHtml;
            qDebug("wrote to google2.html");*/

      qDebug() << "Fetcher Found index: " << index << "Is reg exp valid? " << regExp.isValid();

      if (index == -1)
         break;
      QString fullImagePath = regExp.cap(0);
      fullImagePath = fullImagePath.replace("\"url\":\"", "");
      fullImagePath = fullImagePath.remove( fullImagePath.size()-1 , 1 );

      qDebug() << "Full Image Path: " << fullImagePath;
      index += regExp.matchedLength();

      /*regExp.setPattern("\"([^\"]{1,20}):\"");
      index = regExp.indexIn(html, index);
      if (index == -1)
         break;*/
      QString thumbnailImagePath = fullImagePath;

      qDebug("Thumbnail image path = %s", thumbnailImagePath.toAscii().constData());
      qDebug("Full image Path = %s", fullImagePath.toAscii().constData());

      QBuffer *buffer = new QBuffer;
      buffer -> open(QIODevice::ReadWrite);

      ImageRequest request(fullImagePath, buffer);
      QUrl thumbnailImageUrl(thumbnailImagePath);
      http->setHost(thumbnailImageUrl.host());
      int requestId = http->get(thumbnailImageUrl.toString(), request.buffer);
      imageRequests[requestId] = request;

      qDebug() << "Index: " << index;
   }
   progressBar -> setMaximum(imageRequests.count());
}

void ImageFetcher::updateView()
{
   int y = 10;
   for (int i=0; i<images.size(); i++)
   {
      images[i] -> setPos(0, y);
      y += images[i]->boundingRect().height() + 10;
   }
}

void ImageFetcher::createWidgets()
{
   searchLabel = new QLabel(tr("Search:"), this);
   searchLineEdit = new QLineEdit(this);
   connect(searchLineEdit, SIGNAL(returnPressed()), this, SLOT(search()));
   searchLabel -> setBuddy(searchLineEdit);
   searchLineEdit -> setMinimumSize(70, searchLineEdit->height());

   searchButton = new QToolButton (this);
   searchButton -> setAutoRaise(true);
   if (QApplication::layoutDirection() == Qt::LeftToRight)
      searchButton -> setIcon(QIcon(":/images/go.png"));
   else
      searchButton -> setIcon(QIcon(":/images/goRTL.png"));
   searchButton -> setText(tr("Go"));
   connect (searchButton, SIGNAL(clicked()), this, SLOT(search()));

   view = new QGraphicsView(new QGraphicsScene(this), this);

   progressBar = new QProgressBar(this);
}

void ImageFetcher::createLayout()
{
   QHBoxLayout *searchLayout = new QHBoxLayout;
   searchLayout -> addWidget(searchLabel);
   searchLayout -> addWidget(searchLineEdit);
   searchLayout -> addWidget(searchButton);

   QVBoxLayout *mainLayout = new QVBoxLayout;
   mainLayout -> addLayout(searchLayout);
   mainLayout -> addWidget(view);
   mainLayout -> addWidget(progressBar);
   setLayout(mainLayout);
}

