// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef IMAGEFETCHER_H
#define IMAGEFETCHER_H

#include <QWidget>
#include <QMap>
#include <QBuffer>
#include <QGraphicsPixmapItem>
#include <QDebug>
#include <QString>

class QLabel;
class QLineEdit;
class QToolBar;
class QProgressBar;
class QGraphicsView;
class Card;
class QHttp;
class QGraphicsSceneMouseEvent;
class QToolButton;

class ImageRequest
{
   public:
      ImageRequest(const QString &f = QString(), QBuffer *b=0) { fullSizeUrl = f; buffer = b; }
//      ~ImageRequest() { delete buffer; }
      QString fullSizeUrl;
      QBuffer *buffer;
};

class GraphicsPixmapItem : public QGraphicsPixmapItem
{
   public:
      GraphicsPixmapItem(QGraphicsItem *parent = 0) : QGraphicsPixmapItem(parent) {}
      GraphicsPixmapItem(const QPixmap &pixmap, QGraphicsItem *parent = 0) : QGraphicsPixmapItem(pixmap, parent) {}

      QString getUrl() const;
      void setUrl(const QString&);

   protected:
      void mousePressEvent (QGraphicsSceneMouseEvent*);
      
   private:
      QString url;
};
   


class ImageFetcher : public QWidget
{
   Q_OBJECT

   public:
      ImageFetcher(QWidget* = 0);
      void fetchImageFor(const QString&);

   public slots:
      void stop();

   private slots:
      void search();
      void requestFinished(int, bool);
      void requestStarted(int id);
      void updateView();
      void doneSlot();
      
   private:
      void createLayout();
      void createWidgets();
      void fetchImagesFromHtml(const QString&);

      QLabel *searchLabel;
      QLineEdit *searchLineEdit;
      QToolButton *searchButton;
      QProgressBar *progressBar;
      QGraphicsView *view;
      QMap<int, ImageRequest> imageRequests;
      QHttp *http;
      int pageRequestId;
      QBuffer *pageRequestBuffer;
      QList<GraphicsPixmapItem*> images;

};


#endif

