// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "LanguagesWidget.h"
#include "LanguageDialog.h"

LanguagesWidget::LanguagesWidget(QWidget *parent)
   : QWidget(parent)
{
   setupUi(this);
}

QStringList LanguagesWidget::getLanguages() const
{
   QStringList list;
   list << firstLineEdit -> text()
        << secondLineEdit -> text()
        << (thirdLanguageCheckBox->isChecked() ? thirdLineEdit -> text() : QString())
        << (fourthLanguageCheckBox->isChecked() ? fourthLineEdit -> text() : QString())
        << (fifthLanguageCheckBox->isChecked() ? fifthLineEdit -> text() : QString());
   return list;
}

void LanguagesWidget::setLanguages(const QStringList &list)
{
   firstLineEdit -> setText(list.value(0));
   secondLineEdit -> setText(list.value(1));
   thirdLineEdit -> setText(list.value(2));
   fourthLineEdit -> setText(list.value(3));
   fifthLineEdit -> setText(list.value(4));
   
   thirdLanguageCheckBox -> setChecked(!thirdLineEdit->text().isEmpty());
   fourthLanguageCheckBox -> setChecked(!fourthLineEdit->text().isEmpty());
   fifthLanguageCheckBox -> setChecked(!fifthLineEdit->text().isEmpty());
   updateWidgets();
}

void LanguagesWidget::on_firstButton_clicked()
{
   LanguageDialog dialog(this);
   dialog.setLanguage(firstLineEdit -> text());
   if (dialog.exec())
      firstLineEdit -> setText(dialog.getLanguage());
}

void LanguagesWidget::on_secondButton_clicked()
{
   LanguageDialog dialog(this);
   dialog.setLanguage(secondLineEdit -> text());
   if (dialog.exec())
      secondLineEdit -> setText(dialog.getLanguage());
}

void LanguagesWidget::on_thirdButton_clicked()
{
   LanguageDialog dialog(this);
   dialog.setLanguage(thirdLineEdit -> text());
   if (dialog.exec())
      thirdLineEdit -> setText(dialog.getLanguage());
}

void LanguagesWidget::on_fourthButton_clicked()
{
   LanguageDialog dialog(this);
   dialog.setLanguage(fourthLineEdit -> text());
   if (dialog.exec())
      fourthLineEdit -> setText(dialog.getLanguage());
}

void LanguagesWidget::on_fifthButton_clicked()
{
   LanguageDialog dialog(this);
   dialog.setLanguage(fifthLineEdit -> text());
   if (dialog.exec())
      fifthLineEdit -> setText(dialog.getLanguage());
}

void LanguagesWidget::updateWidgets()
{
   thirdLineEdit -> setEnabled(thirdLanguageCheckBox->isChecked());
   thirdButton -> setEnabled(thirdLanguageCheckBox->isChecked());
   fourthLineEdit -> setEnabled(fourthLanguageCheckBox->isChecked());
   fourthButton -> setEnabled(fourthLanguageCheckBox->isChecked());
   fifthLineEdit -> setEnabled(fifthLanguageCheckBox->isChecked());
   fifthButton -> setEnabled(fifthLanguageCheckBox->isChecked());
}

