// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include <QtXml>
#include "MainWindow.h"
#include "FlashQard.h"
#include "SettingsDialog.h"
#include "ConfigureReviewDialog.h"
#include "Settings.h"
#include "FileReader.h"
#include "SidePanel.h"
#include "InformationDialog.h"
#include "ImportConfigure.h"
#include "ExportDialog.h"
#include "AboutDialog.h"
#include "HelpDialog.h"
#include "TranslatorWidget.h"

#define MAXRECENTFILE 5
CardBoxes* MainWindow::cardBoxes = 0;
MainWindow* MainWindow::_mainWindow = 0;

MainWindow* MainWindow::mainWindow()
{
   if (!_mainWindow)
      _mainWindow = new MainWindow();
   return _mainWindow;
}

CardBoxes* MainWindow::getCardBoxes()
{
   return cardBoxes;
}

MainWindow::MainWindow()
{
    MorfDialog *morf = new MorfDialog(this);
    morf->exec();
    delete morf;

   setupUi(this);
   setIconSize(QSize(22, 22));
   resize(Settings::settings()->windowSize);
   move(Settings::settings()->windowPosition);

   sidePanel = new SidePanel;
   connect(sidePanel, SIGNAL(sizeChanged()), this, SLOT(sidePanelSizeChangedSlot()));
   helpDialog = 0;
   cardBoxes = new CardBoxes;
   startWidget = new StartWidget;
   cardManagerWidget = new CardManagerWidget(sidePanel);
   cardReviewerWidget = new CardReviewerWidget(sidePanel);
   settingsDialog = new SettingsDialog(this);
   configureReviewDialog = new ConfigureReviewDialog(this);
   
   subStackedWidget = new QStackedWidget;
   secondMainPage = new QWidget;
   splitter = new QSplitter(Qt::Horizontal, secondMainPage);
   splitter -> addWidget(sidePanel);
   splitter -> addWidget(subStackedWidget);
   splitter -> setStretchFactor(0, 0);
   splitter -> setStretchFactor(1, 1);

   QGridLayout *secondMainLayout = new QGridLayout(secondMainPage);
   secondMainLayout -> setContentsMargins(0,0,0,0);
   secondMainLayout -> addWidget(splitter, 0, 0);

   subStackedWidget -> addWidget(cardManagerWidget);
   subStackedWidget -> addWidget(cardReviewerWidget);
   
   lastVisitedCardBox = 0;

   //clear mainStackedWidget
   while (QWidget *child = mainStackedWidget->currentWidget())
      mainStackedWidget -> removeWidget(child);
   
   mainStackedWidget -> addWidget(startWidget);
   mainStackedWidget -> addWidget(secondMainPage);
   
   connect(startWidget, SIGNAL(reviewCardsSignal(int)), this, SLOT(reviewCardsSlot(int)));
   connect(startWidget, SIGNAL(manageCardsSignal(int)), this, SLOT(manageCardsSlot(int)));
   connect(actionBackToMainPage, SIGNAL(triggered()), this, SLOT(backToMainPageSlot()));
   connect(actionManageCards, SIGNAL(triggered()), this, SLOT(goToManageCardsSlot()));
   connect(actionReviewCards, SIGNAL(triggered()), this, SLOT(goToReviewCardsSlot()));
   connect(actionConfigureReview, SIGNAL(triggered()), this, SLOT(configureReviewSlot()));
   connect(actionSave, SIGNAL(triggered()), this, SLOT(saveSlot()));
   connect(actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAsSlot()));
   connect(actionOpen, SIGNAL(triggered()), this, SLOT(openSlot()));
   connect(actionImport, SIGNAL(triggered()), this, SLOT(importSlot()));
   connect(actionExport, SIGNAL(triggered()), this, SLOT(exportSlot()));
   connect(actionNew, SIGNAL(triggered()), this, SLOT(newFileSlot()));
   connect(startWidget, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(startWidget, SIGNAL(somethingEdited()), this, SLOT(updateGoToActions()));
   connect(cardManagerWidget, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(cardReviewerWidget, SIGNAL(somethingEdited()), this, SLOT(somethingEditedSlot()));
   connect(cardReviewerWidget, SIGNAL(reviewingFinished()), this, SLOT(reviewingFinishedSlot()));
   connect(actionConfigureFlashQard, SIGNAL(triggered()), this, SLOT(configureFlashQardSlot()));
   connect(actionAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
   connect(actionAboutFlashQard, SIGNAL(triggered()), this, SLOT(aboutFlashQard()));
   connect(actionClearMenu, SIGNAL(triggered()), this, SLOT(clearMenuSlot()));
   connect(actionInformation, SIGNAL(triggered()), this, SLOT(informationSlot()));
   connect(actionHelp, SIGNAL(triggered()), this, SLOT(helpSlot()));
   connect(actionBugReport, SIGNAL(triggered()), this, SLOT(bugReportSlot()));
   connect(actionFeatureRequest, SIGNAL(triggered()), this, SLOT(featureRequestSlot()));
   connect(actionMorf_lv_website, SIGNAL(triggered()), this, SLOT(morfLvOpen()));
   setWindowModified(false);
   setCurrentFile("");
   updateGoToActions();
   if (QApplication::layoutDirection() == Qt::RightToLeft)
      actionBackToMainPage->setIcon(QIcon(":/images/backRTL.png"));
   for (int i=0; i<MAXRECENTFILE; i++)
   {
      recentFilesActions << new QAction(this);
      connect (recentFilesActions[i], SIGNAL(triggered()), this, SLOT(openRecentFileSlot()));
      menuOpenRecent -> addAction(recentFilesActions[i]);
   }
   updateOpenRecentActions();
   createSidePanelWidgets();
}

void MainWindow::morfLvOpen()
{
    QDesktopServices::openUrl(QUrl("http://morf.lv", QUrl::TolerantMode));
}

void MainWindow::createSidePanelWidgets()
{
   translatorWidget = new TranslatorWidget(this);
   connect(translatorWidget, SIGNAL(createVocabularyCardSignal(const QString&, const QString&)),
           this, SLOT(createVocabularyCardSlot(const QString&, const QString&)));
   sidePanel -> addItem(translatorWidget, tr("Translator"), QIcon(":/images/translator.png"));
}

void MainWindow::reviewCardsSlot(int boxNumber)
{
   if (boxNumber < 0 || boxNumber >= cardBoxes->size())
      return;
   bool ok = true;
   if (currentPage() == CurrentPage_Manager)
      ok = cardManagerWidget -> askToFinish();
   else if (currentPage() == CurrentPage_Reviewer)
      ok = cardReviewerWidget -> askToFinish();
   if (ok)
   {
      statusBar() -> clearMessage();
      lastVisitedCardBox = boxNumber;
      mainStackedWidget -> setCurrentWidget(secondMainPage);
      subStackedWidget -> setCurrentWidget(cardReviewerWidget);
      cardReviewerWidget -> reviewCards(cardBoxes->at(boxNumber));
      translatorWidget -> setShowCreateCardOption(false);
   }
}

void MainWindow::manageCardsSlot(int boxNumber)
{
   if (boxNumber < 0 || boxNumber >= cardBoxes->size())
      return;
   bool ok = true;
   if (currentPage() == CurrentPage_Reviewer)
      ok = cardReviewerWidget -> askToFinish();
   if (ok)
   {
      statusBar() -> clearMessage();
      lastVisitedCardBox = boxNumber;
      mainStackedWidget -> setCurrentWidget(secondMainPage);
      subStackedWidget -> setCurrentWidget(cardManagerWidget);
      cardManagerWidget -> manageCards(cardBoxes->at(boxNumber));
      translatorWidget -> setShowCreateCardOption(true);
   }
}

void MainWindow::backToMainPageSlot()
{
   bool ok = true;
   if (currentPage() == CurrentPage_Reviewer)
      ok = cardReviewerWidget -> askToFinish();
   else if (currentPage() == CurrentPage_Manager)
      ok = cardManagerWidget -> askToFinish();
   if (ok)
   {
      statusBar() -> clearMessage();
      mainStackedWidget -> setCurrentWidget(startWidget);
      startWidget->start();
      startWidget->setCurrentRow(lastVisitedCardBox);
   }
}

void MainWindow::goToManageCardsSlot()
{
   if (currentPage() == CurrentPage_Start)
      manageCardsSlot(startWidget->getCurrentRow());
   else if (currentPage() != CurrentPage_Manager)
      manageCardsSlot(lastVisitedCardBox);
}

void MainWindow::goToReviewCardsSlot()
{
   if (currentPage() == CurrentPage_Start)
      reviewCardsSlot(startWidget->getCurrentRow());
   else
      reviewCardsSlot(lastVisitedCardBox);
}

void MainWindow::configureReviewSlot()
{
   configureReviewDialog -> loadSettings();
   int result = configureReviewDialog -> exec();
   if (result == ConfigureReviewDialog_StartReview
       || (result && currentPage() == CurrentPage_Reviewer))
      goToReviewCardsSlot();
}

bool MainWindow::saveSlot()
{
   if (currentFileName.isEmpty())
      return saveAsSlot();
   else
      return saveFile(currentFileName);
}

bool MainWindow::saveAsSlot()
{
   QString fileName = askWhereToSaveFile();
   if (fileName.isEmpty())
      return false;
   else
      return saveFile(fileName);
}

QString MainWindow::askWhereToSaveFile()
{
   QString selectedFilter;
   QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"), tr("."), 
                                                   tr("FlashQard file (*.fq);;"
                                                      "CSV file (*.csv)"),
                                                   &selectedFilter);
   if (fileName.isEmpty())
      return QString();
   QString fileSuffix = QFileInfo(fileName).suffix();
   if (fileSuffix.isEmpty() || (fileSuffix.compare("fq", Qt::CaseInsensitive) && fileSuffix.compare("csv", Qt::CaseInsensitive)))
   {
      if (selectedFilter.contains("*.csv"))
         fileName += ".csv";
      else
         fileName += ".fq";
   }
   
   return fileName;
}

void MainWindow::configureFlashQardSlot()
{
   settingsDialog -> loadSettings();
   if (settingsDialog -> exec())
   {
      if (currentPage() == CurrentPage_Manager)
         cardManagerWidget -> settingsChanged();
      else if (currentPage() == CurrentPage_Reviewer)
         cardReviewerWidget -> settingsChanged();
   }
}

bool MainWindow::saveFile(const QString &fileName, FileWriterSettings conf, bool switchToNewFile)
{
   setCursor(Qt::WaitCursor);

   QString fileSuffix = QFileInfo(fileName).suffix();
   if (fileSuffix.compare("kvtml", Qt::CaseInsensitive) == 0)
   {
      QMessageBox::warning(this, tr("Writing failed"), tr("Saving KVTML file format is not supported.\n"
                                                          "Use \"Save as...\" from \"File\" menu to "
                                                          "choose another file format."));
      unsetCursor();
      return false;
   }
   else if (fileSuffix.compare("csv", Qt::CaseInsensitive) == 0)
      conf.fileFormat = FileWriterSettings::FileFormat_CSV;
   else // fq or other
      conf.fileFormat = FileWriterSettings::FileFormat_FQ;
   
   FileWriter writer;
   bool success = writer.writeToFile(fileName, cardBoxes, conf);
   unsetCursor();
   if (success && switchToNewFile)
   {
      setCurrentFile(fileName);
      setWindowModified(false);
   }
   return success;
}

bool MainWindow::askToSaveChanges()
{
   if (isWindowModified())
   {
      int answer = QMessageBox::warning(this, tr("Save"), 
                                        tr("This document has been modified.\n"
                                           "Do you want to save it?"),
                                        QMessageBox::Yes | QMessageBox::Default,
                                        QMessageBox::No,
                                        QMessageBox::Cancel | QMessageBox::Escape);
      if (answer == QMessageBox::Cancel)
         return false;
      else if (answer == QMessageBox::Yes)
         return saveSlot();
   }
   return true;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
   if (askToSaveChanges())
   {
      Settings::settings() -> save();
      event -> accept();
   }
   else
      event -> ignore();
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
   Settings::settings() -> windowSize = event->size();
}

void MainWindow::moveEvent(QMoveEvent *event)
{
   Settings::settings() -> windowPosition = event->pos();
}

void MainWindow::newFileSlot()
{
   if (!askToSaveChanges())
      return;
   cardBoxes->clear();
   setCurrentFile("");
   setWindowModified(false);
   backToMainPageSlot();
   updateGoToActions();
}

void MainWindow::openSlot(QString fileName)
{
   if (!askToSaveChanges())
      return;
   
   if (fileName.isEmpty())
      fileName = askWhichFileToOpen();
   
   if (fileName.isEmpty())
      return;
   
   setCursor(Qt::WaitCursor);
   FileReader fileReader;
   if (fileReader.readFromFile(fileName, cardBoxes, FileReaderSettings()))
   {
      setCurrentFile(fileName);
      setWindowModified(false);
      backToMainPageSlot();
      updateGoToActions();
      addToRecentFiles(fileName);
   }
   unsetCursor();
}

void MainWindow::showMessageOnStatusBar(const QString & message, int time)
{
   statusBar() -> showMessage(message, time);
}

QString MainWindow::askWhichFileToOpen()
{
   return QFileDialog::getOpenFileName(this, tr("Open"),
                                       QString(), tr("All supported files ( *.fq *.csv *.kvtml );;"
                                                     "FlashQard file (*.fq);;"
                                                     "KDE Vocabulary Document (*.kvtml);;"
                                                     "CSV file (*.csv)"));
}

void MainWindow::importSlot()
{
   QString fileName = askWhichFileToOpen();
   if (fileName.isEmpty())
      return;

   ImportConfigure config(cardBoxes, this);
   if (!config.exec())
      return;
   
   setCursor(Qt::WaitCursor);
   FileReader fileReader;
   if (fileReader.readFromFile(fileName, cardBoxes, config.getFileReaderSettings()))
   {
      backToMainPageSlot();
      updateGoToActions();
      somethingEditedSlot();
   }
   unsetCursor();
}

void MainWindow::exportSlot()
{
   QString fileName = askWhereToSaveFile();
   if (fileName.isEmpty())
      return;

   FileWriterSettings::FileFormat fileFormat = FileWriterSettings::FileFormat_FQ;
   QString fileSuffix = QFileInfo(fileName).suffix();
   if (fileSuffix.compare("csv", Qt::CaseInsensitive) == 0)
      fileFormat = FileWriterSettings::FileFormat_CSV;
   
   ExportDialog dialog(cardBoxes, fileFormat, this);
   if (!dialog.exec())
      return;
   
   saveFile(fileName, dialog.getFileWriterSettings(), false);
}

void MainWindow::informationSlot()
{
   InformationDialog dialog;
   dialog.setInformation(cardBoxes->getInformation());
   if (dialog.exec())
   {
      cardBoxes -> setInformation(dialog.getInformation());
      somethingEditedSlot();
   }
}

void MainWindow::helpSlot()
{
   if(!helpDialog)
      helpDialog = new HelpDialog(this);
   helpDialog -> show();
}

void MainWindow::bugReportSlot()
{
   QDesktopServices::openUrl(QUrl("http://www.flashqard-project.org/bugReport.php"));
}

void MainWindow::featureRequestSlot()
{
   QDesktopServices::openUrl(QUrl("http://www.flashqard-project.org/featureRequest.php"));
}

void MainWindow::createVocabularyCardSlot(const QString &translation1,
                                          const QString &translation2)
{
   cardManagerWidget -> addVocabularyCard(translation1, translation2);
}

void MainWindow::clearMenuSlot()
{
   Settings::settings() -> recentFiles.clear();
   updateOpenRecentActions();
}

void MainWindow::sidePanelSizeChangedSlot()
{
   QList<int> sizes;
   for (int i=0; i<splitter->count(); i++)
      sizes << splitter -> widget(i) -> sizeHint().width();
   splitter -> setSizes(sizes);
}

void MainWindow::updateOpenRecentActions()
{
   QStringList recentFiles = Settings::settings() -> recentFiles;
   for (int i=0; i<recentFilesActions.size(); i++)
   {
      if (i >= recentFiles.size())
         recentFilesActions[i] -> setVisible(false);
      else
      {
         recentFilesActions[i] -> setVisible(true);
         recentFilesActions[i] -> setText(recentFiles[i]);
      }
   }
}

void MainWindow::openRecentFileSlot()
{
   QAction *senderAction = static_cast<QAction*>(sender());
   openSlot(senderAction->text());
}

void MainWindow::setCurrentFile(QString fullFileName)
{
   QString shownName = tr("Untitled");
   if (!fullFileName.isEmpty())
      shownName = QFileInfo(fullFileName).fileName();
   setWindowTitle(shownName + tr("[*] - FlashQard MORF.LV edition (unofficial)"));
   currentFileName = fullFileName;
}

void MainWindow::somethingEditedSlot()
{
   setWindowModified(true);
}

void MainWindow::reviewingFinishedSlot()
{
   backToMainPageSlot();
}

void MainWindow::updateGoToActions()
{
   bool enable = cardBoxes -> size() > 0;
   actionManageCards -> setEnabled(enable);
   actionReviewCards -> setEnabled(enable);
}

void MainWindow::addToRecentFiles(const QString &fileName)
{
   int index;
   if ((index = Settings::settings()->recentFiles.indexOf(fileName)) != -1)
      Settings::settings()->recentFiles.swap(index, 0);
   else
      Settings::settings()->recentFiles.prepend(fileName);
   if (Settings::settings() -> recentFiles.size() > MAXRECENTFILE)
      Settings::settings() -> recentFiles.removeAt(MAXRECENTFILE);
   updateOpenRecentActions();
}

CurrentPage MainWindow::currentPage() const
{
   if (mainStackedWidget -> currentWidget() == startWidget)
      return CurrentPage_Start;
   else if (subStackedWidget -> currentWidget() == cardReviewerWidget)
      return CurrentPage_Reviewer;
   else
      return CurrentPage_Manager;
}

void MainWindow::aboutFlashQard()
{
   AboutDialog aboutDialog(this);
   aboutDialog.exec();
}
