// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QList>
#include <QMainWindow>
#include "StartWidget.h"
#include "CardManagerWidget.h"
#include "CardReviewerWidget.h"
#include "Card.h"
#include "ui_MainWindow.h"
#include "FileWriter.h"
#include "morfdialog.h"

class QStackedWidget;
class SettingsDialog;
class ConfigureReviewDialog;
class QSplitter;
class SidePanel;
class HelpDialog;
class TranslatorWidget;

enum CurrentPage {CurrentPage_Start, CurrentPage_Reviewer, CurrentPage_Manager};

class MainWindow : public QMainWindow, public Ui::MainWindow
{
   Q_OBJECT
   
   public:
      static MainWindow* mainWindow();
      static CardBoxes* getCardBoxes();
      
   protected:
      void closeEvent(QCloseEvent*);
      void resizeEvent(QResizeEvent*);
      void moveEvent(QMoveEvent*);
      
   public slots:
      void openSlot(QString = QString());
      void showMessageOnStatusBar(const QString&, int time = 5000);
      void morfLvOpen();

   private slots:
      void reviewCardsSlot(int);
      void manageCardsSlot(int);
      void backToMainPageSlot();
      void goToManageCardsSlot();
      void goToReviewCardsSlot();
      void configureReviewSlot();
      bool saveSlot();
      bool saveAsSlot();
      void newFileSlot();
      void aboutFlashQard();
      void configureFlashQardSlot();
      void somethingEditedSlot();
      void reviewingFinishedSlot();
      void updateGoToActions();
      void importSlot();
      void exportSlot();
      void updateOpenRecentActions();
      void openRecentFileSlot();
      void clearMenuSlot();
      void sidePanelSizeChangedSlot();
      void informationSlot();
      void helpSlot();
      void bugReportSlot();
      void featureRequestSlot();
      void createVocabularyCardSlot(const QString&, const QString&);
   
   private:
      static MainWindow *_mainWindow;
      static CardBoxes *cardBoxes;
      StartWidget *startWidget;
      CardManagerWidget *cardManagerWidget;
      CardReviewerWidget *cardReviewerWidget;
      SettingsDialog *settingsDialog;
      ConfigureReviewDialog *configureReviewDialog;
      QString currentFileName;
      int lastVisitedCardBox;
      QList<QAction*> recentFilesActions;
      SidePanel *sidePanel;
      QStackedWidget *subStackedWidget;
      QWidget *secondMainPage;
      QSplitter *splitter;
      HelpDialog *helpDialog;
      TranslatorWidget *translatorWidget;

      MainWindow();
      bool writeToFile(const QString&);
      bool askToSaveChanges();
      bool saveFile(const QString&, FileWriterSettings = FileWriterSettings(), bool = true);
      QString askWhereToSaveFile();
      QString askWhichFileToOpen();
      void setCurrentFile(QString);
      void addToRecentFiles(const QString&);
      CurrentPage currentPage() const;
      void createSidePanelWidgets();
};

#endif

