// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef PRETTYBUTTON_H
#define PRETTYBUTTON_H

#include <QAbstractButton>
class MimeData;

enum ButtonState{ButtonState_Hovered, ButtonState_Normal};

class PrettyButton : public QAbstractButton
{
   Q_OBJECT

   public:
      PrettyButton(Qt::Orientation = Qt::Horizontal, QWidget* = 0);
      void setOrientation(Qt::Orientation);
      Qt::Orientation getOrientation() const;
      void setAnimationTime(int);
      int getAnimationTime() const;
      bool isAnimating() const;
      virtual QSize sizeHint() const;

   signals:
      void hovered();
      void hovered_notDragging();
      void hovered_dragging();
      void dropMimeDataSignal(const MimeData*, Qt::DropAction);
   
   protected:
      virtual void paintEvent(QPaintEvent*);
      virtual void enterEvent(QEvent*);
      virtual void leaveEvent(QEvent*);
      virtual void dragEnterEvent(QDragEnterEvent*);      
      virtual void dragLeaveEvent(QDragLeaveEvent*);      
      virtual void mousePressEvent(QMouseEvent*);
      virtual void mouseReleaseEvent(QMouseEvent*);
      virtual void timerEvent(QTimerEvent*);
      virtual void dropEvent(QDropEvent*);
   
   private slots:
      void toggledSlot();

   private:
      Qt::Orientation orientation;
      ButtonState buttonState;
      int timerId;
      QColor normalColor;
      QColor hoveredColor;
      QColor hoveredPressedColor;
      QColor pressedColor;
      int animationPercentage;
      int animationTime;   //It's not accurate

      void mouseEntered();
      void mouseLeft();
};

#endif
