// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QSettings>
#include <QFont>
#include <QColor>
#include <QSize>
#include <QPoint>
#include <QHash>
#include "Settings.h"
#include "FlashQard.h"

Settings* Settings::_settings = 0;

Settings* Settings::settings()
{
   if (_settings)
      return _settings;
   else
      return _settings = new Settings;
}

Settings::Settings()
{
   qSettings = new QSettings("ShApp", "FlashQard");

   espeakPitch_default = 50;
   espeakSpeed_default = 120;
   espeakAccent_default = 0;

   cardAppearanceFontFamily_default = QFont().family();
   cardAppearanceZoomFactor_default = 1.5;
   cardAppearanceTextAlignment_default = (Qt::AlignCenter | Qt::AlignTop);
   cardAppearanceTextDirection_default = (Qt::LeftToRight);
   cardAppearanceForegroundColor_default = QColor("#000ce4");
   cardAppearancePageBackgroundColor_default = QColor("#f3f9f9");

   generalLanguage_default = 0;
   generalAddNewCardsTo_default= BoxSection_ActiveCards;
   generalAutoMoveFromStackToActiveCards_default = false;
   generalCardsInFirstStage_default = 15;

   generalCorrectlyAnsweredCards_default = QColor(Qt::green);
   generalWronglyAnsweredCards_default = QColor(Qt::red);
   generalToBeAnsweredCards_default = QColor(Qt::blue);
   generalNewCards_default = QColor(Qt::yellow);

   vocabularyCardsDefaultLanguages_default << QObject::tr("First Language")
                                           << QObject::tr("Second Language");

   reviewCardsNumber_default = 20;
   reviewAutoPlay_default = false;
   reviewShowHints_default = true;
   reviewShowHintsOnlyOnVerify_default = false;
   reviewShowCardsList_default = true;
   reviewExtractTitles_default = true;

   reviewVocabularyCustomizedLanguage_default = false;
   reviewVocabularyFromLanguage_default = QObject::tr("First Language");
   reviewVocabularyToLanguage_default = QObject::tr("Second Language");

   reviewRegularFrontToBack_default = true;

   reviewMarkCards_default = true;
   reviewNavigation_default = Navigation_SelectNextMarkedCard;

   translatorDatabases_default = QHash<QString, QVariant>();
   translatorServiceGoogle_default = false;
   translatorServiceGoogleSourceLanguage_default = "en";
   translatorServiceGoogleTargetLanguage_default = "fa";
}

void Settings::save()
{
   qSettings -> setValue("windowSize", windowSize);
   qSettings -> setValue("windowPosition", windowPosition);

   qSettings -> setValue("recentFiles", recentFiles);

   qSettings -> setValue("espeakPitch", espeakPitch);
   qSettings -> setValue("espeakSpeed", espeakSpeed);
   qSettings -> setValue("espeakAccent", espeakAccent);
   
   qSettings -> setValue("cardAppearanceFontFamily", cardAppearanceFontFamily);
   qSettings -> setValue("cardAppearanceZoomFactor", cardAppearanceZoomFactor);
   qSettings -> setValue("cardAppearanceTextAlignment", cardAppearanceTextAlignment | Qt::AlignTop);
   qSettings -> setValue("cardAppearanceTextDirection", cardAppearanceTextDirection);
   qSettings -> setValue("cardAppearanceForegroundColor", cardAppearanceForegroundColor);
   qSettings -> setValue("cardAppearancePageBackgroundColor", cardAppearancePageBackgroundColor);

   qSettings -> setValue("generalLanguage", generalLanguage);
   qSettings -> setValue("generalAddNewCardsTo", generalAddNewCardsTo);
   qSettings -> setValue("generalAutoMoveFromStackToActiveCards", generalAutoMoveFromStackToActiveCards);
   qSettings -> setValue("generalCardsInFirstStage", generalCardsInFirstStage);
   qSettings -> setValue("generalCorrectlyAnsweredCards", generalCorrectlyAnsweredCards);
   qSettings -> setValue("generalWronglyAnsweredCards", generalWronglyAnsweredCards);
   qSettings -> setValue("generalToBeAnsweredCards", generalToBeAnsweredCards);
   qSettings -> setValue("generalNewCards", generalNewCards);

   qSettings -> setValue("vocabularyCardsDefaultLanguages", vocabularyCardsDefaultLanguages);

   qSettings -> setValue("reviewCardsNumber", reviewCardsNumber);
   qSettings -> setValue("reviewAutoPlay", reviewAutoPlay);
   qSettings -> setValue("reviewShowHints", reviewShowHints);
   qSettings -> setValue("reviewShowHintsOnlyOnVerify", reviewShowHintsOnlyOnVerify);
   qSettings -> setValue("reviewShowCardsList", reviewShowCardsList);
   qSettings -> setValue("reviewExtractTitles", reviewExtractTitles);

   qSettings -> setValue("reviewVocabularyCustomizedLanguage", reviewVocabularyCustomizedLanguage);
   qSettings -> setValue("reviewVocabularyFromLanguage", reviewVocabularyFromLanguage);
   qSettings -> setValue("reviewVocabularyToLanguage", reviewVocabularyToLanguage);

   qSettings -> setValue("reviewRegularFrontToBack", reviewRegularFrontToBack);

   qSettings -> setValue("reviewMarkCards", reviewMarkCards);
   qSettings -> setValue("reviewNavigation", reviewNavigation);

   qSettings -> setValue("translatorDatabases", translatorDatabases);

   qSettings -> setValue("translatorServiceGoogle", translatorServiceGoogle);
   qSettings -> setValue("translatorServiceGoogleSourceLanguage", translatorServiceGoogleSourceLanguage);
   qSettings -> setValue("translatorServiceGoogleTargetLanguage", translatorServiceGoogleTargetLanguage);

   qSettings -> setValue("versionOfLastUsedFlashQard", VERSION);

   qDebug("Saved settings to %s", qSettings->fileName().toAscii().constData());
}

void Settings::load()
{
   windowSize = qSettings -> value("windowSize", QSize(1000, 700)).toSize();
   windowPosition = qSettings -> value("windowPosition", QPoint(0,0)).toPoint();

   recentFiles = qSettings -> value("recentFiles", QStringList()).toStringList();

   espeakPitch = qSettings -> value("espeakPitch", espeakPitch_default).toInt();
   espeakSpeed = qSettings -> value("espeakSpeed", espeakSpeed_default).toInt();
   espeakAccent = qSettings -> value("espeakAccent", espeakAccent_default).toInt();
   
   cardAppearanceFontFamily = qSettings -> value("cardAppearanceFontFamily", cardAppearanceFontFamily_default).toString();
   cardAppearanceZoomFactor = qSettings -> value("cardAppearanceZoomFactor", cardAppearanceZoomFactor_default).toDouble();
   cardAppearanceTextAlignment = qSettings -> value("cardAppearanceTextAlignment",
                                            cardAppearanceTextAlignment_default | Qt::AlignTop).toInt();
   cardAppearanceTextDirection = qSettings -> value("cardAppearanceTextDirection",
                                            cardAppearanceTextDirection_default).toInt();
   cardAppearanceForegroundColor = qSettings -> value("cardAppearanceForegroundColor", cardAppearanceForegroundColor_default).value<QColor>();
   cardAppearancePageBackgroundColor = qSettings -> value("cardAppearancePageBackgroundColor", cardAppearancePageBackgroundColor_default).value<QColor>();

   generalLanguage = qSettings -> value("generalLanguage", generalLanguage_default).toInt();
   generalAddNewCardsTo = static_cast<BoxSection>(qSettings -> value("generalAddNewCardsTo", generalAddNewCardsTo_default).toInt());
   generalAutoMoveFromStackToActiveCards = qSettings -> value(
      "generalAutoMoveFromStackToActiveCards",
      generalAutoMoveFromStackToActiveCards_default).toBool();
   generalCardsInFirstStage = qSettings -> value("generalCardsInFirstStage", generalCardsInFirstStage_default).toInt();
   generalCorrectlyAnsweredCards = qSettings -> value("generalCorrectlyAnsweredCards", generalCorrectlyAnsweredCards_default).value<QColor>();
   generalWronglyAnsweredCards = qSettings -> value("generalWronglyAnsweredCards", generalWronglyAnsweredCards_default).value<QColor>();
   generalToBeAnsweredCards = qSettings -> value("generalToBeAnsweredCards", generalToBeAnsweredCards_default).value<QColor>();
   generalNewCards = qSettings -> value("generalNewCards", generalNewCards_default).value<QColor>();

   vocabularyCardsDefaultLanguages = qSettings -> value("vocabularyCardsDefaultLanguages",
                                                        vocabularyCardsDefaultLanguages_default).toStringList();

   reviewCardsNumber = qSettings -> value("reviewCardsNumber", reviewCardsNumber_default).toInt();
   reviewAutoPlay = qSettings -> value("reviewAutoPlay", reviewAutoPlay_default).toBool();
   reviewShowHints = qSettings -> value("reviewShowHints", reviewShowHints_default).toBool();
   reviewShowHintsOnlyOnVerify = qSettings -> value("reviewShowHintsOnlyOnVerify", reviewShowHintsOnlyOnVerify_default).toBool();
   reviewShowCardsList = qSettings -> value("reviewShowCardsList", reviewShowCardsList_default).toBool();
   reviewExtractTitles = qSettings -> value("reviewExtractTitles", reviewExtractTitles_default).toBool();

   reviewVocabularyCustomizedLanguage = qSettings -> value("reviewVocabularyCustomizedLanguage",
                                                           reviewVocabularyCustomizedLanguage_default).toBool();
   reviewVocabularyFromLanguage = qSettings -> value("reviewVocabularyFromLanguage",
                                                     reviewVocabularyFromLanguage_default).toString();
   reviewVocabularyToLanguage = qSettings -> value("reviewVocabularyToLanguage",
                                                   reviewVocabularyToLanguage_default).toString();

   reviewRegularFrontToBack = qSettings -> value("reviewRegularFrontToBack",
                                                 reviewRegularFrontToBack_default).toBool();

   reviewMarkCards = qSettings -> value("reviewMarkCards", reviewMarkCards_default).toBool();
   reviewNavigation = static_cast<Navigation>(qSettings -> value("reviewNavigation", reviewNavigation_default).toInt());

   translatorDatabases = qSettings -> value("translatorDatabases", translatorDatabases_default).toHash();

   translatorServiceGoogle = qSettings -> value("translatorServiceGoogle", translatorServiceGoogle_default).toBool();
   translatorServiceGoogleSourceLanguage = qSettings ->
      value("translatorServiceGoogleSourceLanguage", translatorServiceGoogleSourceLanguage_default).toString();
   translatorServiceGoogleTargetLanguage = qSettings ->
      value("translatorServiceGoogleTargetLanguage", translatorServiceGoogleTargetLanguage_default).toString();

   versionOfLastUsedFlashQard = qSettings -> value("versionOfLastUsedFlashQard", "").toString();

   if (versionOfLastUsedFlashQard != VERSION)
   {
      overwriteSettingsForNewVersion();
      qDebug("This is the first run of this version");
   }
   qDebug("Loaded settings from %s", qSettings->fileName().toAscii().constData());
}

QString Settings::makeCss() const
{
   int textAlignmentInt = cardAppearanceTextAlignment;
   QString textAlignmentString = textAlignmentInt&Qt::AlignLeft ? "left"
                               : textAlignmentInt&Qt::AlignHCenter ? "center"
                               : textAlignmentInt&Qt::AlignRight ? "right"
                               : "justify";

   QString css = QString("body { font-family: %1; color: %3; background-color: %4; text-align: %5; direction: %6;} "
                         "p {margin-top: 5px; margin-bottom: 5px;}")
      .arg(cardAppearanceFontFamily)
      .arg(cardAppearanceForegroundColor.name())
      .arg(cardAppearancePageBackgroundColor.name())
      .arg(textAlignmentString)
      .arg(cardAppearanceTextDirection == 0 ? "ltr" : "rtl");

   return css;
}

void Settings::overwriteSettingsForNewVersion()
{
   QStringList lastVersion = versionOfLastUsedFlashQard.split(".", QString::SkipEmptyParts);
   if (lastVersion.value(0).toInt() == 0
       && lastVersion.value(1).toInt() < 14)
   {
      qDebug("overwriting some settings for the new version");
      generalAddNewCardsTo = BoxSection_ActiveCards;
      cardAppearanceForegroundColor = QColor("#000ce4");
      cardAppearanceZoomFactor = 1.5;
   }
}

