// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef SETTINGS_H
#define SETTINGS_H

#include "Card.h"
#include "CardReviewerWidget.h"
class QSettings;
class QColor;
class QSize;
class QPoint;
class QVariant;

class Settings
{
   public:
      static Settings* settings();
      void save();
      void load();

      QString makeCss() const;

      // Variables:
      QSize windowSize;
      QPoint windowPosition;

      QStringList recentFiles;
      
      int espeakPitch;
      int espeakPitch_default;
      int espeakSpeed;
      int espeakSpeed_default;
      int espeakAccent;
      int espeakAccent_default;

      QString cardAppearanceFontFamily;
      QString cardAppearanceFontFamily_default;
      //font size range is between 1 to 7
      double cardAppearanceZoomFactor;
      double cardAppearanceZoomFactor_default;
      int cardAppearanceTextAlignment;
      int cardAppearanceTextAlignment_default;
      int cardAppearanceTextDirection;
      int cardAppearanceTextDirection_default;
      QColor cardAppearanceForegroundColor;
      QColor cardAppearanceForegroundColor_default;
      QColor cardAppearancePageBackgroundColor;
      QColor cardAppearancePageBackgroundColor_default;

      int generalLanguage;
      int generalLanguage_default;
      BoxSection generalAddNewCardsTo;
      BoxSection generalAddNewCardsTo_default;
      bool generalAutoMoveFromStackToActiveCards;
      bool generalAutoMoveFromStackToActiveCards_default;
      int generalCardsInFirstStage;
      int generalCardsInFirstStage_default;
      QColor generalCorrectlyAnsweredCards;
      QColor generalCorrectlyAnsweredCards_default;
      QColor generalWronglyAnsweredCards;
      QColor generalWronglyAnsweredCards_default;
      QColor generalToBeAnsweredCards;
      QColor generalToBeAnsweredCards_default;
      QColor generalNewCards;
      QColor generalNewCards_default;

      QStringList vocabularyCardsDefaultLanguages;
      QStringList vocabularyCardsDefaultLanguages_default;

      int reviewCardsNumber;
      int reviewCardsNumber_default;
      bool reviewAutoPlay;
      bool reviewAutoPlay_default;
      bool reviewShowHints;
      bool reviewShowHints_default;
      bool reviewShowHintsOnlyOnVerify;
      bool reviewShowHintsOnlyOnVerify_default;
      bool reviewShowCardsList;
      bool reviewShowCardsList_default;
      bool reviewExtractTitles;
      bool reviewExtractTitles_default;

      bool reviewVocabularyCustomizedLanguage;
      bool reviewVocabularyCustomizedLanguage_default;
      QString reviewVocabularyFromLanguage;
      QString reviewVocabularyFromLanguage_default;
      QString reviewVocabularyToLanguage;
      QString reviewVocabularyToLanguage_default;

      bool reviewRegularFrontToBack;
      bool reviewRegularFrontToBack_default;

      bool reviewMarkCards;
      bool reviewMarkCards_default;
      Navigation reviewNavigation;
      Navigation reviewNavigation_default;

      QHash<QString, QVariant> translatorDatabases;
      QHash<QString, QVariant> translatorDatabases_default;

      bool translatorServiceGoogle;
      bool translatorServiceGoogle_default;
      QString translatorServiceGoogleSourceLanguage;
      QString translatorServiceGoogleSourceLanguage_default;
      QString translatorServiceGoogleTargetLanguage;
      QString translatorServiceGoogleTargetLanguage_default;

      QString versionOfLastUsedFlashQard;

   private:
      Settings();
      void overwriteSettingsForNewVersion();
   
      QSettings *qSettings;
      static Settings *_settings;
};


#endif

