// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "SettingsCardAppearance.h"
#include "Settings.h"
#include "Utilities.h"
#include <QFontDialog>
#include <QColorDialog>

SettingsCardAppearance::SettingsCardAppearance(QWidget *parent)
   : SettingsWidget(parent)
{
   setupUi(this);
   connect(textEditor, SIGNAL(selectionChanged()), this, SLOT(updateButtons()));
}

void SettingsCardAppearance::saveSettings()
{
   selectAll();
   Settings::settings() -> cardAppearanceFontFamily = fontFamilyComboBox->currentFont().family();
   Settings::settings() -> cardAppearanceZoomFactor = zoomSpinBox -> value();
   Settings::settings() -> cardAppearanceTextAlignment = textEditor -> getAlignment();
   Settings::settings() -> cardAppearanceTextDirection = textEditor ->
      getAction(QWebPage::SetTextDirectionLeftToRight) -> isChecked() ? Qt::LeftToRight : Qt::RightToLeft;
   Settings::settings() -> cardAppearanceForegroundColor = textEditor -> getForegroundColor();
   Settings::settings() -> cardAppearancePageBackgroundColor = textEditor -> getPageBackgroundColor();
}

void SettingsCardAppearance::loadSettings()
{

#if QT_VERSION >= 0x040502
   selectAll();
#else
   textEditor -> startEditing();
#endif

   QFont font (Settings::settings() -> cardAppearanceFontFamily);
   fontFamilyComboBox -> setCurrentFont(font);
   zoomSpinBox -> setValue(Settings::settings() -> cardAppearanceZoomFactor);

   Document doc;
   doc.setText("<p>" + tr("This is only a preview...") + "</p>");
   textEditor -> setDocument(doc);

   updateButtons();
}

void SettingsCardAppearance::loadDefaults()
{
   Document doc;
   doc.setText("<p>" + tr("This is only a preview...") + "</p>");
   textEditor -> setDocument(doc);

#if QT_VERSION < 0x040502
   textEditor -> startEditing();
#endif

   selectAll();

   QFont font (Settings::settings() -> cardAppearanceFontFamily_default);
   fontFamilyComboBox -> setCurrentFont(font);
   zoomSpinBox -> setValue(Settings::settings() -> cardAppearanceZoomFactor_default);

   textEditor -> setFontFamily(font.family());
   textEditor -> setAlignment(static_cast<Qt::Alignment>(Settings::settings() -> cardAppearanceTextAlignment_default));
   textEditor -> setForegroundColor(Settings::settings() -> cardAppearanceForegroundColor_default);
   textEditor -> setPageBackgroundColor(Settings::settings() -> cardAppearancePageBackgroundColor_default);
   if (Settings::settings() -> cardAppearanceTextDirection_default == Qt::LeftToRight)
      textEditor -> getAction(QWebPage::SetTextDirectionLeftToRight) -> trigger();
   else
      textEditor -> getAction(QWebPage::SetTextDirectionRightToLeft) -> trigger();

   updateButtons();
}

void SettingsCardAppearance::updateButtons()
{
   alignLeftButton -> setChecked(textEditor -> getAlignment() & Qt::AlignLeft);
   alignRightButton -> setChecked(textEditor -> getAlignment() & Qt::AlignRight);
   alignCenterButton -> setChecked(textEditor -> getAlignment() & Qt::AlignCenter);
   alignJustifyButton -> setChecked(textEditor -> getAlignment() & Qt::AlignJustify);

   leftToRightButton -> setChecked(textEditor -> getAction(QWebPage::SetTextDirectionLeftToRight) -> isChecked());
   rightToLeftButton -> setChecked(textEditor -> getAction(QWebPage::SetTextDirectionRightToLeft) -> isChecked());
}

void SettingsCardAppearance::selectAll()
{
#if QT_VERSION >= 0x040502
   textEditor -> startEditing();
   textEditor -> getAction(QWebPage::SelectAll) -> trigger();
#else
   textEditor -> getAction(QWebPage::MoveToEndOfDocument) -> trigger();
   textEditor -> getAction(QWebPage::MoveToStartOfDocument) -> trigger();
#endif   

}

void SettingsCardAppearance::on_fontFamilyComboBox_activated()
{
   selectAll();
   textEditor -> setFontFamily(fontFamilyComboBox -> currentFont().family());
}

void SettingsCardAppearance::on_zoomSpinBox_valueChanged(double newValue)
{
   textEditor -> setZoomFactor(newValue);
   zoomSlider -> setValue(newValue*10.0);
}

void SettingsCardAppearance::on_zoomSlider_valueChanged(int newValue)
{
   zoomSpinBox -> setValue(newValue/10.0);
}

void SettingsCardAppearance::on_alignLeftButton_clicked()
{
   selectAll();
   textEditor -> setAlignment(Qt::AlignLeft);
   updateButtons();
}

void SettingsCardAppearance::on_alignCenterButton_clicked()
{
   selectAll();
   textEditor -> setAlignment(Qt::AlignCenter);
   updateButtons();
}

void SettingsCardAppearance::on_alignRightButton_clicked()
{
   selectAll();
   textEditor -> setAlignment(Qt::AlignRight);
   updateButtons();
}

void SettingsCardAppearance::on_alignJustifyButton_clicked()
{
   selectAll();
   textEditor -> setAlignment(Qt::AlignJustify);
   updateButtons();
}

void SettingsCardAppearance::on_foregroundColorButton_clicked()
{
   selectAll();
   QColor color = QColorDialog::getColor(textEditor -> getForegroundColor(), this);
   if (color.isValid())
      textEditor->setForegroundColor(color);
   updateButtons();
}

void SettingsCardAppearance::on_pageBackgroundColorButton_clicked()
{
   QColor color = QColorDialog::getColor(textEditor->getPageBackgroundColor(), this);
   if (color.isValid())
      textEditor->setPageBackgroundColor(color);
   updateButtons();
}

void SettingsCardAppearance::on_leftToRightButton_clicked()
{
   selectAll();
   QAction *action = textEditor -> getAction(QWebPage::SetTextDirectionLeftToRight);
   if (!action -> isChecked())
      action -> trigger();
   updateButtons();
}

void SettingsCardAppearance::on_rightToLeftButton_clicked()
{
   selectAll();
   QAction *action = textEditor -> getAction(QWebPage::SetTextDirectionRightToLeft);
   if (!action -> isChecked())
      action -> trigger();
   updateButtons();
}



