// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "SettingsDialog.h"
#include "SettingsTextToSpeech.h"
#include "SettingsCardAppearance.h"
#include "SettingsGeneral.h"
#include "SettingsVocabularyCards.h"

SettingsDialog::SettingsDialog(QWidget *parent)
   : QDialog(parent)
{
   setupUi(this);
   createPages();
   connect(sectionsListWidget, SIGNAL(currentRowChanged(int)), this, SLOT(currentRowChangedSlot(int)));
      
   sectionsListWidget -> setCurrentRow(0);
}

void SettingsDialog::createPages()
{
   pagesStackedWidget -> removeWidget(pagesStackedWidget->currentWidget()); //clear stacked widget
   
   sectionsListWidget ->addItem(tr("General"));
   pagesStackedWidget -> addWidget(new SettingsGeneral(this));
   sectionsListWidget -> addItem(tr("Text-to-Speech"));
   pagesStackedWidget -> addWidget(new SettingsTextToSpeech(this));
   sectionsListWidget -> addItem(tr("Card Appearance"));
   pagesStackedWidget -> addWidget(new SettingsCardAppearance(this));
   sectionsListWidget -> addItem(tr("Vocabulary Cards"));
   pagesStackedWidget -> addWidget(new SettingsVocabularyCards(this));
}

void SettingsDialog::loadSettings()
{
   for (int i=0; i<pagesStackedWidget->count(); i++)
      static_cast<SettingsWidget*>(pagesStackedWidget->widget(i)) -> loadSettings();
}

void SettingsDialog::currentRowChangedSlot(int currentRow)
{
   if (currentRow == -1)
      sectionsListWidget -> setCurrentRow(0);
   else
      pagesStackedWidget -> setCurrentIndex(currentRow);
}

void SettingsDialog::on_okButton_clicked()
{
   for (int i=0; i<pagesStackedWidget->count(); i++)
      static_cast<SettingsWidget*>(pagesStackedWidget->widget(i)) -> saveSettings();
   accept();
}

void SettingsDialog::on_applyButton_clicked()
{
   for (int i=0; i<pagesStackedWidget->count(); i++)
      static_cast<SettingsWidget*>(pagesStackedWidget->widget(i)) -> saveSettings();
}

void SettingsDialog::on_cancelButton_clicked()
{
   reject();
}

void SettingsDialog::on_defaultsButton_clicked()
{
   static_cast<SettingsWidget*>(pagesStackedWidget -> currentWidget()) -> loadDefaults();
}

void SettingsDialog::on_defaultsForAllButton_clicked()
{
   for (int i=0; i<pagesStackedWidget->count(); i++)
      static_cast<SettingsWidget*>(pagesStackedWidget->widget(i)) -> loadDefaults();
}

   

