// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QColorDialog>
#include "SettingsGeneral.h"
#include "Settings.h"

SettingsGeneral::SettingsGeneral(QWidget *parent)
   : SettingsWidget(parent)
{
   setupUi(this);

}

void SettingsGeneral::saveSettings()
{
   Settings::settings() -> generalLanguage = languageComboBox -> currentIndex();
   Settings::settings() -> generalAddNewCardsTo = newCardsComboBox -> currentIndex() == 0
                           ? BoxSection_Stack : BoxSection_ActiveCards;
   Settings::settings() -> generalAutoMoveFromStackToActiveCards = autoMoveFromStackToActiveCardsCheckBox
                           -> isChecked();
   Settings::settings() -> generalCardsInFirstStage = cardsInFirstStageSpinBox -> value();
   Settings::settings() -> generalCorrectlyAnsweredCards = correctlyAnsweredCards;
   Settings::settings() -> generalWronglyAnsweredCards = wronglyAnsweredCards;
   Settings::settings() -> generalToBeAnsweredCards = toBeAnsweredCards;
   Settings::settings() -> generalNewCards = newCards;
}

void SettingsGeneral::loadSettings()
{
   languageComboBox -> setCurrentIndex(Settings::settings() -> generalLanguage);
   newCardsComboBox -> setCurrentIndex(Settings::settings() -> generalAddNewCardsTo == BoxSection_Stack ? 0 : 1);
   autoMoveFromStackToActiveCardsCheckBox -> setChecked(Settings::settings() -> generalAutoMoveFromStackToActiveCards);
   cardsInFirstStageSpinBox -> setValue(Settings::settings() -> generalCardsInFirstStage);
   correctlyAnsweredCards = Settings::settings() -> generalCorrectlyAnsweredCards;
   wronglyAnsweredCards = Settings::settings() -> generalWronglyAnsweredCards;
   toBeAnsweredCards = Settings::settings() -> generalToBeAnsweredCards;
   newCards = Settings::settings() -> generalNewCards;

   updateIcons();
}

void SettingsGeneral::loadDefaults()
{
   languageComboBox -> setCurrentIndex(Settings::settings() -> generalLanguage_default);
   newCardsComboBox -> setCurrentIndex(Settings::settings() -> generalAddNewCardsTo_default == BoxSection_Stack ? 0 : 1);
   autoMoveFromStackToActiveCardsCheckBox -> setChecked(Settings::settings() -> generalAutoMoveFromStackToActiveCards_default);
   cardsInFirstStageSpinBox -> setValue(Settings::settings() -> generalCardsInFirstStage_default);
   correctlyAnsweredCards = Settings::settings() -> generalCorrectlyAnsweredCards_default;
   wronglyAnsweredCards = Settings::settings() -> generalWronglyAnsweredCards_default;
   toBeAnsweredCards = Settings::settings() -> generalToBeAnsweredCards_default;
   newCards = Settings::settings() -> generalNewCards_default;

   updateIcons();
}

void SettingsGeneral::updateIcons()
{
   QPixmap pixmap (32, 32);
   pixmap.fill(correctlyAnsweredCards);
   correctlyAnsweredButton->setIcon(QIcon(pixmap));

   pixmap.fill(wronglyAnsweredCards);
   wronglyAnsweredButton->setIcon(QIcon(pixmap));

   pixmap.fill(toBeAnsweredCards);
   toBeAnsweredButton->setIcon(QIcon(pixmap));

   pixmap.fill(newCards);
   newCardsButton->setIcon(QIcon(pixmap));

}

void SettingsGeneral::on_correctlyAnsweredButton_clicked()
{
   QColor color = QColorDialog::getColor(correctlyAnsweredCards, this);
   if (color.isValid())
      correctlyAnsweredCards = color;
   updateIcons();
}

void SettingsGeneral::on_wronglyAnsweredButton_clicked()
{
   QColor color = QColorDialog::getColor(wronglyAnsweredCards, this);
   if (color.isValid())
      wronglyAnsweredCards = color;
   updateIcons();
}

void SettingsGeneral::on_toBeAnsweredButton_clicked()
{
   QColor color = QColorDialog::getColor(toBeAnsweredCards, this);
   if (color.isValid())
      toBeAnsweredCards = color;
   updateIcons();
}

void SettingsGeneral::on_newCardsButton_clicked()
{
   QColor color = QColorDialog::getColor(newCards, this);
   if (color.isValid())
      newCards = color;
   updateIcons();
}


