// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "SidePanel.h"
#include <QStackedWidget>
#include <QString>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QList>

// SidePanelButton::SidePanelButton(QWidget *parent)
//    : PrettyButton(Qt::Vertical, parent)
// {
//    initialize();
// }

// SidePanelButton::SidePanelButton(const QIcon &ci, const QIcon &ui, const QString &t, QWidget *parent)
//    : PrettyButton(Qt::Vertical, parent)
// {
//    setCheckedIcon(ci);
//    setUncheckedIcon(ui);
//    setText(t);
//    initialize();
//    updateIcon();
// }

// void SidePanelButton::initialize()
// {
//    setCheckable(true);
//    connect(this, SIGNAL(toggled(bool)), this, SLOT(updateIcon()));
// }

// void SidePanelButton::setCheckedIcon(const QIcon &ci)
// {
//    checkedIcon = ci;
//    updateIcon();
// }

// void SidePanelButton::setUncheckedIcon(const QIcon &ui)
// {
//    unCheckedIcon = ui;
//    updateIcon();
// }

// void SidePanelButton::updateIcon()
// {
//    if (isChecked())
//       setIcon(checkedIcon);
//    else
//       setIcon(unCheckedIcon);
// }

//======================================================================

SidePanel::SidePanel(QWidget *parent)
   : QWidget(parent)
{
   verticalLayout = new QVBoxLayout;
   stackedWidget = new QStackedWidget(this);
   stackedWidget -> hide();

   QHBoxLayout *mainLayout = new QHBoxLayout;
   mainLayout -> addLayout(verticalLayout);
   mainLayout -> addWidget(stackedWidget);
   mainLayout -> setContentsMargins(0,0,0,0);
   setLayout(mainLayout);
}

SidePanel::~SidePanel()
{
   for (int i=0; i<buttons.size(); i++)
      delete buttons[i];
   delete stackedWidget;
}

void SidePanel::addItem(QWidget *widget, const QString &text, const QIcon &icon)
{
   PrettyButton *button = new PrettyButton(Qt::Vertical, this);
   button -> setCheckable(true);
   button -> setText(text);
   button -> setIcon(icon);
   buttons << button;
   verticalLayout -> addWidget(button);
   connect(button, SIGNAL(clicked()), this, SLOT(clickedSlot()));
   button -> setSizePolicy(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding);
   stackedWidget -> addWidget(widget);
}

void SidePanel::clickedSlot()
{
   PrettyButton *clickedButton = static_cast<PrettyButton*>(sender());
   if (clickedButton -> isChecked())
      setCurrentIndex(buttons.indexOf(clickedButton));
   else
      syncActionsAndWidgets();
}

void SidePanel::syncActionsAndWidgets()
{
   int index = 0;
   for (; index<buttons.size(); index++)
      if (buttons[index] -> isChecked())
         break;
   if (index >= buttons.size()) //if not found
      stackedWidget -> hide();
   else
   {
      stackedWidget -> setCurrentIndex(index);
      stackedWidget -> show();
   }
   emit sizeChanged();
}

void SidePanel::removeItem(int index)
{
   if (index > 0 && index < buttons.size())
   {
      verticalLayout -> removeWidget(buttons[index]);
      delete buttons.takeAt(index);
      stackedWidget -> removeWidget(stackedWidget->widget(index));
      syncActionsAndWidgets();
   }
}

void SidePanel::setCurrentIndex(int index)
{
   for (int i=0; i<buttons.size(); i++)
      buttons[i] -> setChecked(i == index);
   syncActionsAndWidgets();
}

void SidePanel::setCurrentWidget(QWidget *widget)
{
   setCurrentIndex(indexOf(widget));
}

void SidePanel::hideCurrentWidget()
{
   setCurrentIndex(-1);
}

void SidePanel::showItemOfWidget(QWidget *widget)
{
   int index = indexOf(widget);
   buttons[index] -> setVisible(true);
}

void SidePanel::hideItemOfWidget(QWidget *widget)
{
   int index = indexOf(widget);
   if (buttons[index] -> isChecked())
      hideCurrentWidget();
   buttons[index] -> setVisible(false);
}

int SidePanel::indexOf(QWidget *widget) const
{
   return stackedWidget -> indexOf(widget);
}

QWidget* SidePanel::currentWidget() const
{
   if (stackedWidget -> isVisible())
      return stackedWidget -> currentWidget();
   else
      return 0;
}

int SidePanel::currentIndex() const
{
   if (stackedWidget -> isVisible())
      return stackedWidget -> currentIndex();
   else
      return -1;
}

void SidePanel::setItemText(int index, const QString &text)
{
   if (index > 0 && index < buttons.size())
      buttons[index] -> setText(text);
}

void SidePanel::setItemIcon(int index, const QIcon &icon)
{
   if (index > 0 && index < buttons.size())
      buttons[index] -> setIcon(icon);
}

void SidePanel::setItemShortcut(int index, const QKeySequence &shortcut)
{
   //doesnt work properly!
   if (index > 0 && index < buttons.size())
   {
      buttons[index] -> setShortcut(shortcut);
      buttons[index] -> setToolTip(buttons[index]->text() + tr(" (") + buttons[index]->shortcut().toString() + tr(")"));
   }
}

QSize SidePanel::sizeHint() const
{
   if (stackedWidget -> isVisible())
      return QSize(verticalLayout->sizeHint().width() + stackedWidget->currentWidget()->sizeHint().width() + 10,
                   verticalLayout->sizeHint().height());
   else
      return verticalLayout->sizeHint();
}


