// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include "StartWidget.h"
#include "MainWindow.h"
#include "Card.h"

StartWidget::StartWidget(QWidget *parent)
   :QWidget(parent)
{
   createActionsAndWidgets();
   createLayouts();
   updateList();
   updateActionsAndButtons();
}

void StartWidget::createActionsAndWidgets()
{
   addBoxAction = new QAction(QIcon(":/images/addCard.png"),tr("Add"), this);
   addBoxAction -> setShortcut(Qt::CTRL + Qt::Key_D);
   addBoxAction -> setToolTip(addBoxAction->toolTip() + tr(" (") + addBoxAction->shortcut().toString() + tr(")"));
   connect(addBoxAction, SIGNAL(triggered()), this, SLOT(addBoxSlot()));

   removeBoxAction = new QAction(QIcon(":/images/removeCard.png"), tr("Remove"), this);
   removeBoxAction -> setShortcut(Qt::CTRL + Qt::Key_R);
   removeBoxAction -> setToolTip(removeBoxAction->toolTip() + tr(" (") + removeBoxAction->shortcut().toString() + tr(")"));
   connect(removeBoxAction, SIGNAL(triggered()), this, SLOT(removeBoxSlot()));

   renameBoxAction = new QAction(QIcon(":/images/advancedEditor.png"), tr("Rename"), this);
   renameBoxAction -> setShortcut(Qt::CTRL + Qt::Key_E);
   renameBoxAction -> setToolTip(renameBoxAction->toolTip() + tr(" (") + renameBoxAction->shortcut().toString() + tr(")"));
   connect(renameBoxAction, SIGNAL(triggered()), this, SLOT(renameBoxSlot()));

   moveUpAction = new QAction(QIcon(":/images/moveUp.png"), tr("Move up"), this);
   moveUpAction -> setShortcut(Qt::CTRL + Qt::Key_U);
   moveUpAction -> setToolTip(moveUpAction->toolTip() + tr(" (") + moveUpAction->shortcut().toString() + tr(") "));
   connect(moveUpAction, SIGNAL(triggered()), this, SLOT(moveUpSlot()));

   moveDownAction = new QAction(QIcon(":/images/moveDown.png"), tr("Move down"), this);
   moveDownAction -> setShortcut(Qt::CTRL + Qt::Key_I);
   moveDownAction -> setToolTip(moveDownAction->toolTip() + tr(" (") + moveDownAction->shortcut().toString() + tr(") "));
   connect(moveDownAction, SIGNAL(triggered()), this, SLOT(moveDownSlot()));

   resetStatisticsCurrentBoxAction = new QAction(QIcon(":/images/reset.png"), tr("Reset selected category"), this);
   connect(resetStatisticsCurrentBoxAction, SIGNAL(triggered()), this, SLOT(resetStatisticsCurrentBoxSlot()));

   resetStatisticsAllBoxesAction = new QAction(QIcon(":/images/reset.png"), tr("Reset all categories"), this);
   connect(resetStatisticsAllBoxesAction, SIGNAL(triggered()), this, SLOT(resetStatisticsAllBoxesSlot()));

   QMenu *resetMenu = new QMenu;
   resetMenu -> addAction(resetStatisticsCurrentBoxAction);
   resetMenu -> addAction(resetStatisticsAllBoxesAction);

   resetStatisticsButton = new QToolButton(this);
   resetStatisticsButton -> setIcon(QIcon(":/images/reset.png"));
   resetStatisticsButton -> setText(tr("Reset statistics"));
   resetStatisticsButton -> setToolTip(resetStatisticsButton->text());
   resetStatisticsButton -> setAutoRaise(true);
   resetStatisticsButton -> setPopupMode(QToolButton::InstantPopup);
   resetStatisticsButton -> setMenu(resetMenu);
//   resetStatisticsButton -> setDefaultAction(resetStatisticsCurrentBoxAction);

   toolBar = new QToolBar(tr("Toolbar"), this);
   toolBar -> setOrientation(Qt::Vertical);
   toolBar -> addAction(addBoxAction);
   toolBar -> addAction(removeBoxAction);
   toolBar -> addAction(renameBoxAction);
   toolBar -> addAction(moveUpAction);
   toolBar -> addAction(moveDownAction);
   toolBar -> addWidget(resetStatisticsButton);

   manageCardsButton = new QPushButton(tr("&Manage cards"), this);
   manageCardsButton -> setIcon(QIcon(":/images/manager.png"));
   connect(manageCardsButton, SIGNAL(clicked()), this, SLOT(manageCardsSlot()));

   reviewCardsButton = new QPushButton(tr("&Review cards"), this);
   reviewCardsButton -> setIcon(QIcon(":/images/verify.png"));
   connect(reviewCardsButton, SIGNAL(clicked()), this, SLOT(reviewCardsSlot()));

   boxList = new QListWidget(this);
   boxList -> setSelectionMode(QAbstractItemView::SingleSelection);
   connect(boxList, SIGNAL(itemSelectionChanged()), this, SLOT(updateActionsAndButtons()));
}

void StartWidget::createLayouts()
{
  QHBoxLayout *buttonsLayout = new QHBoxLayout;
  buttonsLayout -> addWidget(manageCardsButton);
  buttonsLayout -> addWidget(reviewCardsButton);

  QVBoxLayout *listAndButtonsLayout = new QVBoxLayout;
  listAndButtonsLayout -> addWidget(boxList);
  listAndButtonsLayout -> addLayout(buttonsLayout);

  QHBoxLayout *listAndButtonsAndToolBarLayout = new QHBoxLayout;
  listAndButtonsAndToolBarLayout -> addLayout(listAndButtonsLayout);
  listAndButtonsAndToolBarLayout -> addWidget(toolBar);

  QVBoxLayout *centerLayout = new QVBoxLayout;
  centerLayout -> addWidget(new QLabel(tr("Categories:"), this));
  centerLayout -> addLayout(listAndButtonsAndToolBarLayout);

  QGridLayout *mainLayout = new QGridLayout;
  mainLayout -> addLayout(centerLayout, 1, 1);
  mainLayout -> setColumnStretch(0, 1);
  mainLayout -> setColumnStretch(1, 3);
  mainLayout -> setColumnStretch(2, 1);
  mainLayout -> setRowStretch(0, 1);
  mainLayout -> setRowStretch(1, 4);
  mainLayout -> setRowStretch(2, 1);

  setLayout(mainLayout);
}

void StartWidget::start()
{
   updateListAndActionsAndButtons();
}

int StartWidget::getCurrentRow() const
{
   return boxList -> currentRow();
}

void StartWidget::setCurrentRow(int row)
{
   boxList -> setCurrentRow(row);
}

void StartWidget::updateList()
{
   CardBoxes *cardsBoxes = MainWindow::getCardBoxes();
   boxList -> clear();
   for (int i=0; i<cardsBoxes->size(); i++)
      boxList -> addItem(cardsBoxes->at(i)->getBoxName() + tr("   -   %1")
                         .arg(cardsBoxes->at(i)->countTotalCards()));
}

void StartWidget::updateActionsAndButtons()
{
   bool anyBoxSelected = boxList->selectedItems().size();
   removeBoxAction -> setEnabled(anyBoxSelected);
   renameBoxAction -> setEnabled(anyBoxSelected);
   manageCardsButton -> setEnabled(anyBoxSelected);
   reviewCardsButton -> setEnabled(anyBoxSelected);
   moveUpAction -> setEnabled(anyBoxSelected);
   moveDownAction -> setEnabled(anyBoxSelected);
   resetStatisticsCurrentBoxAction -> setEnabled(anyBoxSelected);
   resetStatisticsAllBoxesAction -> setEnabled(MainWindow::getCardBoxes()->size());
   resetStatisticsButton -> setEnabled(resetStatisticsAllBoxesAction->isEnabled()
                                       && resetStatisticsCurrentBoxAction->isEnabled());
}

void StartWidget::updateListAndActionsAndButtons()
{
   updateList();
   updateActionsAndButtons();
}

void StartWidget::addBoxSlot()
{
   bool ok;
   QString name = QInputDialog::getText(this, tr("Add"), tr("Enter a name:"),
                                        QLineEdit::Normal, "", &ok);
   if (ok && !name.isEmpty())
   {
      MainWindow::getCardBoxes()->append(new CardBox(5, name));
      updateListAndActionsAndButtons();
      emit somethingEdited();
      boxList -> setCurrentRow(boxList->count()-1);
   }
}

void StartWidget::removeBoxSlot()
{
   int currentRow = boxList -> currentRow();
   CardBoxes *cardsBoxes = MainWindow::getCardBoxes();
   delete cardsBoxes->takeAt(boxList -> currentRow());
   updateListAndActionsAndButtons();
   boxList -> setCurrentRow(currentRow<boxList->count() ? currentRow : currentRow-1);
   emit somethingEdited();
}

void StartWidget::renameBoxSlot()
{
   QString name = MainWindow::getCardBoxes()->at(boxList -> currentRow())->getBoxName();
   bool ok;
   name = QInputDialog::getText(this, tr("Rename"), tr("Enter a new name:"),
                                QLineEdit::Normal, name, &ok);
   if (ok && !name.isEmpty())
   {
      int currentRow = boxList->currentRow();
      MainWindow::getCardBoxes()->at(boxList -> currentRow())->setBoxName(name);
      updateListAndActionsAndButtons();
      boxList -> setCurrentRow(currentRow);
      emit somethingEdited();
   }
}

void StartWidget::moveUpSlot()
{
   if (boxList -> selectedItems().size() && boxList -> currentRow() != 0)
   {
      int newPosition = boxList->currentRow()-1;
      MainWindow::getCardBoxes() -> swap(boxList->currentRow(), boxList->currentRow()-1);
      updateListAndActionsAndButtons();
      boxList -> setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void StartWidget::moveDownSlot()
{
   if (boxList->selectedItems().size() && boxList->currentRow() != boxList->count()-1)
   {
      int newPosition = boxList->currentRow()+1;
      MainWindow::getCardBoxes() -> swap(boxList->currentRow(), boxList->currentRow()+1);
      updateListAndActionsAndButtons();
      boxList -> setCurrentRow(newPosition);
      emit somethingEdited();
   }
}

void StartWidget::reviewCardsSlot()
{
   emit reviewCardsSignal(boxList->currentRow());
}

void StartWidget::manageCardsSlot()
{
   emit manageCardsSignal(boxList->currentRow());
}

void StartWidget::resetStatisticsCurrentBoxSlot()
{
   if (boxList->selectedItems().size())
   {
      MainWindow::getCardBoxes() -> at(boxList->currentRow()) -> clearAnswers();
      emit somethingEdited();
   }
}

void StartWidget::resetStatisticsAllBoxesSlot()
{
   for (int i=0; i<MainWindow::getCardBoxes() -> size(); i++)
      MainWindow::getCardBoxes() -> at(i) -> clearAnswers();
   if (MainWindow::getCardBoxes() -> size())
      emit somethingEdited();
}


