// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef ADVANCEDTEXTEDITOR_H
#define ADVANCEDTEXTEDITOR_H

#include <QWidget>
#include <QDialog>
class QFontComboBox;
class QComboBox;
class TextEditor;
class QToolBar;
class QKeyEvent;
class Document;
class Card;
class ImageFetcher;
class QSplitter;
class ObjectEditorImage;
class TextEditorObject;
class QMenuBar;

class AdvancedTextEditor : public QDialog
{
   Q_OBJECT
   
   public:
      static AdvancedTextEditor* advancedTextEditor();
      void setDocument(const Document&);
      const Document& getDocument() const;
      void setCursor(int, int);
      int position() const;
      int anchor() const;
      void clear();
      
   signals:
      void editingFinished();
      
   protected:
      void focusInEvent(QFocusEvent*);
      void keyPressEvent(QKeyEvent*);
      void hideEvent(QHideEvent*);
      
   private slots:
      void fontFamily();
      void fontSize();
      void foregroundColor();
      void backgroundColor();
      void insertImage();
      void fetchImage();
      void selectionChangedSlot();
      void clickedOnObject(TextEditorObject*);
      void fontSizeSelectedFromMenu();
      void selectFont();
      
   private:
      TextEditor *textEditor;
      ImageFetcher *imageFetcher;
      ObjectEditorImage *imageEditor;
      QSplitter *horizontalSplitter;
      QSplitter *verticalSplitter;
      QToolBar *mainToolBar;

      QMenuBar *menuBar;

      QAction *copyAction;
      QAction *cutAction;
      QAction *pasteAction;
      QAction *undoAction;
      QAction *redoAction;
      QAction *boldAction;
      QAction *italicAction;
      QAction *underlineAction;
      QAction *strikeThroughAction;
      QAction *numberedListAction;
      QAction *bulletedListAction;
      QAction *alignLeftAction;
      QAction *alignCenterAction;
      QAction *alignRightAction;
      QAction *alignJustifyAction;
      QAction *leftToRightAction;
      QAction *rightToLeftAction;
      QAction *foregroundColorAction;
      QAction *backgroundColorAction;
      QAction *insertImageAction;
      QAction *fetchImageAction;
      QAction *selectFontAction;
      
      QFontComboBox *fontComboBox;
      QComboBox *fontSizeComboBox;
      
      QPushButton *okButton;
      QPushButton *cancelButton;
      
      void createActions();
      void createButtons();
      void createToolBar();
      void createMenus();
      void createLayout();
      QString extractSomeKeyWords();

      AdvancedTextEditor(QWidget* = 0);
      static AdvancedTextEditor *_advancedTextEditor;
};

class SelectFontDialog : public QDialog
{
   Q_OBJECT
   
   public:
      SelectFontDialog(const QFont&, QWidget* = 0);
      QFont getFont() const;

   private:
      QFontComboBox *fontComboBox;
};


#endif

