// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QtGui>
#include "EmbeddedTextEditor.h"
#include "AdvancedTextEditor.h"
#include "PrettyButton.h"

#define INTERVALTIME 50
#define ANIMATIONTIME 500

EmbeddedTextEditor::EmbeddedTextEditor(QWidget *parent)
   : TextEditor(parent)
{
   setMouseTracking(true);
   button = new PrettyButton(Qt::Horizontal, this);
   button -> setIcon(QIcon(":/images/advancedEditor.png"));
   button -> setText(tr("Advanced Editor   (Ctrl+E)"));
   button -> hide();
   connect(button, SIGNAL(clicked()), this, SLOT(advancedEditorButtonClicked()));
   connect(this, SIGNAL(contentsChanged()), button, SLOT(hide()));
   webView -> installEventFilter(this);
}

void EmbeddedTextEditor::resizeEvent(QResizeEvent *event)
{
   TextEditor::resizeEvent(event);

   button -> resize(button->sizeHint().width()+30, button->sizeHint().height());
   button -> move((width() - button->width())/2, height()-button->height()-(height()-webView->height())/2);
}

void EmbeddedTextEditor::advancedEditorButtonClicked()
{
   AdvancedTextEditor *ate = AdvancedTextEditor::advancedTextEditor();
   ate -> setDocument(getDocument());
//   advancedTextEditor -> setCursor(textCursor().position(),
//                                   textCursor().anchor());
   if (ate->exec())
   {
      setDocument(ate->getDocument());
//      QTextCursor newCursor = textCursor();
//      newCursor.setPosition(advancedTextEditor->anchor());
//      newCursor.setPosition(advancedTextEditor->position(), QTextCursor::KeepAnchor);
//      setTextCursor(newCursor);
   }
   emit editingFinished();
   setFocus();
}

bool EmbeddedTextEditor::eventFilter(QObject *object, QEvent *event)
{
   if (object == webView)
   {
      if (event->type() == QEvent::MouseMove)
      {
         QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
         QRect rect(button->x(), button->y()-button->height()/4, button->width(), button->height());
         if (rect.contains(mouseEvent->pos()) && !button->isVisible())
            button->show();
      }
      else if (event->type() == QEvent::KeyPress)
      {
         QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
         if (keyEvent->key() == Qt::Key_E && (keyEvent->modifiers() & Qt::ControlModifier))
         {
            button -> click();
            return true;
         }
      }
      else if (event->type() == QEvent::FocusIn)
      {
         button->show();
      }
   }
   return false;
}

