// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "ObjectEditorImage.h"

ObjectEditorImage::ObjectEditorImage(TextEditor *_textEditor, QWidget *parent)
   : ObjectEditor (_textEditor, parent)
{
   setupUi(this);
}

void ObjectEditorImage::editObject(TextEditorObject *_object)
{
   object = dynamic_cast<TextEditorObjectImage*>(_object);
   if (!object)
   {
      qDebug("ObjectEditorImage::editObject(): the received object is not an image object");
      return;
   }
   
   QImage image = object -> getImage();
   widthSpinBox -> setValue(image.width());
   heightSpinBox -> setValue(image.height());
   if (constrainRatioCheckBox -> isChecked())
   {
      ratioWidth = image.width();
      ratioHeight = image.height();
   }
}

void ObjectEditorImage::finishEditing()
{
   
}

// bool ObjectEditorImage::isChanged() const
// {

// }

// const TextEditorObject* ObjectEditorImage::getObject() const
// {

// }

void ObjectEditorImage::on_widthSpinBox_valueChangedByUser(int value)
{
   if (constrainRatioCheckBox -> isChecked())
      heightSpinBox -> setValue(static_cast<double>(value)*ratioHeight/ratioWidth);
}

void ObjectEditorImage::on_heightSpinBox_valueChangedByUser(int value)
{
   if (constrainRatioCheckBox -> isChecked())
      widthSpinBox -> setValue(static_cast<double>(value)*ratioWidth/ratioHeight);
}

void ObjectEditorImage::on_constrainRatioCheckBox_toggled(bool state)
{
   if (state)
   {
      ratioWidth = widthSpinBox -> value();
      ratioHeight = heightSpinBox -> value();
   }
}

void ObjectEditorImage::on_closeButton_clicked()
{
   emit editingFinished();
}

void ObjectEditorImage::on_resizeButton_clicked()
{
   textEditor -> addCommand(new CommandResizeImage(object, QSize(widthSpinBox->value(), heightSpinBox->value())));
}

void ObjectEditorImage::on_flipButton_clicked()
{
   textEditor -> addCommand(new CommandFlipImage(object, Qt::Vertical));
}

void ObjectEditorImage::on_mirrorButton_clicked()
{
   textEditor -> addCommand(new CommandFlipImage(object, Qt::Horizontal));
}

void ObjectEditorImage::on_rotateRightButton_clicked()
{
   textEditor -> addCommand(new CommandRotateImage(object, CommandRotateImage::RotateRight));
}

void ObjectEditorImage::on_rotateLeftButton_clicked()
{
   textEditor -> addCommand(new CommandRotateImage(object, CommandRotateImage::RotateLeft));
}

