// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "AdvancedTextEditor.h"
#include "TextEditor.h"
#include "ImageFetcher.h"
#include "Utilities.h"
#include "ObjectEditorImage.h"
#include <QtGui>
#include <QtWebKit>

AdvancedTextEditor* AdvancedTextEditor::_advancedTextEditor = 0;

AdvancedTextEditor* AdvancedTextEditor::advancedTextEditor()
{
   if (_advancedTextEditor)
      return _advancedTextEditor;
   else
      return _advancedTextEditor = new AdvancedTextEditor();
}

AdvancedTextEditor::AdvancedTextEditor(QWidget *parent)
   : QDialog(parent)
{
   setWindowTitle(tr("Advanced Text Editor"));
   setWindowIcon(QIcon(":/images/flashqard_icon.png"));
   textEditor = new TextEditor(this);
   QSizePolicy tPolicy = textEditor -> sizePolicy();
   tPolicy.setHorizontalStretch(3);
   textEditor -> setSizePolicy(tPolicy);

   createActions();
   createButtons();
   createMenus();
   createToolBar();

   imageFetcher = new ImageFetcher(this);
   QSizePolicy iPolicy = imageFetcher -> sizePolicy();
   iPolicy.setHorizontalStretch(1);
   imageFetcher -> setSizePolicy(iPolicy);

   imageEditor = new ObjectEditorImage(textEditor, this);
   imageEditor -> hide();
   connect(imageEditor, SIGNAL(editingFinished()), imageEditor, SLOT(hide()));
   
   createLayout();
   connect(textEditor, SIGNAL(selectionChanged()), this, SLOT(selectionChangedSlot()));
   connect(textEditor, SIGNAL(clickedOnObjectSignal(TextEditorObject*)), this, SLOT(clickedOnObject(TextEditorObject*)));
   resize(sizeHint().width(), static_cast<int>(sizeHint().width()/1.6)); //1.6: Golden Ratio!

}

void AdvancedTextEditor::createActions()
{
   cutAction = textEditor -> getAction(QWebPage::Cut);
   cutAction->setIcon(QIcon(":/images/cut.png"));
   cutAction->setShortcut(Qt::CTRL + Qt::Key_X);
   cutAction->setToolTip(cutAction->toolTip() + tr(" (") + cutAction->shortcut().toString() + tr(")"));

   copyAction = textEditor -> getAction(QWebPage::Copy);
   copyAction->setIcon(QIcon(":/images/copy.png"));
   copyAction->setShortcut(Qt::CTRL + Qt::Key_C);
   copyAction->setToolTip(copyAction->toolTip() + tr(" (") + copyAction->shortcut().toString() + tr(")"));

   pasteAction = textEditor -> getAction(QWebPage::Paste);
   pasteAction->setIcon(QIcon(":/images/paste.png"));
   pasteAction->setShortcut(Qt::CTRL + Qt::Key_V);
   pasteAction->setToolTip(pasteAction->toolTip() + tr(" (") + pasteAction->shortcut().toString() + tr(")"));

   undoAction = textEditor -> getAction(QWebPage::Undo);
   undoAction->setIcon(QIcon(":/images/undo.png"));
   undoAction->setShortcut(Qt::CTRL + Qt::Key_Z);
   undoAction->setToolTip(undoAction->toolTip() + tr(" (") + undoAction->shortcut().toString() + tr(")"));

   redoAction = textEditor -> getAction(QWebPage::Redo);
   redoAction->setIcon(QIcon(":/images/redo.png"));
   redoAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_Z);
   redoAction->setToolTip(redoAction->toolTip() + tr(" (") + redoAction->shortcut().toString() + tr(")"));

   boldAction = textEditor -> getAction(QWebPage::ToggleBold);
   boldAction->setIcon(QIcon(":/images/bold.png"));
   boldAction->setShortcut(Qt::CTRL + Qt::Key_B);
   boldAction->setToolTip(boldAction->toolTip() + tr(" (") + boldAction->shortcut().toString() + tr(")"));
   
   italicAction = textEditor -> getAction(QWebPage::ToggleItalic);
   italicAction->setIcon(QIcon(":/images/italic.png"));
   italicAction->setShortcut(Qt::CTRL + Qt::Key_I);
   italicAction->setToolTip(italicAction->toolTip() + tr(" (") + italicAction->shortcut().toString() + tr(")"));
   
   underlineAction = textEditor -> getAction(QWebPage::ToggleUnderline);
   underlineAction->setIcon(QIcon(":/images/under.png"));
   underlineAction->setShortcut(Qt::CTRL + Qt::Key_U);
   underlineAction->setToolTip(underlineAction->toolTip() + tr(" (") + underlineAction->shortcut().toString() + tr(")"));

   strikeThroughAction = textEditor -> getAction(TextEditor::StrikeThrough);
   strikeThroughAction->setIcon(QIcon(":/images/strike.png"));
   strikeThroughAction->setShortcut(Qt::CTRL + Qt::Key_S);
   strikeThroughAction->setToolTip(strikeThroughAction->toolTip() + tr(" (") + strikeThroughAction->shortcut().toString() + tr(")"));
   
   alignLeftAction = textEditor -> getAction(TextEditor::AlignLeft);
   alignLeftAction->setIcon(QIcon(":/images/left.png"));
   alignLeftAction->setShortcut(Qt::CTRL + Qt::Key_L);
   alignLeftAction->setToolTip(alignLeftAction->toolTip() + tr(" (") + alignLeftAction->shortcut().toString() + tr(")"));
   
   alignCenterAction = textEditor -> getAction(TextEditor::AlignCenter);
   alignCenterAction->setIcon(QIcon(":/images/center.png"));
   alignCenterAction->setShortcut(Qt::CTRL + Qt::Key_E);
   alignCenterAction->setToolTip(alignCenterAction->toolTip() + tr(" (") + alignCenterAction->shortcut().toString() + tr(")"));
   
   alignRightAction = textEditor -> getAction(TextEditor::AlignRight);
   alignRightAction->setIcon(QIcon(":/images/right.png"));
   alignRightAction->setShortcut(Qt::CTRL + Qt::Key_R);
   alignRightAction->setToolTip(alignRightAction->toolTip() + tr(" (") + alignRightAction->shortcut().toString() + tr(")"));
   
   alignJustifyAction = textEditor -> getAction(TextEditor::AlignJustify);
   alignJustifyAction->setIcon(QIcon(":/images/justify.png"));
   alignJustifyAction->setShortcut(Qt::CTRL + Qt::Key_J);
   alignJustifyAction->setToolTip(alignJustifyAction->toolTip() + tr(" (") + alignJustifyAction->shortcut().toString() + tr(")"));
   
   leftToRightAction = textEditor -> getAction(QWebPage::SetTextDirectionLeftToRight);
   leftToRightAction->setIcon(QIcon(":/images/ltr.png"));
   leftToRightAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_A);
   leftToRightAction->setToolTip(leftToRightAction->toolTip() + tr(" (") + leftToRightAction->shortcut().toString() + tr(")"));
   
   rightToLeftAction = textEditor -> getAction(QWebPage::SetTextDirectionRightToLeft);
   rightToLeftAction->setIcon(QIcon(":/images/rtl.png"));
   rightToLeftAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_D);
   rightToLeftAction->setToolTip(rightToLeftAction->toolTip() + tr(" (") + rightToLeftAction->shortcut().toString() + tr(")"));

   numberedListAction = textEditor -> getAction(TextEditor::NumberedList);
   numberedListAction->setIcon(QIcon(":/images/numberedList.png"));
   numberedListAction->setShortcut(Qt::CTRL + Qt::Key_N);
   numberedListAction->setToolTip(numberedListAction->toolTip() + tr(" (") + numberedListAction->shortcut().toString() + tr(")"));

   bulletedListAction = textEditor -> getAction(TextEditor::BulletedList);
   bulletedListAction->setIcon(QIcon(":/images/bulletedList.png"));
   bulletedListAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_B);
   bulletedListAction->setToolTip(bulletedListAction->toolTip() + tr(" (") + numberedListAction->shortcut().toString() + tr(")"));
   
   foregroundColorAction = new QAction(QIcon(":/images/foregroundColor.png"), tr("Foreground Color"), this);
   connect(foregroundColorAction, SIGNAL(triggered()), this, SLOT(foregroundColor()));
   
   backgroundColorAction = new QAction(QIcon(":/images/backgroundColor.png"), tr("Background Color"), this);
   connect(backgroundColorAction, SIGNAL(triggered()), this, SLOT(backgroundColor()));

   insertImageAction = new QAction(QIcon(":/images/insertImage.png"), tr("Insert image"), this);
   insertImageAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_I);
   insertImageAction->setToolTip(insertImageAction->toolTip() + tr(" (") + insertImageAction->shortcut().toString() + tr(")"));
   connect(insertImageAction, SIGNAL(triggered()), this, SLOT(insertImage()));

   fetchImageAction = new QAction(QIcon(":/images/fetchImage.png"), tr("Fetch Image from Internet"), this);
   fetchImageAction->setCheckable(true);
   fetchImageAction->setShortcut(Qt::CTRL + Qt::Key_F);
   fetchImageAction->setToolTip(fetchImageAction->toolTip() + tr(" (") + fetchImageAction->shortcut().toString() + tr(")"));
   connect(fetchImageAction, SIGNAL(triggered()), this, SLOT(fetchImage()));

   selectFontAction = new QAction(tr("Font Name..."), this);
   connect(selectFontAction, SIGNAL(triggered()), this, SLOT(selectFont()));
}

void AdvancedTextEditor::createButtons()
{
   okButton = new QPushButton(tr("&Ok"), this);
   connect (okButton, SIGNAL(clicked()), this, SLOT(accept()));
   
   cancelButton = new QPushButton(tr("&Cancel"), this);
   connect (cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
}

void AdvancedTextEditor::createToolBar()
{
   mainToolBar = new QToolBar(this);
   mainToolBar -> setIconSize(QSize(22, 22));
   
   fontComboBox = new QFontComboBox(this);
   connect (fontComboBox, SIGNAL(activated(int)), this, SLOT(fontFamily()));
   
   fontSizeComboBox = new QComboBox(this);
   fontSizeComboBox->setEditable(false);
   fontSizeComboBox->setCompleter(0);
   fontSizeComboBox->setInsertPolicy(QComboBox::NoInsert);
   fontSizeComboBox->addItem(tr("xx-small"));
   fontSizeComboBox->addItem(tr("x-small"));
   fontSizeComboBox->addItem(tr("small"));
   fontSizeComboBox->addItem(tr("medium"));
   fontSizeComboBox->addItem(tr("large"));
   fontSizeComboBox->addItem(tr("x-large"));
   fontSizeComboBox->addItem(tr("xx-large"));
   connect (fontSizeComboBox->lineEdit(), SIGNAL(returnPressed()), this, SLOT(fontSize()));
   connect (fontSizeComboBox, SIGNAL(activated(int)), this, SLOT(fontSize()));
   
   mainToolBar->addWidget(fontComboBox);
   mainToolBar->addWidget(fontSizeComboBox);
   mainToolBar->addSeparator();
   mainToolBar->addAction(boldAction);
   mainToolBar->addAction(italicAction);
   mainToolBar->addAction(underlineAction);
   mainToolBar->addAction(strikeThroughAction);
   mainToolBar->addSeparator();
   mainToolBar->addAction(foregroundColorAction);
   mainToolBar->addAction(backgroundColorAction);
   mainToolBar->addSeparator();

   if (QApplication::layoutDirection() == Qt::LeftToRight)
   {
      mainToolBar->addAction(alignLeftAction);
      mainToolBar->addAction(alignCenterAction);
      mainToolBar->addAction(alignRightAction);
      mainToolBar->addAction(alignJustifyAction);
      mainToolBar->addSeparator();
      mainToolBar->addAction(leftToRightAction);
      mainToolBar->addAction(rightToLeftAction);
   }
   else
   {
      mainToolBar->addAction(alignJustifyAction);
      mainToolBar->addAction(alignRightAction);
      mainToolBar->addAction(alignCenterAction);
      mainToolBar->addAction(alignLeftAction);
      mainToolBar->addSeparator();
      mainToolBar->addAction(rightToLeftAction);
      mainToolBar->addAction(leftToRightAction);
   }

   mainToolBar->addSeparator();
   mainToolBar->addAction(numberedListAction);
   mainToolBar->addAction(bulletedListAction);

   mainToolBar->addSeparator();
   mainToolBar->addAction(insertImageAction);
   mainToolBar->addAction(fetchImageAction);
}

void AdvancedTextEditor::createMenus()
{
   menuBar = new QMenuBar(this);
   QMenu *editMenu = menuBar -> addMenu(tr("&Edit"));
   editMenu -> addAction(undoAction);
   editMenu -> addAction(redoAction);
   editMenu -> addSeparator();
   editMenu -> addAction(cutAction);
   editMenu -> addAction(copyAction);
   editMenu -> addAction(pasteAction);

   QMenu *formatMenu = menuBar -> addMenu(tr("&Format"));

   formatMenu -> addAction(selectFontAction);
   
   QMenu *fontSizeMenu = formatMenu -> addMenu(tr("Font Si&ze"));
   QAction *tempAction;
   tempAction = fontSizeMenu -> addAction(tr("&xx-small"));
   tempAction -> setData(0);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("x-&small"));
   tempAction -> setData(1);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("s&mall"));
   tempAction -> setData(2);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("m&edium"));
   tempAction -> setData(3);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("&large"));
   tempAction -> setData(4);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("x-l&arge"));
   tempAction -> setData(5);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));
   tempAction = fontSizeMenu -> addAction(tr("xx-la&rge"));
   tempAction -> setData(6);
   connect(tempAction, SIGNAL(triggered()), this, SLOT(fontSizeSelectedFromMenu()));

   formatMenu -> addSeparator();
   QMenu *fontStyleMenu = formatMenu -> addMenu(tr("Font &Style"));
   fontStyleMenu -> addAction(boldAction);
   fontStyleMenu -> addAction(italicAction);
   fontStyleMenu -> addAction(underlineAction);
   fontStyleMenu -> addAction(strikeThroughAction);

   formatMenu -> addSeparator();
   QMenu *alignmentMenu = formatMenu -> addMenu(tr("&Alignment"));
   alignmentMenu -> addAction(alignLeftAction);
   alignmentMenu -> addAction(alignCenterAction);
   alignmentMenu -> addAction(alignRightAction);
   alignmentMenu -> addAction(alignJustifyAction);

   formatMenu -> addSeparator();
   QMenu *directionMenu = formatMenu -> addMenu(tr("&Direction"));
   directionMenu -> addAction(leftToRightAction);
   directionMenu -> addAction(rightToLeftAction);

   formatMenu -> addSeparator();
   QMenu *paragraphStyleMenu = formatMenu -> addMenu(tr("&Paragraph Style"));
   paragraphStyleMenu -> addAction(numberedListAction);
   paragraphStyleMenu -> addAction(bulletedListAction);

   QMenu *insertMenu = menuBar -> addMenu(tr("&Insert"));
   insertMenu -> addAction(insertImageAction);

   QMenu *toolsMenu = menuBar -> addMenu(tr("&Tools"));
   toolsMenu -> addAction(fetchImageAction);
   

   formatMenu -> addSeparator();
   formatMenu -> addAction(foregroundColorAction);
   formatMenu -> addAction(backgroundColorAction);

}

void AdvancedTextEditor::createLayout()
{
   QVBoxLayout *mainLayout = new QVBoxLayout(this);
   QHBoxLayout *buttonsLayout = new QHBoxLayout;
   buttonsLayout -> addStretch();
   buttonsLayout -> addWidget(okButton);
   buttonsLayout -> addWidget(cancelButton);

   verticalSplitter = new QSplitter(Qt::Vertical);
   verticalSplitter -> addWidget(textEditor);
   verticalSplitter -> addWidget(imageEditor);

   horizontalSplitter = new QSplitter (Qt::Horizontal);
   horizontalSplitter -> addWidget(verticalSplitter);
   horizontalSplitter -> addWidget(imageFetcher);

   mainLayout -> addWidget(menuBar);
   mainLayout -> addWidget(mainToolBar);
   mainLayout -> addWidget(horizontalSplitter,1);
   mainLayout -> addLayout(buttonsLayout);
   imageFetcher -> hide();
}

void AdvancedTextEditor::setDocument(const Document &document)
{
   textEditor -> setDocument(document);
   imageFetcher -> hide();
   fetchImageAction -> setChecked(false);

   //when compiling with qt-4.4, fontSizeComboBox and fontComboBox
   //are set to disabled by default. (I don't know why!)
   fontSizeComboBox -> setEnabled(true);
   fontComboBox -> setEnabled(true);

   //NOTE: use setCursor instead of this
   textEditor -> startEditing();
}

const Document& AdvancedTextEditor::getDocument() const
{
   return textEditor->getDocument();
}

void AdvancedTextEditor::setCursor(int newPosition, int newAnchor)
{
   //NOTE: temporarily we are using startEditing function of
   //textEditor. But try to support setCursor
   
//    QTextCursor newCursor = textEditor->textCursor();
//    newCursor.setPosition(newAnchor);
//    newCursor.setPosition(newPosition, QTextCursor::KeepAnchor);
//    textEditor->setTextCursor(newCursor);
}

int AdvancedTextEditor::position() const
{
   //return textEditor->textCursor().position();
    return 0;
}

int AdvancedTextEditor::anchor() const
{
   //return textEditor->textCursor().anchor();
    return 0;
}

void AdvancedTextEditor::clear()
{
   textEditor -> clear();
}

void AdvancedTextEditor::focusInEvent(QFocusEvent *event)
{
   QDialog::focusInEvent(event);
   textEditor->setFocus();
}

void AdvancedTextEditor::keyPressEvent(QKeyEvent *event)
{
   if (event->key() == Qt::Key_Enter)
      accept();
}

void AdvancedTextEditor::hideEvent(QHideEvent*)
{
   imageFetcher -> stop();
}

void AdvancedTextEditor::fontFamily()
{
   textEditor->setFontFamily(fontComboBox->currentFont().family());
   textEditor->setFocus();
}

void AdvancedTextEditor::fontSize()
{
   textEditor->setFontSize(fontSizeComboBox->currentIndex());
   textEditor->setFocus();
}

void AdvancedTextEditor::foregroundColor()
{
   QColor color = textEditor -> getForegroundColor();
   color = QColorDialog::getColor(color, this);
   if (color.isValid())
      textEditor->setForegroundColor(color);
}

void AdvancedTextEditor::backgroundColor()
{
   QColor color = textEditor -> getBackgroundColor();
   color = QColorDialog::getColor(color, this);
   if (color.isValid())
      textEditor->setBackgroundColor(color);
}

void AdvancedTextEditor::insertImage()
{
   QList<QByteArray> formats = QImageReader::supportedImageFormats();
   QString filter = tr("Image Files") + tr(" (");
   for (int i=0; i<formats.size(); i++)
      filter += " *." + QString(formats[i].data()).toLower();
   filter += tr(" )");
   QString imageUrl = QFileDialog::getOpenFileName(this, tr("Insert image"), QString(), filter);
   if (!imageUrl.isNull())
      textEditor -> insertImage(imageUrl);

}

void AdvancedTextEditor::fetchImage()
{
   if (fetchImageAction -> isChecked())
   {
      imageFetcher -> show();
      imageFetcher->fetchImageFor(extractSomeKeyWords());
   }
   else
   {
      imageFetcher -> stop();
      imageFetcher -> hide();
   }
}

void AdvancedTextEditor::selectionChangedSlot()
{
   fontComboBox->setCurrentFont(QFont(textEditor->getFontFamily()));
   fontSizeComboBox->setCurrentIndex(textEditor->getFontSize());
   imageEditor -> hide();
   qDebug("AdvancedTextEditor::selectionChangedSlot(): font size: %d", textEditor->getFontSize());
}

void AdvancedTextEditor::clickedOnObject(TextEditorObject *object)
{
   imageEditor -> editObject(object);
   imageEditor -> show();
}

void AdvancedTextEditor::fontSizeSelectedFromMenu()
{
   QAction *senderAction = dynamic_cast<QAction*>(sender());
   if (senderAction)
   {
      fontSizeComboBox -> setCurrentIndex(senderAction->data().toInt());
      fontSize();
   }
}

void AdvancedTextEditor::selectFont()
{
   SelectFontDialog dialog(fontComboBox->currentFont(), this);
   if (dialog.exec())
   {
      fontComboBox -> setCurrentFont(dialog.getFont());
      fontFamily();
   }
}

QString AdvancedTextEditor::extractSomeKeyWords()
{
   ///TODO: Smart keyword extractor
   return HtmlParser::htmlParser()->htmlToPlainText(textEditor -> getDocument().getText());
}

// void AdvancedTextEditor::currentCharFormatChangedSlot(const QTextCharFormat &format)
// {
//    boldAction->setChecked(format.fontWeight() == QFont::Bold);
//    italicAction->setChecked(format.fontItalic());
//    underlineAction->setChecked(format.fontUnderline());
   
//    fontComboBox->setCurrentFont(format.font());
//    fontSizeComboBox->lineEdit()->setText(QString::number(format.font().pointSize()));
// }

// void AdvancedTextEditor::updateAlignmentActions()
// {
//    Qt::Alignment alignment = textEditor->alignment();
//    alignLeftAction->setChecked(alignment & Qt::AlignLeft);
//    alignCenterAction->setChecked(alignment & Qt::AlignCenter);
//    alignRightAction->setChecked(alignment & Qt::AlignRight);
//    alignJustifyAction->setChecked(alignment & Qt::AlignJustify);
//}

//void AdvancedTextEditor::updateDirectionActions()
//{
//    QTextCursor cursor = textEditor->textCursor();
//    QTextBlockFormat format = cursor.blockFormat();
//    leftToRightAction->setChecked(format.layoutDirection() == Qt::LeftToRight);
//    rightToLeftAction->setChecked(format.layoutDirection() == Qt::RightToLeft);
//}

//void AdvancedTextEditor::setAlignment(Qt::Alignment alignment)
//{
//    textEditor->setAlignment(alignment);
//    updateAlignmentActions();
//}

//void AdvancedTextEditor::setTextLayoutDirection(Qt::LayoutDirection direction)
//{
//    QTextCursor cursor = textEditor->textCursor();
//    QTextBlockFormat format = cursor.blockFormat();
//    format.setLayoutDirection(direction);
//    cursor.setBlockFormat(format);
//    updateDirectionActions();
//}


//======================================================================

SelectFontDialog::SelectFontDialog(const QFont &currentFont, QWidget *parent)
   : QDialog(parent)
{
   QVBoxLayout *verticalLayout = new QVBoxLayout(this);

   QLabel *label = new QLabel(tr("Please select a font:"), this);
   fontComboBox = new QFontComboBox(this);
   fontComboBox -> setCurrentFont(currentFont);
   verticalLayout -> addWidget(label);
   verticalLayout -> addWidget(fontComboBox);

   
   QHBoxLayout *horizontalLayout = new QHBoxLayout(this);
   horizontalLayout -> addStretch();

   QPushButton *okButton = new QPushButton(tr("&OK"), this);
   connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
   QPushButton *cancelButton = new QPushButton(tr("&Cancel"), this);
   connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

   horizontalLayout -> addWidget(okButton);
   horizontalLayout -> addWidget(cancelButton);

   verticalLayout -> addLayout(horizontalLayout);
}

QFont SelectFontDialog::getFont() const
{
   return fontComboBox->currentFont();
}
