// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include <QRegExp>
#include <QToolBar>
#include <QMessageBox>
#include <QApplication>
#include <QTextDocument>
#include <QTextCursor>
#include <QtWebKit>
#include "Utilities.h"
#include "Settings.h"

HtmlParser* HtmlParser::_htmlParser = 0;

HtmlParser* HtmlParser::htmlParser()
{
   if (_htmlParser)
      return _htmlParser;
   else
      return _htmlParser = new HtmlParser;
}

HtmlParser::HtmlParser()
{
   webPage = new QWebPage();
}

QString HtmlParser::htmlToPlainText(const QString &html)
{
   webPage->mainFrame()->setHtml(html);
   return webPage->mainFrame()->toPlainText();

   
//    int tagCount=0;
//    int iterator=0;
//    int tagPosition=0;
//    int tagLength=0;
//    QString tagName;
//    QString plainText;

//    if (!findTag(html, iterator, "body", tagPosition, tagLength))
//       iterator = 0;
//    else
//       iterator = tagPosition + tagLength;

//    bool firstP = true;
//    while (findNextTag(html, iterator, tagName, tagPosition, tagLength))
//    {
//       tagCount++;
//       QString tmpText = html.mid(iterator, tagPosition-iterator);
//       bool spaceAtBeginning = false;
//       bool spaceAtEnd = false;
//       if (tmpText.size())
//       {
//          spaceAtBeginning = tmpText[0].isSpace();
//          spaceAtEnd = tmpText[tmpText.size()-1].isSpace();
//       }

//       tmpText = tmpText.simplified();
//       if (spaceAtBeginning)
//          tmpText.insert(0, ' ');
//       if (spaceAtEnd)
//          tmpText.append(' ');
      
//       plainText += tmpText;
//       if (tagName.compare("p", Qt::CaseInsensitive) == 0 && !firstP)
//          plainText += "\n";
//       else if (tagName.compare("br", Qt::CaseInsensitive) == 0)
//          plainText += "\n";

//       firstP = false;
//       iterator = tagPosition+tagLength;
//    }
   
//    if (tagCount==0)
//       plainText = html;

//    plainText.replace("&lt;", "<", Qt::CaseInsensitive);
//    plainText.replace("&gt;", ">", Qt::CaseInsensitive);
//    return plainText;
}

bool HtmlParser::findNextTag(const QString &html, int iterator, QString &tagName, int &tagPosition, int &tagLength) const
{
   QRegExp regExp("<(/?)\\s*(\\w+|!--)[^>]*>", Qt::CaseInsensitive);
   tagPosition = regExp.indexIn(html, iterator);
//   Due to a different behaviour of qt 4.5 from previous versions the line below should not be used
//   (replaced by the line above)
//   tagPosition = html.indexOf(regExp, iterator);

   if (tagPosition == -1)
      return false;

   tagLength = regExp.matchedLength();
   tagName = regExp.cap(1)+regExp.cap(2);
   return true;
}

bool HtmlParser::findTag(const QString &html, int iterator, const QString &tagName, int &tagPosition, int &tagLength) const
{
   QString currentTagName;
   while (findNextTag(html, iterator, currentTagName, tagPosition, tagLength))
   {
      if (currentTagName.compare(tagName, Qt::CaseInsensitive) == 0)
         return true;
      iterator = tagPosition + tagLength;
   }
   return false;
}

bool HtmlParser::setTagAttribute(QString &html, int &iterator, const QString &tagName,
                                    const QString &attributeName, const QString &attributeValue)
{
   int tagPosition=0;
   int tagLength=0;
   
   if (!findTag(html, iterator, tagName, tagPosition, tagLength))
      return false;

   QString completeTag = html.mid(tagPosition, tagLength);

   //name attributeName = "attributeValue"
   QRegExp regExp("\\W(" + attributeName + "\\s*=\\s*)\"([^\"]*)\"", Qt::CaseInsensitive);
   if (regExp.indexIn(completeTag) != -1)
   {
      completeTag.remove(regExp.pos(2), regExp.cap(2).length());
      completeTag.insert(regExp.pos(2), attributeValue);
   }
   else
      completeTag.insert(completeTag.length()-1, " " + attributeName + "=\"" + attributeValue + "\"");

   html.remove(tagPosition, tagLength);
   html.insert(tagPosition, completeTag);

   iterator += completeTag.length();
   return true;
}

bool HtmlParser::getTagAttribute(const QString &html, int &iterator, const QString &tagName,
                                    const QString &attributeName, QString &attributeValue)
{
   int tagPosition=0;
   int tagLength=0;
   
   if (!findTag(html, iterator, tagName, tagPosition, tagLength))
      return false;

   QString completeTag = html.mid(tagPosition, tagLength);

   //tagName attributeName = "attributeValue"
   QRegExp regExp("\\W(" + attributeName + "\\s*=\\s*)\"([^\"]*)\"", Qt::CaseInsensitive);
   if (regExp.indexIn(completeTag) != -1)
      attributeValue = regExp.cap(2);
   else
      attributeValue = QString();

   iterator += completeTag.length();
   return true;
}


bool HtmlParser::setTagStyleAttribute(QString &html, int iterator, const QString &tagName,
                                         const QString &attributeName, const QString &attributeValue)
{
   int tagPosition=0;
   int tagLength=0;
   
   if (!findTag(html, iterator, tagName, tagPosition, tagLength))
      return false;

   QString completeTag = html.mid(tagPosition, tagLength);

   QRegExp regExp("\\W(style\\s*=\\s*)\"([^\"]*)\"", Qt::CaseInsensitive);
   if (regExp.indexIn(completeTag) != -1)
   {
      QRegExp innerRegExp("\\W" + attributeName + "\\s*:\\s*([^;$]*)[;$]", Qt::CaseInsensitive);
      if (innerRegExp.indexIn(regExp.cap(2)) != -1)
         {
            QString styleAttributes = regExp.cap(2);
            styleAttributes.remove(innerRegExp.pos(1), innerRegExp.cap(1).length());
            styleAttributes.insert(innerRegExp.pos(1), attributeValue);
            completeTag.remove(regExp.pos(2), regExp.cap(2).length());
            completeTag.insert(regExp.pos(2), styleAttributes);
         }
         else
         {
            completeTag.insert(regExp.pos(2), attributeName + ":" + attributeValue + ";");
         }
   }
   else
      completeTag.insert(completeTag.length()-1, " style=\"" + attributeName + ":" + attributeValue + ";\"");

   html.remove(tagPosition, tagLength);
   html.insert(tagPosition, completeTag);

   return true;
}

bool HtmlParser::contains(const QString &html, const QString &string, bool isRegExp) const
{
   bool contains = false;
   QRegExp regExp = isRegExp ? QRegExp(string) : QRegExp(string, Qt::CaseInsensitive, QRegExp::Wildcard);
   contains = html.contains(regExp);
   if (contains)
   {
      QTextDocument document;
      document.setHtml(html);
      contains = !document.find(regExp).isNull();
   }
   return contains;
}

QString HtmlParser::contentsOfTag(const QString &html, int iterator, const QString &tagName) const
{
   int tagPosition=0;
   int tagLength=0;
   int contentsPosition=0;
   
   if (!findTag(html, iterator, tagName, tagPosition, tagLength))
      return QString();

   contentsPosition = tagPosition+tagLength;

   if (!findTag(html, iterator, "/"+tagName, tagPosition, tagLength))
      return QString();

   return html.mid(contentsPosition, tagPosition-contentsPosition);
}

//======================================================================

Reader* Reader::_reader = 0;

Reader* Reader::reader()
{
   if (_reader)
      return _reader;
   else
      return _reader = new Reader;
}

Reader::Reader(QObject *parent)
   :QObject(parent)
{
   textID = 0;
   process = new QProcess(this);
   connect (process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(finished()));
   connect (process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(processErrorSlot(QProcess::ProcessError)));
}
   
Reader::~Reader()
{
   process -> terminate();
}
      
int Reader::readPlainText(const QString &plainText)
{
   QString tempText;
   //get rid of non english letters
   //and convert \n to \n\n
   for (int i=0; i<plainText.size(); i++)
      if (plainText[i] == '\n')
         tempText += "\n\n";
      else if (plainText[i].toAscii())
         tempText += plainText[i];
   return read(tempText);
}

int Reader::readHtml(const QString &html)
{
   QString plainText = HtmlParser::htmlParser()->htmlToPlainText(html);
   return readPlainText(plainText);
}

bool Reader::isReading() const
{
   return process -> state() != QProcess::NotRunning;
}

void Reader::processErrorSlot(QProcess::ProcessError error)
{
   if (error == QProcess::FailedToStart)
      didNotFindEspeak();
   emit readingFinished(textID);
}

void Reader::finished()
{
   if (process -> exitCode() != 0)
      didNotFindEspeak();
   emit readingFinished(textID);
}

void Reader::didNotFindEspeak()
{
   QMessageBox::warning(0, tr("Faild to read"),
                        tr("<p>Program faild to start \"espeak\".\n"
                           "Please make sure that \"espeak\" program is installed and try again.</p>")
#ifdef Q_WS_WIN
                        + tr("<p>You can download espeak from here:<br>"
                             "<a href=\"http://kent.dl.sourceforge.net/sourceforge/espeak/espeak-1.40.02-win.zip\">Download link</a></p>")
#endif
                        );
}

int Reader::read(const QString &text)
{
   process -> terminate();
   process -> waitForFinished(500);
   QStringList arguments;
   arguments << "-p" << QString::number(Settings::settings()->espeakPitch);
   arguments << "-s" << QString::number(Settings::settings()->espeakSpeed);
   arguments << "-v" << (Settings::settings()->espeakAccent == 0 ? "en" : "en-r");
   arguments << text;
#ifdef Q_WS_WIN
   process -> start("espeak.exe", arguments);
#else
   process -> start("espeak", arguments);
#endif

   emit readingStarted(++textID);
   return textID;
}

void Reader::stop()
{
   process -> terminate();
}

//======================================================================

// QString TextDocumentUtilities::defaultHtmlCode()
// {
//    //NOTE: do we need it any more?
//    return QString();

//    return QString(
//       "<html><head></head>"
//       "<body style=\" font-family:'%1'; font-size:%2pt; font-style:%3; color:%4;\" "
//       "bgcolor=\"%5\" >"
//       "<p align=\"%6\" dir='%7' style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; "
//       "margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:%8pt; color:%9;\"></p></body></html>")
//       .arg(Settings::settings()->editorFont.family())
//       .arg(Settings::settings()->editorFont.pointSize())
//       .arg(Settings::settings()->editorFont.style() == QFont::StyleNormal ? "normal"
//            : Settings::settings()->editorFont.style() == QFont::StyleItalic ? "italic"
//            : "oblique")
//       .arg(Settings::settings()->editorForegroundColor.name())
//       .arg(Settings::settings()->editorBackgroundColor.name())
//       .arg(Settings::settings()->editorTextAlignment & Qt::AlignLeft ? "left"
//            : Settings::settings()->editorTextAlignment & Qt::AlignCenter ? "center"
//            : Settings::settings()->editorTextAlignment & Qt::AlignRight ? "right"
//            : Settings::settings()->editorTextAlignment & Qt::AlignJustify ? "justify"
//            : "")
//       .arg(Settings::settings()->editorTextDirection == Qt::LeftToRight ? "ltr" : "rtl")
//       .arg(Settings::settings()->editorFont.pointSize())
//       .arg(Settings::settings()->editorForegroundColor.name());
// }

// QString TextDocumentUtilities::insertTextInDefaultHtml(const QString &text)
// {
//    QTextDocument document;
//    document.setHtml(defaultHtmlCode());

//    QTextCursor cursor(&document);
//    cursor.insertText(text);
//    return document.toHtml();
// }

//======================================================================

Random* Random::_random = 0;

Random* Random::random()
{
   if (_random)
      return _random;
   else
      return (_random = new Random);
}

Random::Random()
{
   srand(time(0));
}

int Random::getARandomNumber(int min, int max)
{
   return rand()%(max-min+1)+min;
}

QString Random::getARandomName(int maxLength)
{
   QString characters ("abcdefghijklmnopqrstuvwxyz0123456789");
   QString name;
   for (int i=0; i<maxLength; i++)
      name.append(characters[getARandomNumber(0, characters.size()-1)]);
   return name;
}

//======================================================================

QString Paths::getKgeographyDataPath()
{
   QString kgeographyDataPath = QApplication::applicationDirPath() + QDir::separator() + "kgeography_data";
   if (!QDir(kgeographyDataPath).exists())
   {
#ifdef KGEOGRAPHY_DATA_PATH
      kgeographyDataPath = KGEOGRAPHY_DATA_PATH;
#else
      //NOTE:
      kgeographyDataPath = "";
#endif
   }

   return kgeographyDataPath;
}

QString Paths::getDatabasePath()
{
   QString databasePath;   
#ifdef Q_WS_WIN
   databasePath = QApplication::applicationDirPath() + QDir::separator() + "databases" + QDir::separator();
#else
   databasePath = QDir::home().path() + "/.flashqard/databases/";
#endif
   return databasePath;
}
