// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#ifndef UTILITIES_H
#define UTILITIES_H

#include <cstdlib>
#include <ctime>
#include <QString>
#include <QObject>
#include <QToolBar>
#include <QProcess>
#include <QToolButton>
class QWebPage;

class HtmlParser
{
   public:
      static HtmlParser *htmlParser();
      QString htmlToPlainText(const QString&);
      bool setTagAttribute(QString&, int&, const QString&, const QString&, const QString&);
      bool getTagAttribute(const QString&, int&, const QString&, const QString&, QString&);
      bool setTagStyleAttribute(QString&, int, const QString&, const QString&, const QString&);
      bool contains(const QString&, const QString&, bool) const;
      QString contentsOfTag(const QString&, int, const QString&) const;

      bool findNextTag(const QString&, int, QString&, int&, int&) const;
      bool findTag(const QString&, int, const QString&, int&, int&) const;

   private:
      static HtmlParser *_htmlParser;
      QWebPage *webPage;

      HtmlParser();
};

//======================================================================

class Reader : public QObject
{
   Q_OBJECT
   
   public:
      static Reader* reader();
      int readPlainText(const QString&);
      int readHtml(const QString&);
      bool isReading() const;

   signals:
      void readingStarted(int);
      void readingFinished(int);
      
   public slots:
      void stop();

   private slots:
      void processErrorSlot(QProcess::ProcessError);
      void finished();

   private:
      static Reader *_reader;
      QProcess *process;
      int textID;
      int read(const QString&);

      Reader(QObject* = 0);
      ~Reader();
      void didNotFindEspeak();
};

//======================================================================

// class TextDocumentUtilities
// {
//    public:
//       QString defaultHtmlCode();
//       QString insertTextInDefaultHtml(const QString&);
// };

//======================================================================

class Random
{
   public:
      static Random* random();
      //get random number x where min <= x <= max
      int getARandomNumber(int min, int max);
      //get a name which includes characters a-z and 0-9.
      QString getARandomName(int maxLength = 10);
   
   private:
      Random();
      static Random *_random;

};

//======================================================================

class Paths
{
   public:
      static QString getKgeographyDataPath();
      static QString getDatabasePath();
};

#endif

