/***************************************************************************
 *   Copyright (C) 2004-2007 by Albert Astals Cid                          *
 *   aacid@kde.org                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/***************************************************************************
 *   This file is from KGeography project and is modified by               *
 *   Shahab Shirazi to fit FlashQard                                       *
 *   Copyright (C) 2009 Shahab <shahab@flashqard-project.org>              *
 ***************************************************************************/

#ifndef MAPWIDGET_H
#define MAPWIDGET_H

#include <QGraphicsView>
#include <QTimer>
#include <QAbstractButton>


class QGraphicsRectItem;
class QGraphicsScene;
class ZoomButton;

class mapWidget : public QGraphicsView
{
Q_OBJECT
	public:
		explicit mapWidget(QWidget *parent);
		
		void init(const QString &path);
                void clear();

		void setMapMove(bool b);
		void setMapZoom(bool b);
		
		QSize mapSize() const;
		void highlightRegion(QRgb, QRgb, int = 0);
   
                virtual QSize sizeHint () const;
	
	public slots:
		void setOriginalImage();
		void setAutomaticZoom(bool b);
		void clearHighlight();
		void zoomIn();
		void zoomOut();
   
	signals:
		void setMoveActionChecked(bool b);
		void setZoomActionChecked(bool b);
		void setMoveActionEnabled(bool b);
		void clicked(QRgb, const QPoint&);
	
	protected:
		void mousePressEvent(QMouseEvent *e);
		void mouseMoveEvent(QMouseEvent *e);
		void mouseReleaseEvent(QMouseEvent *e);
		void wheelEvent(QWheelEvent *e);
		void resizeEvent(QResizeEvent *e);
		void drawBackground(QPainter *painter, const QRectF &rect);

	private:
		/**
		 * Updates the move and zoom toggle states from the current mode.
		 */
		void updateActions();
		void updateZoom();
		
		enum Mode { Zooming, WantZoom, Moving, WantMove, None };
		Mode p_mode, p_modeBeforeMidClick;
		QImage p_originalImage;
//		QPixmap p_originalPixmap;
		QImage p_displayImage;
		QPixmap p_displayPixmap;
		QGraphicsRectItem *p_zoomRect;
		QGraphicsScene *p_scene;
		QPointF p_initial; // for rubberbanding, in scene coords
		QPoint p_prev; // for moving, in view coords
		bool p_automaticZoom;
		QTimer p_timer;
		ZoomButton *p_zoomInButton;
		ZoomButton *p_zoomOutButton;
};

class ZoomButton : public QAbstractButton
{
   Q_OBJECT

   public:
      ZoomButton(QWidget* = 0);
      virtual QSize sizeHint() const;
   
   protected:
      virtual void paintEvent(QPaintEvent*);

   private:
      QSize iconSize;
};


#endif
