// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

// +-------------------------------------------------------------------------+
// | Some of the following codes are from MDic project.                      |
// | Thanks to:                                                              |
// | Mehrdad Momeny <mehrdad.momeny@gmail.com>                               |
// | Majid Ramezanpour <thinkgnu@gmail.com>                                  |
// +-------------------------------------------------------------------------+

#include "ConfigureTranslatorDialog.h"
#include "Settings.h"
#include "Utilities.h"
#include "DatabasePropertiesDialog.h"
#include "GooglePropertiesDialog.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QtSql>
#include <QCheckBox>

ConfigureTranslatorDialog::ConfigureTranslatorDialog(QWidget *parent)
   : QDialog(parent)
{
   setupUi(this);

   QStringList dbnames=Settings::settings() -> translatorDatabases.keys();

   for (int row=0;row<dbnames.size();row++) {
      QString dbName=dbnames.at(row) ;
      loadDB( dbName, Settings::settings() -> translatorDatabases.value(dbName).toBool() );
   }

   dbTable -> setColumnWidth(0, dbTable->sizeHint().width()*2/3);

   addToOnlineServicesTable(tr("Google Translate"), Settings::settings() -> translatorServiceGoogle);
   onlineServicesTable -> setColumnWidth(0, onlineServicesTable->sizeHint().width()*2/3);

   offlineDatabasesNoteLabel -> setText("<p>"
                                        + tr("FlashQard supports MDic Dictionary databases (m2, mdb). "
                                             "It is also possible to use <b>Babylon</b> glossaries (.bgl) , "
                                             "<b>Stardict</b> dictionaries (.ifo) , <b>Freedict</b> "
                                             "dictionaries (.tei) and <b>Sdictionary</b> dictionary "
                                             "(.dct) by converting them via "
                                             "<a href='http://mdic.sourceforge.net/download.html#mdicconv'>"
                                             "MDicConv</a> convertor or "
                                             "<a href='http://ospdev.net/projects/glossary-pywork'>"
                                             "PyGlossary</a> tool.<br>You can also send them to "
                                             "<a href='mailto:shahab@flashqard-project.org'>this email</a> "
                                             "and they will be converted soon.")
                                        + "</p>");
}

void ConfigureTranslatorDialog::on_addButton_clicked()
{
   QString fileFilter;
#ifdef Q_WS_WIN
   fileFilter = tr("MDic Dictionary database(*.m2 *.mdb);;All files(*)");
#else
   fileFilter = tr("MDic Dictionary database(*.m2 *.mdb *.m2.bz2 *.m2.gz *.mdb.bz2 *.mdb.gz);;All files(*)");
#endif
   
   QString filePath = QFileDialog::getOpenFileName(this,tr("Select Database"), ".", fileFilter);

   if (!filePath.isEmpty()) {
      QFileInfo infoFile(filePath);
      QFile dbfile(filePath);
//      directory = infoFile.dir().path();
      QString FileName=infoFile.fileName();
      QString newFilePath = Paths::getDatabasePath() +FileName;
      if (!QDir(Paths::getDatabasePath()).exists())
         QDir().mkpath(Paths::getDatabasePath());
      else if (QFile::exists(newFilePath))
         QFile::remove(newFilePath);

      if (!dbfile.copy(newFilePath)) {
         QMessageBox::warning(this, tr("Error!"), tr("Can not copy selected file to Databases file, folder!\n") + dbfile.errorString());
      } else {
         if (FileName.right(4)==".bz2") {
            QProcess::execute("bunzip2", QStringList(Paths::getDatabasePath() +FileName));
            FileName.remove(FileName.length()-4,4);
         }
         if (FileName.right(3)==".gz") {
            QProcess::execute("gunzip", QStringList(Paths::getDatabasePath() +FileName));
            FileName.remove(FileName.length()-3,3);
         }

         QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", FileName);
         db.setDatabaseName(Paths::getDatabasePath() +FileName);
         if (!db.open()) {
            QMessageBox::warning(this, tr("Cannot Add Database File"), tr("Cannot open database file, it seems it's not a typical MDic Database file...") + db.lastError().text());
            QFile::remove(Paths::getDatabasePath() +FileName);
         } else {
            QSqlQuery q(db);
            q.exec("select * from word limit 1");
            if (!q.next()) {
               QMessageBox::warning(this, tr("Cannot Add Database File"), tr("It seems, it's not a typical MDic Database file.\nCheck it and try again...") + db.lastError().text());
               QFile::remove(Paths::getDatabasePath() +FileName);
            }
            else
            {
               loadDB(FileName,true);
//               Settings::settings() -> translatorDatabases.insert(FileName, true);
            }
         }
      }
   }
}

void ConfigureTranslatorDialog::on_removeButton_clicked()
{
   int rowToDelete=dbTable->currentRow();
   if (rowToDelete >= 0)	{
      QString FileName;
      FileName=dbTable->item(rowToDelete,0)->text();
      dbTable->removeRow(rowToDelete);
      if (!FileName.isEmpty()) {
         QFileInfo infoFile(FileName);
         QFile dbfile(FileName);
         FileName=infoFile.fileName();
         if (!dbfile.remove(Paths::getDatabasePath()+FileName))
            QMessageBox::warning(this, tr("Error on Delete"),
                                 tr("Can not Delete Database file please delete it manually from: %1")
                                 .arg(Paths::getDatabasePath()));
      }

//      Settings::settings() -> translatorDatabases.remove(FileName);
   } else {
      QMessageBox::warning(this, tr("Select a Database"), tr("Please select a database to remove"));
   }
}

void ConfigureTranslatorDialog::on_propertiesButton_clicked()
{
   int rowToChange = dbTable->currentRow();
   if (rowToChange >= 0)
   {
      QString dbName = dbTable->item(rowToChange,0)->text();
      DatabasePropertiesDialog dialog(this);
      dialog.loadDatabase(dbName);
      dialog.exec();
   }
}

void ConfigureTranslatorDialog::on_onlineServicesPropertiesButton_clicked()
{
   if (onlineServicesTable -> selectedItems().value(0) == googleTableItem)
   {
      GooglePropertiesDialog dialog(Settings::settings() -> translatorServiceGoogleSourceLanguage,
                                    Settings::settings() -> translatorServiceGoogleTargetLanguage,
                                    this);
      dialog.exec();
      Settings::settings() -> translatorServiceGoogleSourceLanguage = dialog.getSourceLanguage();
      Settings::settings() -> translatorServiceGoogleTargetLanguage = dialog.getTargetLanguage();
   }
}

void ConfigureTranslatorDialog::loadDB(const QString &FileName, bool checkStatus)
{
   int rc=dbTable->rowCount();
   dbTable->setRowCount(rc+1);

   QTableWidgetItem *dbNameItem=new QTableWidgetItem(FileName,0);
   dbTable->setItem(rc,0,dbNameItem);

   dbTable->setCellWidget(rc,1,new QCheckBox(this));  //show checkbox

   if (checkStatus)
      qobject_cast<QCheckBox*>(dbTable->cellWidget(rc, 1))->setCheckState(Qt::Checked);
   else
      qobject_cast<QCheckBox*>(dbTable->cellWidget(rc, 1))->setCheckState(Qt::Unchecked);

   QSqlQuery q("select * from sqlite_master WHERE type='index'", QSqlDatabase::database(FileName));
   if (q.exec()) {
      if (q.next())
         if (q.value(1).toString()=="wnameidx") {
//NOTE:
//            dbTable->setItem(rc, 2, new QTableWidgetItem(QIcon((QString)REC_PATH+(QString)"yes.png"), "Yes", 0));
            return;
         }
   } else {
      qDebug("ConfigureTranslatorDialog::loadDB: database(%s) error: %s", FileName.toLatin1().data(), q.lastError().text().toLatin1().data());
//NOTE:
//      dbTable->setItem(rc, 2, new QTableWidgetItem(QIcon((QString)REC_PATH+(QString)"unknown.png"), "Unknown", 0));
      return;
   }
   
//NOTE:
//   dbTable->setItem(rc, 2, new QTableWidgetItem(QIcon((QString)REC_PATH+(QString)"no.png"), "No", 0));
}

void ConfigureTranslatorDialog::on_dialogButtonBox_clicked(QAbstractButton *button)
{
   if (dialogButtonBox -> standardButton(button) == QDialogButtonBox::Close)
   {
      Settings::settings() -> translatorDatabases.clear();
      for (int row=0;row<dbTable->rowCount();row++) {
         bool dbStatus= qobject_cast<QCheckBox*>(dbTable->cellWidget(row, 1))->checkState() == Qt::Checked;
         QString dbName=(dbTable->item(row,0))->text();
         Settings::settings() -> translatorDatabases[dbName] = dbStatus;
      }

      bool status = qobject_cast<QCheckBox*>(onlineServicesTable->cellWidget(
                                                googleTableItem->row(), 1))->checkState() == Qt::Checked;
      Settings::settings() -> translatorServiceGoogle = status;
      
      accept();
   }
}

void ConfigureTranslatorDialog::addToOnlineServicesTable(const QString &name, bool show)
{
   int rowCount=onlineServicesTable->rowCount();
   onlineServicesTable->setRowCount(rowCount+1);

   googleTableItem = new QTableWidgetItem(name,0);
   onlineServicesTable->setItem(rowCount, 0, googleTableItem);

   onlineServicesTable->setCellWidget(rowCount,1,new QCheckBox(this));  //show checkbox
   if (show)
      qobject_cast<QCheckBox*>(onlineServicesTable->cellWidget(rowCount, 1))->setCheckState(Qt::Checked);
   else
      qobject_cast<QCheckBox*>(onlineServicesTable->cellWidget(rowCount, 1))->setCheckState(Qt::Unchecked);
}

