// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

// +-------------------------------------------------------------------------+
// | Some of the following codes are from MDic project.                      |
// | Thanks to:                                                              |
// | Mehrdad Momeny <mehrdad.momeny@gmail.com>                               |
// | Majid Ramezanpour <thinkgnu@gmail.com>                                  |
// +-------------------------------------------------------------------------+

#include "DatabasePropertiesDialog.h"
#include "Utilities.h"
#include <QtSql>
#include <QMessageBox>


DatabasePropertiesDialog::DatabasePropertiesDialog(QWidget *parent)
   : QDialog(parent)
{
   setupUi(this);
}

void DatabasePropertiesDialog::loadDatabase(const QString &_dbName)
{
   dbName = _dbName;

   QSqlDatabase db = QSqlDatabase::database(dbName, true);
   db.setDatabaseName(Paths::getDatabasePath()+dbName);
   if (!db.open()) {
      QMessageBox::critical(this, tr("Cannot Open"), tr("Cannot Get Dictionary Database informations\n")+ db.lastError().text());
      return;
   }

   QSqlQuery query(db);
   query.setForwardOnly(true);

   query.exec("SELECT * FROM dbinfo limit 1");
   //SELECT dbname,category,author,version,direction,origLang,destLang,license,description

   int dbnameNo = query.record().indexOf("dbname");
   int categoryNo = query.record().indexOf("category");
   int authorNo = query.record().indexOf("author");
   int versionNo = query.record().indexOf("version");
   int directionNo = query.record().indexOf("direction");
   int origLangNo = query.record().indexOf("origLang");
   int destLangNo = query.record().indexOf("destLang");
   int licenseNo = query.record().indexOf("license");
   int descriptionNo = query.record().indexOf("description");

   if (query.next()) {
      txtDbName->setText(query.value(dbnameNo).toString());
      txtCat->setText(query.value(categoryNo).toString());
      lblAuthor->setText(query.value(authorNo).toString());
      lblVersion->setText(query.value(versionNo).toString());
      comboDir->setCurrentIndex(query.value(directionNo).toString()=="rtl"?1:0);
      lblSrc->setText(query.value(origLangNo).toString());
      lblDest->setText(query.value(destLangNo).toString());
      lblLicense->setText(query.value(licenseNo).toString());
      lblDesc->setText(query.value(descriptionNo).toString());
   } else {
      if (query.exec("insert into dbinfo(dbname) Values('"+dbName+"')"))
         txtDbName->setText(dbName);
      else
         QMessageBox::critical (this, tr("Cannot set database properties"), query.lastError().text());
   }
   query.exec("select count(*) from word");
   if (query.next())
      lblCount->setText(query.value(0).toString());
}

void DatabasePropertiesDialog::on_dialogButtonBox_accepted()
{
   QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", dbName+"ok");
   db.setDatabaseName(Paths::getDatabasePath()+dbName);
   if (!db.open()) {
      QMessageBox::critical(this, tr("Cannot Open"), tr("Cannot Open Dictionary Database to update informations\n") + db.lastError().text());
      return;
   }
   QSqlQuery query(db);
   query.setForwardOnly(true);

   QString qText="UPDATE dbinfo SET dbname='"+txtDbName->text()+"', category='"+txtCat->text()+"', direction='"+(comboDir->currentIndex()==0?"ltr":"rtl")+"'";

   if (!query.exec(qText))
      QMessageBox::warning(this, tr("Cannot Execute"), tr("Cannot Update Dictionary Database informations\n")+db.lastError().text());

   db.close();

   accept();
}

