// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "TranslatorServiceGoogle.h"
#include "Settings.h"
#include <QtWebKit>

TranslatorServiceGoogle::TranslatorServiceGoogle(QObject *parent)
   : TranslatorService(parent)
{
   searching = false;
   //create on demand   
   webPage = 0;
}

QString TranslatorServiceGoogle::getName() const
{
   return tr("Google Translate");
}

QString TranslatorServiceGoogle::getLastErrorMessage() const
{
   return errorMessage;
}

QList<TranslationRecord> TranslatorServiceGoogle::getLastTranslationRecords() const
{
   return translationRecords;
}

bool TranslatorServiceGoogle::isAvailable() const
{
   return Settings::settings() -> translatorServiceGoogle;
}

bool TranslatorServiceGoogle::isSearching() const
{
   return searching;
}

void TranslatorServiceGoogle::find(const QString &_searchPhrase)
{
   //create on demand
   if (!webPage)
   {
      webPage = new QWebPage(this);
      connect(webPage, SIGNAL(loadFinished(bool)), this, SLOT(loadFinishedSlot(bool)));
   }

   errorMessage.clear();
   translationRecords.clear();

   QUrl url = QUrl("http://translate.google.com/translate_t?text="
                   + _searchPhrase + "&sl="
                   + Settings::settings() -> translatorServiceGoogleSourceLanguage + "&tl="
                   + Settings::settings() -> translatorServiceGoogleTargetLanguage);

   qDebug("TranslatorServiceGoogle::find(): url=%s", qPrintable(url.toString()));
   webPage -> mainFrame() -> load(url);
   searching = true;
   searchPhrase = _searchPhrase;
}

void TranslatorServiceGoogle::loadFinishedSlot(bool ok)
{
   searching = false;
   if (ok)
   {
      TranslationRecord record;
      record.translatorName = getName();
      record.searchPhrase = searchPhrase;
      record.translation = getResultBoxFromWebPage();

      QString textDirection = getResultBoxDirectionFromWebPage();
      if (!textDirection.isEmpty())
      {
         record.translation.prepend("<div dir='" + textDirection + "'>");
         record.translation.append("</div>");
      }
      translationRecords << record;

      record.translation = getDictionaryFromWebPage();
      if (!record.translation.isEmpty())
      {
         record.translatorName = tr("Google Dictionary");
         record.searchPhrase = searchPhrase;
         translationRecords << record;
      }

      if (translationRecords.size() == 0)
         errorMessage = tr("Sorry, phrase not found!");
   }
   else
      errorMessage = tr("Could not access the server!");
   
   emit searchFinishedSignal(this);
}

QString TranslatorServiceGoogle::getResultBoxFromWebPage()
{
   return webPage -> mainFrame() -> evaluateJavaScript(
      "function getResultBox() "
      "{ "
      "var node = document.getElementById(\"result_box\"); "
      "return node.innerHTML; "
      "} "
      "getResultBox(); ").toString();
}

QString TranslatorServiceGoogle::getResultBoxDirectionFromWebPage()
{
   return webPage -> mainFrame() -> evaluateJavaScript(
      "function getDirection() "
      "{ "
      "var node = document.getElementById(\"result_box\"); "
      "return node.getAttribute(\"dir\"); "
      "} "
      "getDirection(); ").toString();
}

QString TranslatorServiceGoogle::getDictionaryFromWebPage()
{
   return webPage -> mainFrame() -> evaluateJavaScript(
      "function getDictionary() "
      "{ "
      "var node = document.getElementById(\"dict\"); "
      "var child = node.firstChild; "
      "while (child) "
      "{ "
      "if (child.className == \"thead\") "
      "{ "
      "child.parentNode.removeChild(child); "
      "break; "
      "} "
      "child = child.nextSibling; "
      "} "
      "return node.innerHTML; "
      "} "
      "getDictionary(); ").toString();
}
