// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

// +-------------------------------------------------------------------------+
// | Some of the following codes are from MDic project.                      |
// | Thanks to:                                                              |
// | Mehrdad Momeny <mehrdad.momeny@gmail.com>                               |
// | Majid Ramezanpour <thinkgnu@gmail.com>                                  |
// +-------------------------------------------------------------------------+

#include "TranslatorServiceOffline.h"
#include "dbman.h"
#include "Utilities.h"
#include "Settings.h"
#include <QtSql>

TranslatorServiceOffline::TranslatorServiceOffline(QObject *parent)
   : TranslatorService(parent)
{
   initialize();
}

QString TranslatorServiceOffline::getName() const
{
   return tr("Local databases");
}

QString TranslatorServiceOffline::getLastErrorMessage() const
{
   return errorMessage;
}

QList<TranslationRecord> TranslatorServiceOffline::getLastTranslationRecords() const
{
   return translationRecords;
}

bool TranslatorServiceOffline::isAvailable() const
{
   return Settings::settings() -> translatorDatabases.keys(true).size();
}

bool TranslatorServiceOffline::isSearching() const
{
   return false;
}

void TranslatorServiceOffline::find(const QString &searchPhrase)
{
   translationRecords.clear();
   errorMessage.clear();
   
   DbMan *db=new DbMan();
   int res = db->getMean(searchPhrase);
   qDebug("TranslatorServiceOffline::find(): res=%d", res);
   if (res > 0)
   {
      for (int i=0; i<db->meanLst.size(); i++)
      {
         TranslationRecord record;
         record.searchPhrase = db->meanLst[i].word;
         record.translation = db->meanLst[i].htmlMean;
         record.translatorName = db->meanLst[i].dbname;
         translationRecords << record;
      }
   }
   else if (res == 0)
      errorMessage = tr("Sorry, phrase not found!");
   else
      errorMessage = tr("No database found!<br>"
                        "Databases can be added in <a href=\"configure_translator\">"
                        "\"Configure Translator\"</a> dialog");

   emit searchFinishedSignal(this);
}

void TranslatorServiceOffline::initialize()
{
   QStringList allDBNames = Settings::settings() -> translatorDatabases.keys();

   for (int row=0; row < allDBNames.count(); row++) {
      QString dbName=allDBNames.at(row);
      if (QSqlDatabase::contains(dbName))
         continue;
      QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", dbName);
      db.setDatabaseName(Paths::getDatabasePath()+dbName);
      if (!db.open()) {
         qDebug("TranslatorServiceOffline::TranslatorServiceOffline(): could not open db!");
      }
   }
}   


