// +-------------------------------------------------------------------------+
// | FlashQard - An efficient way to memorize.                               |
// | Copyright (C) 2008-2009 Shahab <shahab@flashqard-project.org>           |
// +-------------------------------------------------------------------------+
// | License:                                                                |
// |                                                                         |
// | FlashQard is free software; you can redistribute it and/or modify       |
// | it under the terms of the GNU General Public License as published by    |
// | the Free Software Foundation; either version 2 of the License, or       |
// | (at your option) any later version.                                     |
// |                                                                         |
// | FlashQard is distributed in the hope that it will be useful,            |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            |
// | GNU General Public License for more details.                            |
// |                                                                         |
// | You should have received a copy of the GNU General Public License       |
// | along with FlashQard; if not, write to the Free Software                |
// | Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA|
// |                                                                         |
// +-------------------------------------------------------------------------+
// | Author:  Shahab Shirazi <shahab@flashqard-project.org>                  |
// +-------------------------------------------------------------------------+

#include "TranslatorWidget.h"
#include "TranslatorServiceOffline.h"
#include "TranslatorServiceGoogle.h"
#include "ConfigureTranslatorDialog.h"
#include "Utilities.h"
#include "Settings.h"
#include "MainWindow.h"
#include <QWebView>
#include <QWebFrame>
#include <QLineEdit>
#include <QToolButton>
#include <QApplication>
#include <QHBoxLayout>
#include <QVBoxLayout>

TranslatorWidget::TranslatorWidget(QWidget *parent)
   : QWidget(parent)
{
   configureDialog = 0;
   showCreateCardOption = false;
   createServices();
   layout();
   tellUserTypeSomething();
}

QSize TranslatorWidget::sizeHint() const
{
   //NOTE:
   //try to find a way to set the width of
   //Card list and Translator to a same number
   return QSize(391, 600);
}

void TranslatorWidget::setShowCreateCardOption(bool show)
{
   showCreateCardOption = show;
   updateTranslator();
}

void TranslatorWidget::updateTranslator()
{
   searchSlot();
}

void TranslatorWidget::createServices()
{
   TranslatorService *translatorServiceOffline = new TranslatorServiceOffline(this);
   connect(translatorServiceOffline, SIGNAL(searchFinishedSignal(TranslatorService*)),
           this, SLOT(searchFinishedSlot(TranslatorService*)));
   services << translatorServiceOffline;

   TranslatorService *translatorServiceGoogle = new TranslatorServiceGoogle(this);
   connect(translatorServiceGoogle, SIGNAL(searchFinishedSignal(TranslatorService*)),
           this, SLOT(searchFinishedSlot(TranslatorService*)));
   services << translatorServiceGoogle;
}

void TranslatorWidget::layout()
{
   searchLineEdit = new QLineEdit(this);
   connect(searchLineEdit, SIGNAL(returnPressed()), this, SLOT(searchSlot()));

   searchButton = new QToolButton(this);
   searchButton -> setAutoRaise(true);
   if (QApplication::layoutDirection() == Qt::LeftToRight)
      searchButton -> setIcon(QIcon(":/images/go.png"));
   else
      searchButton -> setIcon(QIcon(":/images/goRTL.png"));
   searchButton -> setText(tr("Go"));
   searchButton -> setToolTip(searchButton->text());
   connect (searchButton, SIGNAL(clicked()), this, SLOT(searchSlot()));

   configureButton = new QToolButton(this);
   configureButton -> setAutoRaise(true);
   configureButton -> setText(tr("Configure Translator..."));
   configureButton -> setIcon(QIcon(":/images/configureTranslator.png"));
   configureButton -> setToolTip(configureButton->text());
   connect (configureButton, SIGNAL(clicked()), this, SLOT(configureSlot()));

   readButton = new QToolButton(this);
   readButton -> setAutoRaise(true);
   readButton -> setText(tr("Read"));
   if (QApplication::layoutDirection() == Qt::RightToLeft)
      readButton->setIcon(QIcon(":/images/playRTL.png"));
   else
      readButton -> setIcon(QIcon(":/images/play.png"));
   connect(readButton, SIGNAL(clicked()), this, SLOT(readSlot()));
   readButton -> setToolTip(readButton->text());
   connect(Reader::reader(), SIGNAL(readingFinished(int)), this, SLOT(setReadButtonIcon()));
   connect(Reader::reader(), SIGNAL(readingStarted(int)), this, SLOT(setReadButtonIcon()));
   
   webView = new QWebView(this);
   webView -> setHtml("<p></p>");
   webView -> page() -> setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
   connect(webView, SIGNAL(linkClicked(const QUrl&)), this, SLOT(linkClickedSlot(const QUrl&)));

   QHBoxLayout *horizontalLayout = new QHBoxLayout;
   horizontalLayout -> addWidget(searchLineEdit);
   horizontalLayout -> addWidget(searchButton);
   horizontalLayout -> addWidget(readButton);
   horizontalLayout -> addWidget(configureButton);

   QVBoxLayout *verticalLayout = new QVBoxLayout(this);
   verticalLayout -> addLayout(horizontalLayout);
   verticalLayout -> addWidget(webView);
}

void TranslatorWidget::searchSlot()
{
   qDebug("TranslatorWidget::searchSlot()");
   
   QString searchPhrase = searchLineEdit -> text();
   if (searchPhrase.isEmpty())
      tellUserTypeSomething();
   else
   {
      for (int i=0; i<services.size(); i++)
         if (services[i] -> isAvailable())
            services[i] -> find(searchPhrase);
      updateBoard();
      if (showCreateCardOption)
         MainWindow::mainWindow() -> showMessageOnStatusBar(tr("You can create a vocabulary card "
                                                               "by pressing \"create a card\" above "
                                                               "each translation."));
   }
}

void TranslatorWidget::searchFinishedSlot(TranslatorService *service)
{
   qDebug("TranslatorWidget::searchFinishedSlot()");

   updateBoard();
}

void TranslatorWidget::configureSlot()
{
   if (!configureDialog)
      configureDialog = new ConfigureTranslatorDialog(this);

   if (configureDialog -> exec())
      searchSlot();
}

void TranslatorWidget::readSlot()
{
   if (Reader::reader() -> isReading())
      Reader::reader() -> stop();
   else
      Reader::reader() -> readPlainText(searchLineEdit -> text());
}

void TranslatorWidget::setReadButtonIcon()
{
   if (Reader::reader() -> isReading())
      readButton->setIcon(QIcon(":/images/stop.png"));
   else
   {
      if (QApplication::layoutDirection() == Qt::RightToLeft)
         readButton->setIcon(QIcon(":/images/playRTL.png"));
      else
         readButton -> setIcon(QIcon(":/images/play.png"));
   }
}

void TranslatorWidget::tellUserTypeSomething()
{
   webView -> setHtml("<h2 align='center'><br><br><br><br>"
                      + tr("Please enter a phrase above to translate")
                      + "</h2>");
}

void TranslatorWidget::linkClickedSlot(const QUrl &url)
{
   if (url.toString() == "configure_translator")
      configureSlot();
   else if (url.toString().contains(QRegExp("^add_card:\\d+$")))
   {
      int index = url.toString().remove("add_card:").toInt();
      emit createVocabularyCardSignal(translationRecords[index].searchPhrase,
                                      translationRecords[index].translation);
   }
   else
      qDebug("TranslatorWidget::linkClickedSlot(): Unknown link clicked: %s", qPrintable(url.toString()));
}

void TranslatorWidget::updateBoard()
{
   translationRecords.clear();
   QString html;
   for (int i=0; i<services.size(); i++)
   {
      QString errorMessage = services[i] -> getLastErrorMessage();
      if (!errorMessage.isEmpty())
      {
         html += "<p style=\"text-align:center; background-color:#fff0f0\">" + services[i] -> getName() + "</p>"
            + "<p><font color='red'><i>" + errorMessage + "</i></font></p>";
      }
      else if (services[i] -> isSearching())
      {
         html += "<p style=\"text-align:center; background-color:#f0f0f0\">" + services[i] -> getName() + "</p>"
            + "<p><i>" + tr("Searching...") + "</i></p>";
      }
      else if (services[i] -> isAvailable())
      {
         QList<TranslationRecord> translations = services[i] -> getLastTranslationRecords();
         for (int j=0; j<translations.size(); j++)
         {
            QString id = QString::number(translationRecords.size());
            QString createCardHtml;
            if (showCreateCardOption)
               createCardHtml = "<br><font size=\"-1\"><u><a href='add_card:" + id + "'>"
                  + tr("(create a card)") + "</a></u></font>";
            
            html += "<p style=\"text-align:center; background-color:#f0f0ff\">"
               + translations[j].translatorName
               + createCardHtml + "</p>"
               + "<p><b>" + translations[j].searchPhrase + ":</b><br>"
               + translations[j].translation
               + "</p>";
            translationRecords << translations[j];
         }
      }
   }

   int disabledServices = 0;
   for(int i=0; i<services.size(); i++)
      if (!services[i] -> isAvailable())
         disabledServices++;
   if (disabledServices == services.size())
   {
      html = "<h2 align='center'><font color='red'><br><br><br><br>"
         + tr("No local or online service found!")
         + "</font></h2><p align='center'>"
         + tr("Local or online services can be added in <a href='configure_translator'>"
         "\"Configure Translator\"</a> dialog")
         + "</p>";
   }      
   
   webView -> setHtml(html);
}
